package jsetl.test;

import java.io.*;

import jsetl.exception.Failure;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import junit.framework.TestCase;
import jsetl.*;

//Author: Michele Giacobini

public class ANDTest extends TestCase{

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
	@Test
	public void testEqAndEqCheckFalse() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar x = new IntLVar("x");
		solver.add(x.eq(1).and(x.eq(2)));
		//solver.showStoreAll();
		// Create a stream to hold the output
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
	    // Save the old System.out
	    PrintStream old = System.out;
	    // Set my stream
	    System.setOut(constraintStore);
	    // Show store on constraintStore output
	    solver.showStore();
	    assertTrue("Store: _x = 1 AND _x = 2".equals(newStream.toString().trim()));
	    assertEquals(false, solver.check());
	    // Return to old Stream
	    System.out.flush();
	    System.setOut(old);
	    //System.out.println("Check(): " + solver.check());
	    //solver.showStoreAll();
	    // Show what happened
	    /*System.out.println(actualValue);
	    System.out.println(expectValue);
	    if (actualValue.equals(expectValue))
	    	System.out.println("Uguali");
	    else
	    	System.out.println("Diverse");
		*/
	    //solver.solve(); // Fallisce => non esegue il test
	    //solver.showStore();
	}
	
	@Test
	public void testEqAndEqCheckTrue() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar xx = new IntLVar("xx");
		solver.add(xx.eq(1).and(xx.eq(1)));
		//solver.showStore();
		// Create a stream to hold the output
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
	    // Save the old System.out
	    PrintStream old = System.out;
	    // Set my stream
	    System.setOut(constraintStore);
	    // Show store on constraintStore output
	    solver.showStore();
	    assertTrue("Store: _xx = 1 AND _xx = 1".equals(newStream.toString().trim()));
	    assertEquals(true, solver.check());
	    // Return to old Stream
	    System.out.flush();
	    System.setOut(old);
	    //System.out.println("Check(): " + solver.check());
	    //solver.showStoreAll();
	}
	    
	@Test
	public void testNeqAndNeqCheckTrue() throws Failure, IOException {
		Solver solver = new Solver();
		IntLVar y = new IntLVar("y");
		solver.add(y.neq(1).and(y.neq(2).and(y.eq(3))));
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _y neq 1 AND _y neq 2 AND _y = 3".equals(newStream.toString().trim()));
		assertEquals(true, solver.check());
		System.out.flush();
		System.setOut(old);
		}
	
	@Test
	public void testNeqAndNeqCheckFalse() throws Failure, IOException {
		Solver solver = new Solver();
		IntLVar yy = new IntLVar("yy");
		solver.add(yy.neq(1).and(yy.neq(2).and(yy.eq(2))));
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _yy neq 1 AND _yy neq 2 AND _yy = 2".equals(newStream.toString().trim()));
		assertEquals(false, solver.check());
		System.out.flush();
		System.setOut(old);
		}
	
	@Test
	public void testGtAndLtAndNeqCheckTrue() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar z = new IntLVar("z");
		solver.add(z.gt(1).and(z.lt(4)).and(z.neq(2)));
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _z > 1 AND _z < 4 AND _z neq 2".equals(newStream.toString().trim()));
		assertEquals(true, solver.check());
		System.out.flush();
		System.setOut(old);
		//System.out.println("Check: " + solver.check());
	    //solver.solve();
	    //solver.showStoreAll();
	}
	
	@Test
	public void testGtAndLtAndNeqCheckFalse() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar zz = new IntLVar("zz");
		solver.add(zz.gt(1).and(zz.lt(3)).and(zz.neq(2)));
//		solver.showStore();
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _zz > 1 AND _zz < 3 AND _zz neq 2".equals(newStream.toString().trim()));
		assertEquals(false, solver.check());
		System.out.flush();
		System.setOut(old);
		//System.out.println("Check: " + solver.check());
	    //solver.solve();
	    //solver.showStoreAll();
	}
	
	@Test
	public void testGtAndLtAndNeqAndEqCheckTrue() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar w = new IntLVar("w");
		solver.add((w.gt(1).and(w.lt(10))).and((w.neq(2).and(w.eq(5)))));
		//solver.showStore();
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _w > 1 AND _w < 10 AND _w neq 2 AND _w = 5".equals(newStream.toString().trim()));
		assertEquals(true, solver.check());
		System.out.flush();
		System.setOut(old);
	}
	
	@Test
	public void testGtAndLtAndNeqAndEqCheckFalse() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar ww = new IntLVar("ww");
		solver.add((ww.gt(1).and(ww.lt(4))).and((ww.neq(2).and(ww.eq(4)))));
		//solver.showStore();
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _ww > 1 AND _ww < 4 AND _ww neq 2 AND _ww = 4".equals(newStream.toString().trim()));
		assertEquals(false, solver.check());
		System.out.flush();
		System.setOut(old);
	}
}