package jsetl.test;

//import static org.junit.Assert.*;

//import java.io.IOException;

//import java.util.ArrayList;
//import java.util.EmptyStackException;
//import java.util.HashSet;
//import java.util.List;
//import java.util.Set;

import jsetl.exception.Failure;
import jsetl.exception.NotDefConstraintException;
import junit.framework.TestCase;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jsetl.*;


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//     Testing Arithmetic Constraints - class IntLVar     %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


	public class ConstraintArithmTest extends TestCase {

		@Before
		public void setUp() {
			LObject.USEJOLLYTOKEN = true;
		}

		@After
		public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
//%%%%%%%%%%%%%%%%%%%%%%% eq %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// IntLVar NON INIZIALIZZATA ---> INTERO
// x = 1
	@Test
	public void testEQIntLVarUninitializedx_Integer() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;        
		IntLVar x = new IntLVar("x");
		solver.solve(x.eq(1));
		c = solver.getConstraint();     
		assertEquals("[]",c.toString());
	    assertEquals("1",x.toString());
	}

// IntLVar NON INIZIALIZZATA x ---> IntLVar NON INIZIALIZZATA x
// x = x
	@Test
	public void testEQIntLVarUninitializedx_IntLVarUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;        
		IntLVar x = new IntLVar("x");
		solver.solve(x.eq(x));
		c = solver.getConstraint();    
		assertEquals("[]",c.toString());
	}

// IntLVar NON INIZIALIZZATA x ---> IntLVar NON INIZIALIZZATA y
// x = y
	@Test
	public void testEQIntLVarUninitializedx_IntLVarUninitializedy() 
	throws NotDefConstraintException, Failure {
	 	Solver solver = new Solver();
	 	Constraint c;        
	 	IntLVar x = new IntLVar("x");
	 	IntLVar y = new IntLVar();
	 	solver.solve(x.eq(y));
	 	c = solver.getConstraint();	     
	 	assertEquals("[]",c.toString());	
	    assertEquals("_x",x.toString());
	    assertEquals("_x",y.toString());
	}

// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTERO
// y = 2, x = y
	@Test
	public void testEQIntLVarUninitializedx_IntLVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x = new IntLVar("x");
		IntLVar y = new IntLVar(2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	    assertEquals("2",x.toString());
	}

// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTERVALLO
// y in 2..6, x = y
	@Test
	public void testEQIntLVarUninitializedx_IntLVarInitializedWithIntervall()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x = new IntLVar("x");
		IntLVar y = new IntLVar(2,6);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	    assertEquals("[2..6]",x.getDomain().toString());
	    assertEquals("_x",x.toString());
	}

// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
// y1 = 3, y = y1, x = y
	@Test
	public void testEQIntLVarUninitializedx_IntLVarInitializedIntLVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x = new IntLVar("x");
		IntLVar y1 = new IntLVar(3);
		IntLVar y = new IntLVar(y1);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	    assertEquals("3",x.toString());
	    assertEquals("{3}",x.getDomain().toString());
	    assertEquals("[]",x.getConstraint().toString());
	}
	
// IntLVar INIZIALIZZATA CON INTERO ---> INTERO
// x = 8, x = 1
	@Test
	public void testEQIntLVarinitializedWithInteger_Integer() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar(8);
		try{		    	
	    	solver.solve(x.eq(1));
			fail("FALLIMENTO!!!!");
		}
	    catch(Failure e) {}
	}

// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar NON INIZIALIZZATA x
// x = 8, x = y
	@Test
	public void testEQIntLVarinitializedWithInteger_IntLVarUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x = new IntLVar(8);
		IntLVar y = new IntLVar();
		solver.solve(x.eq(y));
		c = solver.getConstraint();	     
		assertEquals("[]",c.toString());	
	    assertEquals("8",y.toString());
	}

// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar INIZIALIZZATA CON INTERO
// x = 8, y = 2, x = y
	@Test
	public void testEQIntLVarinitializedWithInteger_IntLVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar(8);
		IntLVar y = new IntLVar(2);
		try{	    	
			solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		}
		catch(Failure e) {}
	}

// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar INIZIALIZZATA CON INTERVALLO
// x = 8, y in 2..6, x = y
	@Test
	public void testEQIntLVarinitializedWithInteger_IntLVarInitializedWithIntervall()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar(8);
		IntLVar y = new IntLVar(2,6);
		try{	    	
			solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		}
		catch(Failure e) {}
	}

// IntLVar INIZIALIZZATA CON INTERO---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
// x in 8..12, y1 = 3, y = y1, x = y
	@Test
	public void testEQIntLVarinitializedWithInteger_IntLVarInitializedIntLVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar(8,12);
		IntLVar y1 = new IntLVar(3);
		IntLVar y = new IntLVar(y1);
		try{	    	
			solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		}
		catch(Failure e) {}
	}

// IntLVar INIZIALIZZATA CON INTERVALLO ---> INTERO
// x in 8..12, x = 1
	@Test
	public void testEQIntLVarinitializedWithIntervall_Integer() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar(8,12);
		try{		    	
	    	solver.solve(x.eq(1));
			fail("FALLIMENTO!!!!");
		}
	    catch(Failure e) {}	
	}

// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar NON INIZIALIZZATA x
// x in 8..12, x = y
	@Test
	public void testEQIntLVarinitializedWithIntervall_IntLVarUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x = new IntLVar(8,12);
		IntLVar y = new IntLVar();
		solver.solve(x.eq(y));
		c = solver.getConstraint();    
		assertEquals("[]",c.toString());
	    assertEquals("_?",x.toString());
	    assertEquals("[8..12]",x.getDomain().toString());
	    assertEquals("[8..12]",y.getDomain().toString());
	    assertEquals("[]",x.getConstraint().toString());
	}

// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERO
// x in 8..12, y = 2, x = y
	@Test
	public void testEQIntLVarinitializedWithIntervall_IntLVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar(8,12);
		IntLVar y = new IntLVar(2);
		try{	    	
			solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		}
		catch(Failure e) {}
	}

// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERVALLO
// x in 8..12, y in 2..6, x = y
	@Test
	public void testEQIntLVarinitializedWithIntervall_IntLVarInitializedWithIntervall()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar(8,12);
		IntLVar y = new IntLVar(2,6);
		try{	    	
			solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		}
		catch(Failure e) {}
	}

// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERVALLO
// x in 8..12, y in 10..20, x = y
	@Test
	public void testEQIntLVarinitializedWithIntervall_IntLVarInitializedWithIntervall_OK()  
		throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x = new IntLVar(8,12);
		IntLVar y = new IntLVar(10,20);
		solver.solve(x.eq(y));
		c = solver.getConstraint();    
		assertEquals("[]",c.toString());
	    assertEquals("[10..12]",x.getDomain().toString());
	    assertEquals("[10..12]",y.getDomain().toString());
	    assertEquals("[]",x.getConstraint().toString());
	}

//IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERVALLO
//x in 8..12, y in 1..20, x = y
	@Test
	public void testEQIntLVarinitializedWithIntervall_IntLVarInitializedWithIntervall_OK2()  
		throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x = new IntLVar(8,12);
		IntLVar y = new IntLVar(1,20);
		solver.solve(x.eq(y));
		c = solver.getConstraint();    
		assertEquals("[]",c.toString());
	    assertEquals("[8..12]",x.getDomain().toString());
	    assertEquals("[8..12]",y.getDomain().toString());
	}

// IntLVar INIZIALIZZATA CON INTERVALLO---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
//x in 8..12, y1 = 3, y = y1, x = y
	@Test
	public void testEQIntLVarInitWithIntervall_IntLVarInitIntLVarInitWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x = new IntLVar(8,12);
		IntLVar y1 = new IntLVar(3);
		IntLVar y = new IntLVar(y1);
		try{	    	
			solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		}
		catch(Failure e) {}
	}

// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> INTERO
// x1 = 3, x = x1, x = 1
	@Test
	public void testEQIntLVarInitializedIntLVarInitializedWithInteger_Integer()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x1 = new IntLVar(3);
		IntLVar x = new IntLVar(x1);
		try{	    	
			solver.solve(x.eq(1));
			fail("FALLIMENTO!!!!");
		}
		catch(Failure e) {}
	}
	
// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar NON INIZIALIZZATA 
// x1 = 3, x = x1, x = y
	@Test
	public void testEQIntLVarInitializedIntLVarInitializedWithInteger_IntLVarUninitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x1 = new IntLVar(3);
		IntLVar x = new IntLVar(x1);
		IntLVar y = new IntLVar();
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	    assertEquals("{3}",x.getDomain().toString());
	}
	
// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar INIZIALIZZATA CON INTERO
// x1 = 3, x = x1, y = 2, x = y
	@Test
	public void testEQIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedWithIntEGER()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		IntLVar x1 = new IntLVar(3);
		IntLVar x = new IntLVar(x1);
		IntLVar y = new IntLVar(2);
		try{	    	
			solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		}
		catch(Failure e) {}
	}

// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro ---> IntLVar INIZIALIZZATA CON INTERVALLO
// x1 = 3, x = x1, y = 2, x = y
	@Test
	public void testEQIntLVarInitIntLVarInitWithInteger_IntLVarInitWithIntervall()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x1 = new IntLVar(3);
		IntLVar x = new IntLVar(x1);
		IntLVar y = new IntLVar(2,6);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testEQIntLVarInitIntLVarInitWithInteger_IntLVarInitIntLVarInitWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		IntLVar x1 = new IntLVar(3);
		IntLVar x = new IntLVar(x1);		
		IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}


//%%%%%%%%%%%%%%%%%%%%%%% neq %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// IntLVar NON INIZIALIZZATA ---> INTERO
	@Test
	public void testNEQIntLVarUninitializedx_Integer() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	        
	     IntLVar x = new IntLVar("x1");
	     solver.solve(x.neq(1));
	     c = solver.getConstraint();
	     
	   assertEquals("[]",c.toString());
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar NON INIZIALIZZATA x
	@Test
	public void testNEQIntLVarUninitializedx_IntLVarUninitializedx() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     IntLVar x = new IntLVar("x1");
	     try{		    	
		    	solver.solve(x.neq(x));
				fail("FALLIMENTO!!!!");
			 }
		    catch(Failure e) {}
	
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar NON INIZIALIZZATA y
	@Test
	public void testNEQIntLVarUninitializedx_IntLVarUninitializedy() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	        
	     IntLVar x = new IntLVar("x");
	     IntLVar y = new IntLVar("y");
	     solver.solve(x.neq(y));
	     c = solver.getConstraint();
	     
	     assertEquals("_x neq _y",c.toString());
	
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testNEQIntLVarUninitializedx_IntLVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x1");
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testNEQIntLVarUninitializedx_IntLVarInitializedWithIntervall()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x");
	    IntLVar y = new IntLVar("y",2,6);
	    solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("_x neq _y",c.toString());
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testNEQIntLVarUninitializedx_IntLVarInitializedIntLVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x1");
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTERO ---> INTERO
	@Test
	public void testNEQIntLVarinitializedWithInteger_Integer() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	        
	     IntLVar x = new IntLVar(8);
	     solver.solve(x.neq(1));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar NON INIZIALIZZATA x
	@Test
	public void testNEQIntLVarinitializedWithInteger_IntLVarUninitializedx() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     IntLVar x = new IntLVar(8);
	     IntLVar y = new IntLVar();
	     solver.solve(x.neq(y));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testNEQIntLVarinitializedWithInteger_IntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8);
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.neq(y));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testNEQIntLVarinitializedWithInteger_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8);
	    IntLVar y = new IntLVar(2,6);
	    solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERO---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testNEQIntLVarinitializedWithInteger_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> INTERO
	@Test
	public void testNEQIntLVarinitializedWithIntervall_Integer() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;	        
	     IntLVar x = new IntLVar(8,12);
	     solver.solve(x.neq(1));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	
	}
	// IntLVar INIZIALIZZATA CON INTERVALLO---> IntLVar NON INIZIALIZZATA x
	@Test
	public void testNEQIntLVarinitializedWithIntervall_IntLVarUninitializedx() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;	        
	     IntLVar x = new IntLVar("x",8,12);
	     IntLVar y = new IntLVar("y");
	     solver.solve(x.neq(y));
	     c = solver.getConstraint();
	     assertEquals("_x neq _y",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testNEQIntLVarinitializedWithIntervall_IntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.neq(y));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testNEQIntLVarinitializedWithIntervall_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x",8,12);
	    IntLVar y = new IntLVar("y",2,6);
	    solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("_x neq _y",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testNEQIntLVarinitializedWithIntervall_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> INTERO
	@Test
	public void testNEQIntLVarInitializedIntLVarInitializedWithInteger_Integer()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    solver.solve(x.neq(1));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar NON INIZIALIZZATA 
	@Test
	public void testNEQIntLVarInitializedIntLVarInitializedWithInteger_IntLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar();
	    solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testNEQIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedWithIntEGER()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testNEQIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar(2,6);
	    solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testNEQIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		try{
	    	solver.solve(x.neq(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
//%%%%%%%%%%%%%%%%%%%%%%% lt %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	// IntLVar NON INIZIALIZZATA ---> INTERO
	@Test
	public void testLTIntLVarUninitializedx_Integer() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	        
	     IntLVar x = new IntLVar("x1");
	     solver.solve(x.lt(1));
	     c = solver.getConstraint();
	     
	   assertEquals("[]",c.toString());
	}

	// IntLVar NON INIZIALIZZATA x ---> IntLVar NON INIZIALIZZATA y
	@Test
	public void testLTIntLVarUninitializedx_IntLVarUninitializedy() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	        
	     IntLVar x = new IntLVar("x");
	     IntLVar y = new IntLVar("y");
	     solver.solve(x.lt(y));
	     c = solver.getConstraint();
	     assertEquals("_x < _y",c.toString());
	
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testLTIntLVarUninitializedx_IntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x1");
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.lt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testLTIntLVarUninitializedx_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x");
	    IntLVar y = new IntLVar("y",2,6);
	    solver.solve(x.lt(y));
	    c = solver.getConstraint();
	    assertEquals("_x < _y",c.toString());
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testLTIntLVarUninitializedx_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x1");
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		solver.solve(x.lt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTERO ---> INTERO
	@Test
	public void testLTIntLVarinitializedWithInteger_Integer() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     IntLVar x = new IntLVar(8);
	     try{		    	
		    	solver.solve(x.lt(1));
				fail("FALLIMENTO!!!!");
			 }
		    catch(Failure e) {}	
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar NON INIZIALIZZATA x
	@Test
	public void testLTIntLVarinitializedWithInteger_IntLVarUninitializedx() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     IntLVar x = new IntLVar(8);
	     IntLVar y = new IntLVar(8);
	     try{
		    	solver.solve(x.lt(y));
				fail("FALLIMENTO!!!!");
			 }
		    catch(Failure e) {}	
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testLTIntLVarinitializedWithInteger_IntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x = new IntLVar(8);
	    IntLVar y = new IntLVar(2);
	    try{
	    	solver.solve(x.lt(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testLTIntLVarinitializedWithInteger_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x = new IntLVar(8);
	    IntLVar y = new IntLVar(2,6);
	    try{	    	
	    	solver.solve(x.lt(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	// IntLVar INIZIALIZZATA CON INTERO---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testLTIntLVarinitializedWithInteger_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		try{	    	
	    	solver.solve(x.lt(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> INTERO
	@Test
	public void testLTIntLVarinitializedWithIntervall_Integer() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     IntLVar x = new IntLVar(8,12);
	     try{		    	
		    	solver.solve(x.lt(1));
				fail("FALLIMENTO!!!!");
			 }
		    catch(Failure e) {}	
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO---> IntLVar NON INIZIALIZZATA x
	@Test
	public void testLTIntLVarinitializedWithIntervall_IntLVarUninitializedx() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;	        
	     IntLVar x = new IntLVar("x",8,12);
	     IntLVar y = new IntLVar("y");
	     solver.solve(x.lt(y));
		 c = solver.getConstraint();
		 assertEquals("_x < _y",c.toString());	
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testLTIntLVarinitializedWithIntervall_IntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y = new IntLVar(2);
	    try{	    	
	    	solver.solve(x.lt(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}

	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testLTIntLVarinitializedWithIntervall_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y = new IntLVar(2,6);
	    try{	    	
	    	solver.solve(x.lt(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testLTIntLVarinitializedWithIntervall_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		try{	    	
	    	solver.solve(x.lt(y));
			 fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> INTERO
	@Test
	public void testLTIntLVarInitializedIntLVarInitializedWithInteger_Integer()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
		try{	    	
	    	solver.solve(x.lt(1));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
		
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar NON INIZIALIZZATA 
	@Test
	public void testLTIntLVarInitializedIntLVarInitializedWithInteger_IntLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar();
	    solver.solve(x.lt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testLTIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedWithIntEGER()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar(2);
	    try{	    	
	    	solver.solve(x.lt(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testLTIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar(2,6);
	    solver.solve(x.lt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testLTIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);		
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		try{	    	
	    	solver.solve(x.lt(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
//%%%%%%%%%%%%%%%%%%%%%%% gt %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	// IntLVar NON INIZIALIZZATA ---> INTERO
	@Test
	public void testGTIntLVarUninitializedx_Integer() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	        
	     IntLVar x = new IntLVar("x1");
	     solver.solve(x.gt(1));
	     c = solver.getConstraint();
	     
	   assertEquals("[]",c.toString());
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar NON INIZIALIZZATA y
	@Test
	public void testGTIntLVarUninitializedx_IntLVarUninitializedy() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	        
	     IntLVar x = new IntLVar("x");
	     IntLVar y = new IntLVar("y");
	     solver.solve(x.gt(y));
	     c = solver.getConstraint();	     
	     assertEquals("_x > _y",c.toString());	
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testGTIntLVarUninitializedx_IntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x1");
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testGTIntLVarUninitializedx_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x");
	    IntLVar y = new IntLVar("y",2,6);
	    solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("_x > _y",c.toString());
	}
	
	// IntLVar NON INIZIALIZZATA x ---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testGTIntLVarUninitializedx_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x1");
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	
	// IntLVar INIZIALIZZATA CON INTERO ---> INTERO
	@Test
	public void testGTIntLVarinitializedWithInteger_Integer() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;	        
	     IntLVar x = new IntLVar(8);
	     solver.solve(x.gt(1));
		  c = solver.getConstraint();
		     assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar NON INIZIALIZZATA x
	@Test
	public void testGTIntLVarinitializedWithInteger_IntLVarUninitializedx() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;	        
	     IntLVar x = new IntLVar(8);
	     IntLVar y = new IntLVar();
	     solver.solve(x.gt(y));
		 c = solver.getConstraint();
		 assertEquals("[]",c.toString());	     	
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testGTIntLVarinitializedWithInteger_IntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8);
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.gt(y));
		c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERO ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testGTIntLVarinitializedWithInteger_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8);
	    IntLVar y = new IntLVar(2,6);
	    solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERO---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testGTIntLVarinitializedWithInteger_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> INTERO
	@Test
	public void testGTIntLVarinitializedWithIntervall_Integer() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;	        
	     IntLVar x = new IntLVar(8,12);
	     solver.solve(x.gt(1));
    	 c = solver.getConstraint();
	     assertEquals("[]",c.toString());	
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO---> IntLVar NON INIZIALIZZATA x
	@Test
	public void testGTIntLVarinitializedWithIntervall_IntLVarUninitializedx() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;	        
	     IntLVar x = new IntLVar("x",8,12);
	     IntLVar y = new IntLVar("y");
	     solver.solve(x.gt(y));
         c = solver.getConstraint();
	     assertEquals("_x > _y",c.toString());	
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testGTIntLVarinitializedWithIntervall_IntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.gt(y));
		c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testGTIntLVarinitializedWithIntervall_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x",8,12);
	    IntLVar y = new IntLVar("y",2,6);
	    solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("_x > _y",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTERVALLO---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testGTIntLVarinitializedWithIntervall_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar(8,12);
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> INTERO
	@Test
	public void testGTIntLVarInitializedIntLVarInitializedWithInteger_Integer()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    solver.solve(x.gt(1));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar NON INIZIALIZZATA 
	@Test
	public void testGTIntLVarInitializedIntLVarInitializedWithInteger_IntLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar();
	    solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar INIZIALIZZATA CON INTERO
	@Test
	public void testGTIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedWithIntEGER()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar(2);
	    solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
	
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro ---> IntLVar INIZIALIZZATA CON INTERVALLO
	@Test
	public void testGTIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedWithIntervall()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);
	    IntLVar y = new IntLVar(2,6);
	    solver.solve(x.gt(y));
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	}
		
	// IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro---> IntLVar INIZIALIZZATA CON INTLVAR INIZIALIZZATA INTEro
	@Test
	public void testGTIntLVarInitializedIntLVarInitializedWithInteger_IntLVarInitializedIntLVarInitializedWithInteger()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x1 = new IntLVar(3);
		IntLVar x= new IntLVar(x1);		
	    IntLVar y1 = new IntLVar(3);
		IntLVar y= new IntLVar(y1);
		try{	    	
    	    solver.solve(x.gt(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

////////////////// NEW
	
	// IntLVar con mul
	@Test
	public void test1()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    IntLVar x = new IntLVar("x",-10,10);
	    IntLVar y = new IntLVar("y",9);
	    solver.solve(y.eq(x.mul(x)));
	    c = solver.getConstraint();
	    assertEquals("9 = _x * _x",c.toString());
	    assertEquals("{-3, 3}",x.getDomain().toString());
	}
	
	@Test
	public void test2()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    IntLVar x = new IntLVar("x");
	    IntLVar sum = new IntLVar("sum");
	    IntLVar sub = new IntLVar("sub");
	    IntLVar mul = new IntLVar("mul");
	    IntLVar div = new IntLVar("div");
	    MultiInterval dx = new MultiInterval(-2, 0);
	    dx.add(5);
	    dx.add(8);
	    solver.add(x.dom(dx));
	    solver.add(sum.eq(x.sum(10)));
	    solver.add(sub.eq(x.sub(5)));
	    solver.add(mul.eq(x.mul(-3)));
	    solver.add(div.eq(x.div(-1)));
	    assertTrue(solver.check());
	    assertEquals(x.getDomain().toString(), "{-2..0, 5, 8}");
	    assertEquals(sum.getDomain().toString(), "{8..10, 15, 18}");
	    assertEquals(sub.getDomain().toString(), "{-7..-5, 0, 3}");
	    assertEquals(mul.getDomain().toString(), "{-24, -15, 0..6}");
	    assertEquals(div.getDomain().toString(), "{-8, -5, 0..2}");
	}
	
	@Test
	public void testMod1() {
		IntLVar p = new IntLVar("p", -6, -2);
		IntLVar q = new IntLVar("q", -3, 4);
		IntLVar r = new IntLVar("r");
		Solver solver = new Solver();
		solver.add(r.eq(p.mod(q)));
		assertTrue(solver.check());
		assertEquals(r.getDomain(), new MultiInterval(-3, 0));
		
		solver.clearStore();
		IntLVar p1 = new IntLVar("p1", -6, 2);
		IntLVar q1 = new IntLVar("q1", -3, 4);
		IntLVar r1 = new IntLVar("r1");
		solver.add(r1.eq(p1.mod(q1)));
		assertTrue(solver.check());
		assertEquals(r1.getDomain(), new MultiInterval(-3, 2));
		
		solver.clearStore();
		IntLVar p2 = new IntLVar("p2", 2, 10);
		IntLVar q2 = new IntLVar("q2", 0, 6);
		IntLVar r2 = new IntLVar("r2");
		solver.add(r2.eq(p2.mod(q2)));
		assertTrue(solver.check());
		assertEquals(r2.getDomain(), new MultiInterval(0, 5));
	}
	
	@Test
	public void testMod2() {
		IntLVar p = new IntLVar("p", -6, -2);
		IntLVar q = new IntLVar("q", 2);
		IntLVar r = new IntLVar("r");
		Solver solver = new Solver();
		solver.add(r.eq(p.mod(q)));
		assertTrue(solver.check());
		assertEquals(r.getDomain(), new MultiInterval(-1, 0));
		
		solver.clearStore();
		IntLVar p1 = new IntLVar("p1", -6, 2);
		IntLVar q1 = new IntLVar("q1", 10);
		IntLVar r1 = new IntLVar("r1");
		solver.add(r1.eq(p1.mod(q1)));
		assertTrue(solver.check());
		assertEquals(r1.getDomain(), new MultiInterval(-6, 2));
		
		solver.clearStore();
		IntLVar p2 = new IntLVar("p2", 2, 10);
		IntLVar q2 = new IntLVar("q2", 5);
		IntLVar r2 = new IntLVar("r2");
		solver.add(r2.eq(p2.mod(q2)));
		assertTrue(solver.check());
		assertEquals(r2.getDomain(), new MultiInterval(0, 4));
	}
	
	@Test
	public void testMod3() {
		Solver solver = new Solver();
		IntLVar p3 = new IntLVar("p3", 3);
		IntLVar q3 = new IntLVar("q3", 2);
		IntLVar r3 = new IntLVar("r3");
		solver.add(r3.eq(p3.mod(q3)));
		assertTrue(solver.check());
		assertEquals((int) r3.getValue(), 1);
		
		solver.clearStore();
		IntLVar p4 = new IntLVar("p4", -3);
		IntLVar q4 = new IntLVar("q4", 2);
		IntLVar r4 = new IntLVar("r4");
		solver.add(r4.eq(p4.mod(q4)));
		assertTrue(solver.check());
		assertEquals((int) r4.getValue(), -1);
		
		solver.clearStore();
		IntLVar p5 = new IntLVar("p5", 3);
		IntLVar q5 = new IntLVar("q5", -2);
		IntLVar r5 = new IntLVar("r5");
		solver.add(r5.eq(p5.mod(q5)));
		assertTrue(solver.check());
		assertEquals((int) r5.getValue(), 1);
		
		solver.clearStore();
		IntLVar p6 = new IntLVar("p6", -3);
		IntLVar q6 = new IntLVar("q6", -2);
		IntLVar r6 = new IntLVar("r6");
		solver.add(r6.eq(p6.mod(q6)));
		assertTrue(solver.check());
		assertEquals((int) r6.getValue(), -1);
	}
	
	@Test
	public void testTruncDiv() {
		Solver solver = new Solver();
		IntLVar x = new IntLVar("x", 3);
		IntLVar y = new IntLVar("y", 2);
		IntLVar z = new IntLVar("z");
		solver.add(z.eq(x.truncDiv(y)));
		assertTrue(solver.check());
		assertEquals(z, 1);
		
		IntLVar x1 = new IntLVar("x1", -3);
		IntLVar z1 = new IntLVar("z1");
		solver.add(z1.eq(x1.truncDiv(2)));
		assertTrue(solver.check());
		assertEquals(z1, -1);
		
		IntLVar x2 = new IntLVar("x2", 3);
		IntLVar y2 = new IntLVar("y2", -2);
		IntLVar z2 = new IntLVar("z2");
		solver.add(z2.eq(x2.truncDiv(y2)));
		assertTrue(solver.check());
		assertEquals(z2, -1);
		
		IntLVar x3 = new IntLVar("x3", -3);
		IntLVar z3 = new IntLVar("z3");
		solver.add(z3.eq(x3.truncDiv(-2)));
		assertTrue(solver.check());
		assertEquals(z3, 1);
		
		IntLVar x4 = new IntLVar("x4", 5);
		IntLVar y4 = new IntLVar("y4");
		IntLVar z4 = new IntLVar("z4", 1);
		solver.add(z4.eq(x4.truncDiv(y4)));
		assertTrue(solver.check());
		// FIXME: The domain of y4 should be [3..5]: probably, for a more 
		// precise domains reduction, a specific rule for truncDiv is needed.
		assertEquals(y4.getDomain(), new MultiInterval(1, 5));
	}
	
	@Test
	public void testAbs() {
		Solver solver = new Solver();
		IntLVar x = new IntLVar("x");
		IntLVar y = new IntLVar("y", -5, 9);
		solver.add(y.eq(x.abs()));
		assertTrue(solver.check());
		assertEquals(x.getDomain(), new MultiInterval(-9, 9));
		assertEquals(y.getDomain(), new MultiInterval(0, 9));
	}

	@Test
	public void testAbs1() {
		Solver solver = new Solver();
		IntLVar x1 = new IntLVar("x1", -3, 10);
		IntLVar y1 = new IntLVar("y1");
		solver.add(y1.eq(x1.abs()));
		assertTrue(solver.check());
		assertEquals(x1.getDomain(), new MultiInterval(-3, 10));
		assertEquals(y1.getDomain(), new MultiInterval(0, 10));
	}

	@Test
	public void testAbs2() {
		Solver solver = new Solver();
		IntLVar x2 = new IntLVar("x2");
		IntLVar y2 = new IntLVar("y2", 5);
		solver.add(y2.eq(x2.abs()));
		assertTrue(solver.check());
		assertEquals(x2.getDomain(), new MultiInterval(-5).union(
				                     new MultiInterval(5)));
		
	}

	@Test
	public void testAbs3() {
		Solver solver = new Solver();
		IntLVar x3 = new IntLVar("x3", -4);
		IntLVar y3 = new IntLVar("y3");
		solver.add(y3.eq(x3.abs()));
		assertTrue(solver.check());
		assertEquals((int) y3.getValue(), 4);
	}

	@Test
	public void testAbs4() {
		Solver solver = new Solver();
		IntLVar x4 = new IntLVar("x4", -23, -16);
		IntLVar y4 = new IntLVar("y4");
		solver.add(y4.eq(x4.abs()));
		assertTrue(solver.check());
		assertEquals(y4.getDomain(), new MultiInterval(16, 23));
	}

	@Test
	public void testAbs5() {
		Solver solver = new Solver();
		IntLVar x5 = new IntLVar("x5", -23, -16);
		IntLVar y5 = new IntLVar("y5");
		solver.add(y5.eq(x5.abs()));
		assertTrue(solver.check());
		assertEquals(y5.getDomain(), new MultiInterval(16, 23));
	}
	
	@Test
	public void testAbs6() {
		Solver solver = new Solver();
		IntLVar x6 = new IntLVar("x6");
		IntLVar y6 = new IntLVar("y6", -5, 8);
		solver.add(y6.eq(x6.abs()));
		assertTrue(solver.check());
		assertEquals(x6.getDomain(), new MultiInterval(-8, 8));
		assertEquals(y6.getDomain(), new MultiInterval(0, 8));
	}
	
	@Test
	public void testAbs7() {
		Solver solver = new Solver();
		IntLVar x7 = new IntLVar("x7");
		IntLVar y7 = new IntLVar("y7", 2, 10);
		solver.add(y7.eq(x7.abs()));
		assertTrue(solver.check());
		assertEquals(x7.getDomain(), new MultiInterval(-10, -2).union(
				                     new MultiInterval(2, 10)));
	}
	
}