package jsetl.test;

//import static org.junit.Assert.*;

//import java.io.IOException;

import java.util.ArrayList;
//import java.util.EmptyStackException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import jsetl.exception.Failure;
import jsetl.exception.NotDefConstraintException;
import junit.framework.TestCase;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jsetl.*;

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//           Testing Equality Constraints                 %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


public class ConstraintEqTest extends TestCase {

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//                 Using LVar							  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
// LVAR NON INIZIALIZZATA x = LVAR NON INIZIALIZZATA x
// x = x
	@Test
	public void testEQLVarUninitializedx_LVarUninitializedx() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x");
	     solver.solve(x.eq(x));              // x = x
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());	
	}
	
// LVAR NON INIZIALIZZATA x = LVAR NON INIZIALIZZATA y
// x = y
	@Test
	public void testEQLVarUninitializedx_LVarUninitializedy() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x");
	     LVar y = new LVar("y");
	     solver.solve(x.eq(y));             // x = y
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());	
	     assertEquals("_y",x.toString());
	     assertEquals("_y",y.toString());
	}

//LVAR NON INIZIALIZZATA = INTERO
//x = 1
	@Test
	public void testEQLVarUninitializedx_Integer() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x");
	     solver.solve(x.eq(1));               // x = 1
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	     assertEquals("1",x.toString());
	}

// LVAR NON INIZIALIZZATA  = LVAR INIZIALIZZATA CON INTERO
// y = 2, x = y
	@Test
	public void testEQLVarUninitialized_LVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LVar x = new LVar("x");
	    LVar y = new LVar(2);
	    solver.solve(x.eq(y));          // y = 2, x = y
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	    assertEquals("2",x.toString());
	    assertEquals("2",y.toString());
	}
	
// LVAR NON INIZIALIZZATA  = LVAR INIZIALIZZATA CON INTERO (con eq)
// y = 2, x = y
		@Test
		public void testEQLVarUninitialized_LVarEqInteger()  
		throws NotDefConstraintException, Failure {
			Solver solver = new Solver();
		    Constraint c;
		    LVar x = new LVar("x");
		    LVar y = new LVar();
		    solver.solve(y.eq(2));          // y = 2
		    solver.solve(x.eq(y));          // y = 2, x = y
		    c = solver.getConstraint();
		    assertEquals("[]",c.toString());
		    assertEquals("2",x.toString());
		    assertEquals("2",y.toString());
		}	

//LVAR NON INIZIALIZZATA  = LVAR INIZIALIZZATA CON INTERO (con eq e AND)
//y = 2, x = y
		@Test
		public void testEQLVarUninitialized_LVarEqANDInteger()  
		throws NotDefConstraintException, Failure {
			Solver solver = new Solver();
		    Constraint c;
		    LVar x = new LVar("x");
		    LVar y = new LVar();
		    solver.solve(y.eq(2).and(x.eq(y)));      // y = 2, x = y
		    c = solver.getConstraint();
		    assertEquals("[]",c.toString());
		    assertEquals("2",x.toString());
		    assertEquals("2",y.toString());
		}

//LVAR NON INIZIALIZZATA  = LVAR INIZIALIZZATA CON INTERO (con eq, AND, reordered)
//y = 2, x = y
		@Test
		public void testEQLVarUninitialized_LVarEqANDreorderedInteger()  
		throws NotDefConstraintException, Failure {
			Solver solver = new Solver();
		    Constraint c;
		    LVar x = new LVar("x");
		    LVar y = new LVar();
		    solver.solve(x.eq(y).and(y.eq(2)));      // y = 2, x = y
		    c = solver.getConstraint();
		    assertEquals("[]",c.toString());
		    assertEquals("2",x.toString());
		    assertEquals("2",y.toString());
		}

//LVAR NON INIZIALIZZATA  = LVAR INIZIALIZZATA CON INTERO (add + solve)
//y = 2, x = y
		@Test
		public void testEQLVarUninitialized_LVarInteger_add()  
		throws NotDefConstraintException, Failure {
			Solver solver = new Solver();
		    Constraint c;
		    LVar x = new LVar("x");
		    LVar y = new LVar();
		    solver.add(y.eq(2));               // y = 2
		    solver.add(x.eq(y));               // x = y
		    solver.solve();                    // y = 2, x = y
		    c = solver.getConstraint();
		    assertEquals("[]",c.toString());
		    assertEquals("2",x.toString());
		    assertEquals("2",y.toString());
		}

//LVAR NON INIZIALIZZATA  = LVAR INIZIALIZZATA CON INTERO (add + solve + add)
//y = 2, x = y
		@Test
		public void testEQLVarUninitialized_LVarInteger_add2()  
		throws NotDefConstraintException, Failure {
			Solver solver = new Solver();
		    Constraint c;
		    LVar x = new LVar("x");
		    LVar y = new LVar();
		    solver.add(y.eq(2));               // y = 2
		    solver.solve();                    
		    solver.add(x.eq(y));               // x = y
		    solver.solve();                    
		    c = solver.getConstraint();
		    assertEquals("[]",c.toString());
		    assertEquals("2",x.toString());
		    assertEquals("2",y.toString());
		}
	
// LVAR INIZIALIZZATA CON INTERO = INTERO
// x = 8, x = 1
	@Test
	public void testEQLVarinitializedWithInteger_Integer() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LVar x = new LVar(8);
	     try{		    	
		    	solver.solve(x.eq(1));
				fail("FALLIMENTO!!!!");
			 }
		  catch(Failure e) {}
	
	}
	
// LVAR INIZIALIZZATA CON INTERO = LVAR NON INIZIALIZZATA 
// x = 8, x = y	
	@Test
	public void testEQLVarinitializedWithInteger_LVarUninitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar(8);
	     LVar y = new LVar();
	     solver.solve(x.eq(y));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());	
		 assertEquals("8",x.toString());
		 assertEquals("8",y.toString());
	}
	
// LVAR INIZIALIZZATA CON INTERO = LVAR INIZIALIZZATA CON INTERO
// x = 8, y = 2, x = y
	@Test
	public void testEQLVarinitializedWithInteger_LVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    LVar x = new LVar(8);
	    LVar y = new LVar(2);
	    try{
	        solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}

	}
	
// LVAR INIZIALIZZATA CON INTERO = LVAR INIZIALIZZATA CON LLIST INIZIALIZZATA 
// x = 8, y = [2,1], x = y
	@Test
	public void testEQLVarinitializedWithInteger_LVarInitializedLListInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    LVar x = new LVar(8);
	    LList l = LList.empty().ins(1).ins(2);
		LVar y = new LVar(l);
		try{
		 	solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		 }
		 catch(Failure e) {}	
	}
		
// LVAR INIZIALIZZATA LLIST INIZ = INTERO
// x = [2,1], x = 8
	@Test
	public void testEQLVarInitializedWithLListInitialized_Integer()
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
		 LList l = LList.empty().ins(1).ins(2);
		 LVar x = new LVar(l);
	     try{
		    solver.solve(x.eq(8));
			fail("FALLIMENTO!!!!");
		 }
		 catch(Failure e) {}	
	}
	
// LVAR INIZIALIZZATA LLIST INIZ = LVAR NON INIZIALIZZATA 
// x = [2,1], x = y
	@Test
	public void testEQLVarInitializedWithLListInitialized_LVarUninitialized() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
		 LList l = LList.empty().ins(1).ins(2);
		 LVar x = new LVar(l);
		 LVar y = new LVar();
		 solver.solve(x.eq(y));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
		 assertEquals("[2,1]",x.toString());
		 assertEquals("[2,1]",y.toString());
	}

//LVAR NON INIZIALIZZATA = LVAR INIZIALIZZATA CON LLIST INIZIALIZZATA 
//y = [2,1], x = y
	@Test
	public void testEQLVarUninitialized_LVarInitializedLListInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LVar x = new LVar("x");
	    LList l = LList.empty().ins(1).ins(2);
		LVar y = new LVar(l);
		solver.solve(x.eq(y));           // y = [2,1], x = y
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());
	    assertEquals("[2,1]",x.toString());
	    assertEquals("[2,1]",y.toString());
	}

// LVAR INIZIALIZZATA LLIST INIZ = LVAR INIZIALIZZATA CON INTERO
// x = [2,1], y = 2, x = y
	@Test
	public void testEQLVarInitializedWithLListInitialized_LVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList l = LList.empty().ins(1).ins(2);
		LVar x= new LVar(l);
	    LVar y = new LVar(2);
	    try{
	        solver.solve(x.eq(y));
			fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
// LVAR INIZIALIZZATA CON LLIST INIZ. = LVAR INIZIALIZZATA CON LLIST INIZ.
// x = [2,1], y = [2,1], x = y
	@Test
	public void testEQLVarInitializedWithLListInitialized_LVarInitializedLListInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
		LList l1 = LList.empty().ins(1).ins(2);
		LVar x = new LVar(l1);
		LList l2 = LList.empty().ins(1).ins(2);
		LVar y= new LVar(l2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//  					Using LSet      				  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
// LSet NON INIZIALIZZATO x = LSet NON INIZIALIZZATO x
// x = x
	@Test
	public void testEQLSetUninitializedx_LSetUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
 		LSet x = new LSet("x");
		solver.solve(x.eq(x));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
// LSet NON INIZIALIZZATO x = LSet NON INIZIALIZZATO y
// x = y
	@Test
	public void testEQLSetUninitializedx_LSetUninitializedy() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		solver.solve(x.eq(y));
		c = solver.getConstraint();		
		assertEquals("[]",c.toString());
	    assertEquals("_x",x.toString());
	    assertEquals("_x",y.toString());
	}
	
// LSet NON INIZIALIZZATA x = LSET INIZIALIZZATO
// x = {2,3}
	@Test
	public void testEQLSetUninitializedx_LSetInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		LSet y = LSet.empty().ins(3).ins(2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	    assertEquals("{2,3}",x.toString());
	}

// LSet NON INIZIALIZZATA x = SET INIZIALIZZATO
// x = [2, 3]
	@Test
	public void testEQLSetUninitializedx_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		Set<Integer> y = new HashSet<Integer>();
		y.add(3); y.add(2);  
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	    assertEquals("{2,3}",x.toString());
	}

// LSet NON INIZIALIZZATA x = LSET CON RESTO
// x = {2|x}
	@Test
	public void testEQLSetUninitializedx_LSetWithRestSameVar()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		LSet y = x.ins(2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{2/_?}",x.toString());   // x = {2|N}		
	}

//LSet NON INIZIALIZZATA CON RESTO = LSet NON INIZIALIZZATA x
//{1|s} = s
	@Test
	public void testEQLSetInitializedWithRest_LSetUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet();
		LSet x = s.ins(1);
		solver.solve(x.eq(s));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1/_?}",s.toString());  
	}
	
// LSet NON INIZIALIZZATA x = LSET INIZ CON LVAR NON INIZ
// x = {3,z}
	@Test
	public void testEQLSetUninitializedx_LSetInitializedWithLVarUninitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z = new LVar();
		LSet x = new LSet("x");
		LSet y = LSet.empty().ins(z).ins(3);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{3,_?}",x.toString());   // x = {3,z}		
	}

//LSet INIZIALIZZATA CON LVAR NON INIZ. = LSet NON INIZIALIZZATA 
//{z2,z1} = y
	@Test
	public void testEQLSetInitializedWithLVarUninitialized_LSetUninitialized() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z1 = new LVar("z1");
		LVar z2 = new LVar("z2");
		LSet x = LSet.empty().ins(z1).ins(z2);
		LSet y = new LSet();
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{_z2,_z1}",y.toString());  
	}

// LSet INIZIALIZZATO CON INTERO = LSet NON INIZIALIZZATA 
// {1} = y
	@Test
	public void testEQLSetInitializedWithInteger_LSetUninitialized() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(1);
		LSet y = new LSet();
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1}",y.toString());   		
	}

//LSet NON INIZIALIZZATA = LSET CON RESTO
//x = {2|r}
	@Test
	public void testEQLSetUninitializedx_LSetWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		LSet r = new LSet("r");
		LSet y = r.ins(2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{2/_r}",x.toString());   // x = {2|r}
	}
	
// LSet INIZIALIZZATO CON INTERO = LSET INIZIALIZZATO
// {1} = {2,3}
	@Test
	public void testEQLSetInitializedWithInteger_LSetInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1);
		LSet y = LSet.empty().ins(3).ins(2);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

// LSet NON INIZIALIZZATO CON INTERO = SET INIZIALIZZATO
// {1} = [2, 3]
	@Test
	public void testEQLSetInitializedWithInteger_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1);
		Set<Integer> y = new HashSet<Integer>();
		y.add(3); y.add(2); 
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

// LSet INIZIALIZZATO CON INTERO = LSET CON RESTO
// {1} = {2|r}
	@Test
	public void testEQLSetInitializedWithInteger_LSetWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1);
		LSet r = new LSet();
		LSet y = r.ins(2);		
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

//LSet INIZIALIZZATO CON INTERO = LSET CON RESTO
//{1} = {1|r}
	@Test
	public void testEQLSetInitWithInteger_LSetWithRest_OK()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(1);
		LSet r = new LSet();
		LSet y = r.ins(1);		
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1}",x.toString());   		
		assertEquals("{1}",y.toString());   		
		assertEquals("{}",r.toString());   		
	}
	
// LSet INIZIALIZZATA CON INTERO = LSET INIZ CON LVAR NON INIZ
// {1} = {3,z}
	@Test
	public void testEQLSetInitWithInteger_LSetInitWithLVarUninit()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1);
		LVar z = new LVar();
		LSet y = LSet.empty().ins(z).ins(3);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

//LSet INIZIALIZZATA CON INTERO = LSET INIZ CON LVAR NON INIZ
//{1} = {1,z}
	@Test
	public void testEQLSetInitWithInt_LSetInitWithLVarUninitOK()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(1);
		LVar z = new LVar();
		LSet y = LSet.empty().ins(z).ins(1);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("1",z.toString());   		
	}

// LSet NON INIZIALIZZATA CON RESTO = LSET INIZIALIZZATO
// {1|s} = {3,2}
	@Test
	public void testEQLSetInitializedWithRest_LSetInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet s = new LSet();
		LSet x = s.ins(1);
		LSet y = LSet.empty().ins(3).ins(2);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

// LSet NON INIZIALIZZATA CON RESTO = SET INIZIALIZZATO
// {1|s} = [2, 3]
	@Test
	public void testEQLSetInitializedWithRest_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet s = new LSet();
		LSet x = s.ins(1);
		Set<Integer> y = new HashSet<Integer>();
		y.add(3); y.add(2); 
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
//LSet INIZIALIZZATO CON RESTO = LSET INIZIALIZZATO CON RESTO
//{1|s} = {2|r}
	@Test
	public void testEQLSetInitWithRest_LSetInitWithRest_OK()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet();
        LSet x = s.ins(1);
		LSet r = new LSet();
		LSet y = r.ins(2);		
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1,2/_?}",x.toString());   		
		assertEquals("{2,1/_?}",y.toString());   		
		assertEquals("{2/_?}",s.toString());   		
		assertEquals("{1/_?}",r.toString());   		
	}

// LSet NON INIZIALIZZATA CON RESTO = LSET CON RESTO
// {1|s} = {2|s}
	@Test
	public void testEQLSetInitializedWithRest_LSetWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet();
		LSet x = s.ins(1); 
		LSet y = s.ins(2); 
		solver.solve(x.eq(y));
	    c = solver.getConstraint();
	    solver.solve(c);
	    assertEquals("[]",c.toString());
		assertEquals("{1,2/_?}",s.toString());  
	}	

//LSet INIZIALIZZATO CON RESTO = LSET INIZIALIZZATO CON RESTO
//{1|s} = {2|r}
	@Test
	public void testEQLSetInitWithRestPS_LSetInitWithRestPS_OK()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet("s");
		LSet r = new LSet("r");
		LVar z1 = new LVar("z1");
		LVar z2 = new LVar("z2");
		solver.solve(s.ins(z1).eq(r.ins(z2)));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("_z2",z1.toString());   		
		assertEquals("_z2",z2.toString());   		
		assertEquals("_s",s.toString());   		
		assertEquals("_s",r.toString());  		
		if(solver.nextSolution()) {
			assertEquals("_z2",z1.toString());   		
			assertEquals("_z2",z2.toString());   		
			assertEquals("_s",s.toString());   		
			assertEquals("{_z2/_s}",r.toString());
		}
		if(solver.nextSolution()) {
			assertEquals("_z2",z1.toString());   		
			assertEquals("_z2",z2.toString());   		
			assertEquals("{_z2/_r}",s.toString());
			assertEquals("_r",r.toString());  		
		}
		if(solver.nextSolution()) {
			assertEquals("_z1",z1.toString());   		
			assertEquals("_z2",z2.toString());   		
			assertEquals("{_z2/_?}",s.toString());   		
			assertEquals("{_z1/_?}",r.toString());  		
		}
	}

// LSet NON INIZIALIZZATA CON RESTO = LSET INIZIALIZZATO CON LVAR NON INIZ.
// {1|s} = {3,z}
	@Test
	public void testEQLSetInitializedWithRest_LSetInitializedWithLVarUnitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet();
		LSet x = s.ins(1);
		LVar z = new LVar();
		LSet y = LSet.empty().ins(z).ins(3);
		solver.solve(x.eq(y));
	    c = solver.getConstraint();
	    solver.solve(c);
	    assertEquals("[]",c.toString());
		assertEquals("{3}",s.toString());  
		assertEquals("1",z.toString());  
	}
		
//LSet INIZIALIZZATA CON LVAR NON INIZ = LSET INIZIALIZZATO
// {z2,z1} = {2,3}
	@Test
	public void testEQLSetInitializedWithLVarUninitialized_LSetInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z1 = new LVar();
		LVar z2 = new LVar();
		LSet x = LSet.empty().ins(z1).ins(z2);
		LSet y = LSet.empty().ins(3).ins(2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());			
		assertEquals("3",z1.toString());  
		assertEquals("2",z2.toString());  
	}

//LSet INIZIALIZZATA CON LVAR NON INIZ = SET INIZIALIZZATO
// {z2,z1} = [2,3]
	@Test
	public void testEQLSetInitializedWithLVarUninitialized_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z1 = new LVar();
		LVar z2 = new LVar();
		LSet x = LSet.empty().ins(z1).ins(z2);
		Set<Integer> y = new HashSet<Integer>();
		y.add(3); y.add(2); 
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("3",z1.toString());  
		assertEquals("2",z2.toString());  
	}

//LSet INIZIALIZZATA CON LVAR NON INIZ = LSET INIZ CON LVAR NON INIZ
// {z2,z1} = {3,z1}
	@Test
	public void testEQLSetInitWithLVarUninit_LSetInitWithLVarUninit_SameVar()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z1 = new LVar("z1");
		LVar z2 = new LVar("z2");
		LSet x = LSet.empty().ins(z1).ins(z2);
		LSet y = LSet.empty().ins(z1).ins(3);
		solver.solve(x.eq(y));
	    c = solver.getConstraint();
	    solver.add(c);
	    solver.solve();
	    assertEquals("[]",c.toString());
		assertEquals("_z1",z1.toString());  
		assertEquals("3",z2.toString());     //BUG: infinite soluzioni!!
		}	

//LSet INIZIALIZZATA CON LVAR NON INIZ = LSET INIZ CON LVAR NON INIZ
//{z2,z1} = {3,4}
	@Test
	public void testEQLSetInitWithLVarUninit_LSetInitWithInt()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z1 = new LVar("z1");
		LVar z2 = new LVar("z2");
		LSet x = LSet.empty().ins(z1).ins(z2);
		LSet y = LSet.empty().ins(3).ins(2);
		solver.solve(x.eq(y));
	    c = solver.getConstraint();
	    solver.add(c);
	    solver.solve();
	    assertEquals("[]",c.toString());
		assertEquals("3",z1.toString());  
		assertEquals("2",z2.toString());  	     
		if(solver.nextSolution()) {
			assertEquals("2",z1.toString());  
			assertEquals("3",z2.toString());  
		}					
	}

//LSet INIZIALIZZATA CON LVAR NON INIZ = LSET INIZ CON LVAR NON INIZ
//{z2,z1} = {3,z3}
	@Test
	public void testEQLSetInitWithLVarUninit_LSetInitWithLVarUninit()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z1 = new LVar("z1");
		LVar z2 = new LVar("z2");
		LVar z3 = new LVar("z3");
		LSet x = LSet.empty().ins(z1).ins(z2);
		LSet y = LSet.empty().ins(3).ins(z3);
		solver.solve(x.eq(y));
	    c = solver.getConstraint();
	    solver.add(c);
	    solver.solve();
	    assertEquals("[]",c.toString());
		assertEquals("3",z1.toString());  
		assertEquals("_z3",z2.toString());  	     
		if(solver.nextSolution()) {
			assertEquals("3",z1.toString());  
			assertEquals("3",z2.toString());  
		}							
	}
	
//LSet INIZIALIZZATA CON LVAR NON INIZ = LSET CON RESTO
//{z2,z1} = {2|r}
	@Test
	public void testEQLSetInitializedWithLVarUninitialized_LSetWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z1 = new LVar("z1");
		LVar z2 = new LVar("z2");
		LSet x = LSet.empty().ins(z1).ins(z2);
		LSet r = new LSet();
		LSet y = r.ins(2);
		solver.solve(x.eq(y));
	    c = solver.getConstraint();
	    solver.solve(c);
	    assertEquals("[]",c.toString());
		assertEquals("_z1",z1.toString());  
		assertEquals("2",z2.toString());  
		assertEquals("{_z1}",r.toString());  
	}
	
//LSet INIZIALIZZATA CON LVAR NON INIZ = LSET CON RESTO
//{z2,z1} = {z3|r}
	@Test
	public void testEQLSetInitWithLVarUninit_LSetWithRestPS()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar z1 = new LVar("z1");
		LVar z2 = new LVar("z2");
		LVar z3 = new LVar("z3");
		LSet x = LSet.empty().ins(z1).ins(z2);
		LSet r = new LSet();
		LSet y = r.ins(z3);
		solver.solve(x.eq(y));
	    c = solver.getConstraint();
	    solver.solve(c);
	    assertEquals("[]",c.toString());
		assertEquals("_z1",z1.toString());  
		assertEquals("_z3",z2.toString());  
		assertEquals("_z3",z3.toString());  
		assertEquals("{_z1}",r.toString());  
	}
	
	
//%%%%%%%%%%%%%%%%%%%%%%  DA SISTEMARE %%%%%%%%%%%%%%%%%

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//               Using LList      						  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	
	// LList NON INIZIALIZZATA x ---> LList NON INIZIALIZZATA x
	@Test
	public void testEQLListUninitializedx_LListUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
 		LList x = new LList("x1");
		solver.solve(x.eq(x));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());

	}
	
	// LList NON INIZIALIZZATA x ---> LList NON INIZIALIZZATA y
	@Test
	public void testEQLListUninitializedx_LListUninitializedy() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = new LList("x1");
		LList y = new LList();
		solver.solve(x.eq(y));
		c = solver.getConstraint();		
		assertEquals("[]",c.toString());
	}
	
	// LList NON INIZIALIZZATA x ---> LList INIZIALIZZATO
	@Test
	public void testEQLListUninitializedx_LListInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = new LList("x1");
		LList y = LList.empty().ins(3).ins(2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LList NON INIZIALIZZATA x ---> SET INIZIALIZZATO
	@Test
	public void testEQLListUninitializedx_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = new LList("x1");
		List<Integer> y = new ArrayList<Integer>();
	    y.add(1);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LList NON INIZIALIZZATA x ---> LList CON RESTO
	@Test
	public void testEQLListUninitializedx_LListWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList x = new LList("x1");		
		LList y = x.ins(2);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
		
	// LList NON INIZIALIZZATA x ---> LList INIZ CON LVAR NON INIZ
	@Test
	public void testEQLListUninitializedx_LListInitializedWithLVarUninitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList x = new LList("x1");
		LList y = LList.empty().ins(x).ins(3);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
		
	// LList INIZIALIZZATO CON INTERO ---> LList NON INIZIALIZZATA x
	@Test
	public void testEQLListInitializedWithInteger_LListUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = LList.empty().ins(1);
		solver.solve(x.eq(x));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LList INIZIALIZZATO CON INTERO ---> LList INIZIALIZZATO
	@Test
	public void testEQLListInitializedWithInteger_LListInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList x = LList.empty().ins(1);
		LList y = LList.empty().ins(3).ins(2);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

	// LList NON INIZIALIZZATO CON INTERO ---> SET INIZIALIZZATO
	@Test
	public void testEQLListInitializedWithInteger_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = LList.empty().ins(1);
		List<Integer> y = new ArrayList<Integer>();
	    y.add(1);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LList INIZIALIZZATO CON INTERO ---> LList CON RESTO
	@Test
	public void testEQLListInitializedWithInteger_LListWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList x = LList.empty().ins(1);
		LList y1 = new LList();
		LList y = y1.ins(2);		
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}	
	
	// LList NON INIZIALIZZATA CON INTERO ---> LList INIZ CON LVAR NON INIZ
	@Test
	public void testEQLListInitilizadWithInteger_LListInitializedWithLVarUninitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList x = LList.empty().ins(1);
		LVar y1 = new LVar();
		LList y = LList.empty().ins(y1).ins(3);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	// LList NON INIZIALIZZATA CON RESTO ---> LList NON INIZIALIZZATA x
	@Test
	public void testEQLListInitializedWithRest_LListUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList s = new LList();
		LList x = s.ins(1);
		solver.solve(x.eq(x));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LList NON INIZIALIZZATA CON RESTO ---> LList INIZIALIZZATO
	@Test
	public void testEQLListInitializedWithRest_LListInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList s = new LList();
		LList x = s.ins(1);
		LList y = LList.empty().ins(3).ins(2);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

	// LList NON INIZIALIZZATA CON RESTO ---> SET INIZIALIZZATO
	@Test
	public void testEQLListInitializedWithRest_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList s = new LList();
		LList x = s.ins(1);
		List<Integer> y = new ArrayList<Integer>();
	    y.add(1); 
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());	    
	}

	// LList NON INIZIALIZZATA CON RESTO ---> LList CON RESTO
	// [1|s] = [2|s]
	@Test
	public void testEQLListInitializedWithRest_LListWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList s = new LList();
		LList x = s.ins(1);
		LList y = s.ins(2);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	
	// LList NON INIZIALIZZATA CON RESTO ---> LList INIZ CON LVAR NON INIZ
	@Test
	public void testEQLListInitializedWithRest_LListInitializedWithLVarInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList s = new LList();
		LList x = s.ins(1);
		LVar y1 = new LVar();
		LList y = LList.empty().ins(y1).ins(3);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
			
	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LList NON INIZIALIZZATA x
	@Test
	public void testEQLListInitializedWithLVarUninitialized_LListUninitializedx() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar();
		LVar x2 = new LVar();
		LList x = LList.empty().ins(x1).ins(x2);
		LList y= new LList();
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LList INIZIALIZZATO
	@Test
	public void testEQLListInitializedWithLVarUninitialized_LListInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar();
		LVar x2 = new LVar();
		LList x = LList.empty().ins(x1).ins(x2);
		LList y = LList.empty().ins(3).ins(2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());				
	}

	//LList INIZIALIZZATA CON LVAR NON INIZ ---> SET INIZIALIZZATO
	@Test
	public void testEQLListInitializedWithLVarUninitialized_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar x1 = new LVar();
		LVar x2 = new LVar();
		LList x = LList.empty().ins(x1).ins(x2);
		List<Integer> y = new ArrayList<Integer>();
	    y.add(1);
	    try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LList CON RESTO
	@Test
	public void testEQLListInitializedWithLVarUninitialized_LListWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar();
		LVar x2 = new LVar();
		LList x = LList.empty().ins(x1).ins(x2);
		LList s = new LList();
		LList y = s.ins(2);
		solver.solve(x.eq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());	   		
	}
		
	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LList NON DEL TUTTO INIZIALIZZATO
	@Test
	public void testEQLListInitializedWithLVarUninitialized_LListiNIZ()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar x1 = new LVar();
		LVar x2 = new LVar();
		LList x = LList.empty().ins(x1).ins(x2);
		LList y = LList.empty().ins(x).ins(3);
		try{
	       	solver.solve(x.eq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	//LList NONINIZIALIZZATA eq LList CON RESTO
	// l = [x|r]
	@Test
	public void testEQLListNotInit_LListWithRest()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList r = new LList("r");
		LList l = new LList();
		solver.solve(l.eq(r.ins(new LVar("x"))));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("[_x|_r]",l.toString());
	}
	
	//aggiunto da andrea fois
	@Test
	public void testEq2() {
		assertEquals(LSet.empty().ins(3).ins(2).eq(LSet.empty().ins(2).ins(3)).check(), true);
		LSet set = LSet.mkSet(4);
		LSet set1 = LSet.empty().ins(1).ins(2).ins(3).ins(4);
		
		Solver solver = new Solver();
		solver.add(set1.eq(set));
		solver.check();
		int nsolutions = 0;
		do {
			++nsolutions;
		}while(solver.nextSolution());
		
		assertEquals(24, nsolutions);
	}
}