package jsetl.test;

import java.util.LinkedList;

import jsetl.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import junit.framework.TestCase;

public class HeuristicsTest extends TestCase {

	private Solver s = new Solver();
	private IntLVar x = new IntLVar("x");
	private IntLVar y = new IntLVar("y");
	private IntLVar z = new IntLVar("z");
    private MultiInterval mx = new MultiInterval(1, 10)
						.union(new MultiInterval(28,30));
	private MultiInterval my = new MultiInterval(1, 50);
	private LinkedList<IntLVar> V = new LinkedList<IntLVar>();
	private LabelingOptions lop = new LabelingOptions();
	
	@Before
    public void setUp() 
	throws Exception {
		LObject.USEJOLLYTOKEN = true;
        // Code executed before each test.
		s = new Solver();
		x = new IntLVar("x");
		y = new IntLVar("y");
		z = new IntLVar("z", 0, 40);
		my.add(100);
		my.add(1000);
		V.add(x);
		V.add(y);
		V.add(z);
		s.add(x.dom(mx).and(y.dom(my)));
		lop = new LabelingOptions();
    }

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
	@Test
	public void
	testValGlbVarLeftMost() {
		lop.valueForIntLVar = ValHeuristic.GLB;
		lop.variableChosenInAList = VarHeuristic.LEFT_MOST;
		s.add(IntLVar.label(lop, V));
		assertTrue(s.check());
		assertEquals(x, 1);
		assertEquals(y, 1);
		assertEquals(z, 0);
	}
	
	@Test
	public void
	testValLubVarMidMost() {
		lop.valueForIntLVar = ValHeuristic.LUB;
		lop.variableChosenInAList = VarHeuristic.MID_MOST;
		s.add(IntLVar.label(lop, V));
		assertTrue(s.check());
		assertEquals(x, 30);
		assertEquals(y, 1000);
		assertEquals(z, 40);
	}
	
	@Test
	public void
	testValMidMostVarRightMost() {
		lop.valueForIntLVar = ValHeuristic.MID_MOST;
		lop.variableChosenInAList = VarHeuristic.RIGHT_MOST;
		s.add(IntLVar.label(lop, V));
		assertTrue(s.check());
		assertEquals(x, 5);
		assertEquals(y, 100);
		assertEquals(z, 20);
	}
	
	@Test
	public void
	testValMedianVarMin() {
		lop.valueForIntLVar = ValHeuristic.MEDIAN;
		lop.variableChosenInAList = VarHeuristic.MIN;
		s.add(IntLVar.label(lop, V));
		assertTrue(s.check());
		assertEquals(x, 7);
		assertEquals(y, 26);
		assertEquals(z, 20);
	}
	
	@Test
	public void
	testValEquiRandomVarMax() {
		lop.valueForIntLVar = ValHeuristic.EQUI_RANDOM;
		lop.variableChosenInAList = VarHeuristic.MAX;
		s.add(IntLVar.label(lop, V));
		assertTrue(s.check());
		int vx = x.getValue();
		int vy = y.getValue();
		int vz = z.getValue();
		assertTrue((1 <= vx && vx <= 10) || (28 <= vx && vx <= 30));
		assertTrue((1 <= vy && vy <= 50) || vy == 100 || vy == 1000);
		assertTrue(0 <= vz && vz <= 40);
	}
	
	@Test
	public void
	testValRangeRandomVarFirstFail() {
		lop.valueForIntLVar = ValHeuristic.RANGE_RANDOM;
		lop.variableChosenInAList = VarHeuristic.FIRST_FAIL;
		s.add(IntLVar.label(lop, V));
		assertTrue(s.check());
		int vx = x.getValue();
		int vy = y.getValue();
		int vz = z.getValue();
		assertTrue((1 <= vx && vx <= 10) || (28 <= vx && vx <= 30));
		assertTrue((1 <= vy && vy <= 50) || vy == 100 || vy == 1000);
		assertTrue(0 <= vz && vz <= 40);
	}
	
	@Test
	public void
	testValMidRandomVarRandom() {
		lop.valueForIntLVar = ValHeuristic.MID_RANDOM;
		lop.variableChosenInAList = VarHeuristic.RANDOM;
		s.add(IntLVar.label(lop, V));
		assertTrue(s.check());
		int vx = x.getValue();
		int vy = y.getValue();
		int vz = z.getValue();
		assertTrue(vx == 5 || vx == 29);
		assertTrue(vy == 25 || vy == 100 || vy == 1000);
		assertEquals(vz, 20);
	}
}
