package jsetl.test;

import jsetl.exception.NotInitLObjectException;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import jsetl.*;

import java.util.*;

import junit.framework.TestCase;


public class IntLSetTest extends TestCase {

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}
    //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	// SETNAME
	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	// INTLSET SENZA NOME
	@Test
	public void testSetNameNoName() {
		IntLSet x = new IntLSet();
		assertEquals("?",x.getName());
	}
	
	@Test
	//INTLSET CON NOME
	public void testSetNameWithName() {
		IntLSet x = new IntLSet("x");
		assertEquals("x",x.getName());
	}
	
	//INTLSET VUOTO
	@Test
	public void testSetNameEmpty() {
		IntLSet x = IntLSet.empty();
		
		assertEquals("emptyIntLSet",x.getName());
	}
    //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	//          GET NAME
	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	//INTLSET CON NOME
	@Test
	public void testGetNameWithName() {
		IntLSet x = IntLSet.empty().ins(1).setName("x"); 
		assertTrue("x".equals(x.getName()));
    }
	
	//INTLSET SENZA NOME
	@Test
	public void testGetNameNotName() {
		IntLSet x = new IntLSet(); 
		assertTrue("?".equals(x.getName()));
	}
	
	//INTLSET VUOTO
	@Test
	public void testGetNameEmpty() {
		IntLSet x = new IntLSet(); 
		assertTrue("?".equals(x.getName()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%%%%%
	//        GETVALUE
	//%%%%%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testGetValueInitializedEmpty() {
		IntLSet x = IntLSet.empty().setName("x"); 
		Set y = new HashSet();
		assertEquals(y,(x.getValue()));
	}
	
	//NON INIZIALIZZATO MODO DIRETTO
	@Test
	public void testGetValueUninitialized() {
		IntLSet x = new IntLSet(); 
		assertEquals(null,(x.getValue()));
	}
	
	@Test // INS, VUOTO , INTERO 
	public void testGetValueInitializedWithInsEmpty() {
		IntLSet x = IntLSet.empty().ins(1);   
        Set y = new HashSet();
		y.add(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS,VUOTO , LVAR INIZIALIZZATO 
	public void testGetValueInitializedWithInsIntLVarInitialized() {
		IntLVar x1 = new IntLVar("x1",1);   
		IntLSet x = IntLSet.empty().ins(x1);
        Set y = new HashSet();
		y.add(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, VUOTO E IntLVar NON INIZIALIZZATO 
	public void testGetValueInitializedWithInsIntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1");   
		IntLSet x = IntLSet.empty().ins(x1);
        Set y = new HashSet();
		y.add(x1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS VUOTO E IntLVar INIZIALIZZATO, IntLVar NON INIZIALIZZATO E INTERO 
	public void testGetValueInitializedWithInsInteger_IntLVarInitialized_IntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1",2);
		IntLVar x2 = new IntLVar("x2");   
		IntLSet x = IntLSet.empty().ins(1).ins(x1).ins(x2);
		Set y = new HashSet();
		y.add(1);y.add(2);y.add(x2);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, INTLSET NON INIZIALIZZATO E INTERO
	public void testGetValueUninitialized_insInteger() {
		IntLSet r = new IntLSet("r");
		IntLSet x = r.ins(1);
		IntLSet y = IntLSet.empty().ins(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, INTLSET NON INIZIALIZZATO E IntLVar NON INIZIALIZZATO
	public void testGetValueUninitialized_insIntLVarUninitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = r.ins(x1);
		IntLSet y = IntLSet.empty().ins(x1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, INTLSET NON INIZIALIZZATO E IntLVar INIZIALIZZATO 
	public void testGetValueUninitialized_insIntLVarInitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = r.ins(x1);
		IntLSet y = IntLSet.empty().ins(x1).ins(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, INTLSET NON INIZIALIZZATO INTERO, IntLVar NON INIZIALIZZATO, IntLVar INIZIALIZZATO 
	public void testGetValueUninitialized_insInteger_IntLVarUninitialized_IntLVarInitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x0 = new IntLVar("x0");
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = r.ins(x1).ins(x0).ins(2);
		Set y = new HashSet();
		y.add(1);y.add(x0);y.add(2);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INSALL, VUOTO e ARRAY 
	public void testGetValueInitializedWithInsAllEmpty_Array() {
		Integer [] x1={1,2,3};
		IntLSet x = IntLSet.empty().insAll(x1);   
        Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INSALL, VUOTO VECTOR 
	public void testGetValueInitializedWithInsAllEmpty_Vector() {
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		IntLSet x = IntLSet.empty().insAll(x1);
		Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INSALL, INTLSET non inizializzato e ARRAY 
	public void testGetValueUnitializedWithInsAllRest_Array() {
		IntLSet r= new IntLSet ();
		Integer [] x1={1,2,3};
		IntLSet x = r.insAll(x1);   
        Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals(y,(x.getValue()));
	}
	@Test // INSALL, INTLSET non inizializzato e VECTOR 
	public void testGetValueUnitializedWithInsAllRest_Vector() {
		IntLSet r= new IntLSet ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		IntLSet x = r.insAll(x1);   
        Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals(y,(x.getValue()));
	}
	
		
	@Test // INSALL, INTLSET INIZIALIZZATO VECTOR 
	public void testGetValueInitializedWithInsAll_Vector() {
		IntLSet r= IntLSet.empty().ins(1).ins(2);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(3); x1.add(4);x1.add(5);
		IntLSet x = r.insAll(x1);
		Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);y.add(4);y.add(5);
        assertEquals(y,(x.getValue()));
    }
	
	@Test // INSALL, INTLSET  inizializzato e ARRAY 
	public void testGetValueInitializedWithInsAll_Array() {
		IntLSet r= IntLSet.empty().ins(1).ins(2);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(3); x1.add(4);x1.add(5);
		IntLSet x = r.insAll(x1);   
        Set y = new HashSet();
        y.add(1);y.add(2);y.add(3);y.add(4);y.add(5);
		assertEquals(y,(x.getValue()));
	}
		
	@Test //NEW INTLSET, VUOTO 
	public void testGetValueInitializedWithNewIntLSetEmpty() {
		IntLSet x1=IntLSet.empty();
		IntLSet x= new IntLSet(x1);
		Set y = new HashSet();
		assertEquals(y,(x.getValue()));
	}

	@Test //NEW INTLSET, INS
	public void testGetValueInitializedWithNewIntLSetIns() {
		IntLSet x1 = IntLSet.empty().ins(1); 
		IntLSet x= new IntLSet(x1);
		Set y = new HashSet();
		y.add(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test //NEW LSET, COSTRUTTORE NON INIZIALIZZATO
	public void testGetValueInitializedWithNewIntLSetCostruttoriUninitialized() {
		IntLSet x1= new IntLSet();
		IntLSet x= new IntLSet(x1);
		IntLSet y = IntLSet.empty();
		assertEquals(null,(x.getValue()));
	}
		
	@Test //NEW SET, VUOTO 
	public void testGetValueInitializedWithSetEmpty() {
		Set x1 = new HashSet();
		IntLSet x= new IntLSet(x1);
		Set y = new HashSet();
		assertEquals(y,(x.getValue()));
	}
	
	@Test //NEW SET, ADD
	public void testGetValueInitializedWithSetAdd() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2); 
		IntLSet x= new IntLSet(x1);
		Set y = new HashSet();
		y.add(1);y.add(2);
		assertEquals(y,(x.getValue()));
	}
	//%%%%%%%%%%%%%%%%%%%%%%%%	
	//          NORMALIZE
	//%%%%%%%%%%%%%%%%%%%%%%%
	//NON INIZIALIZZATO MODO DIRETTO
	@Test
	public void testNormalizedUninitialized() {
		IntLSet x = new IntLSet();
		assertEquals(x.getTail(),(x.normalizeSet()));
	}

	@Test //VUOTO
	public void testNormalizedInitializedEmpty() {
		IntLSet x = IntLSet.empty().setName("x"); 
		assertEquals(IntLSet.empty(),(x.normalizeSet()));
	}
		
	@Test // INS, VUOTO e INTERO 
	public void testNormalizedInitializedWithInsEmpty() {
		IntLSet x = IntLSet.empty().ins(1).ins(2).ins(1);
		IntLSet y = IntLSet.empty().ins(1).ins(2).setName("y");
		assertEquals(y,(x.normalizeSet()));
	}
	
	@Test // INS,VUOTO E IntLVar INIZIALIZZATO 
	public void testNormalizedInitializedWithInsIntLVarInitialized() {
		IntLVar x1 = new IntLVar("x1",1);   
		IntLSet x = IntLSet.empty().ins(1).ins(x1);
		IntLSet y = IntLSet.empty().ins(1).setName("y");
//	    assertEquals(y.toArrayList(),(x.normalizeSet().toArrayList()));
	    assertEquals(y,(x.normalizeSet()));
	}
		
	@Test // INS, VUOTO E IntLVar NON INIZIALIZZATO 
	public void testNormalizedInitializedWithInsIntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1");   
		IntLSet x = IntLSet.empty().ins(x1).ins(x1);
		IntLSet y =IntLSet.empty().ins(x1);
	    assertEquals(y,(x.normalizeSet()));
	}
		
	@Test // INS VUOTO E IntLVar INIZIALIZZATO, IntLVar NON INIZIALIZZATO E INTERO 
	public void testNormalizedInitializedWithInsInteger_IntLVarInitialized_IntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1",2);
		IntLVar x2 = new IntLVar("x2");   
		IntLSet x = IntLSet.empty().ins(2).ins(x1).ins(x2);
		IntLSet y = IntLSet.empty().ins(x1).ins(x2);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test // INS, INTLSET NON INIZIALIZZATO E INTERO
	public void testNormalizedUninitialized_insInteger() {
		IntLSet r = new IntLSet("r");
		IntLSet x = r.ins(1).ins(1).ins(2);
		IntLSet y = r.ins(1).ins(2);
		assertEquals(y,(x.normalizeSet()));
	}
		
	@Test // INS, INTLSET NON INIZIALIZZATO E IntLVar NON INIZIALIZZATO
	public void testNormalizedUninitialized_insIntLVarUninitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = r.ins(x1).ins(x1);
		IntLSet y= r.ins(x1);
		assertEquals(y,(x.normalizeSet()));
    }
		
	@Test // INS, INTLSET NON INIZIALIZZATO E IntLVar INIZIALIZZATO 
	public void testNormalizedUninitialized_insIntIntLVarInitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = r.ins(x1).ins(1);
		IntLSet y = r.ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test // INS, INTLSET NON INIZIALIZZATO INTERO, IntLVar NON INIZIALIZZATO, IntLVar INIZIALIZZATO 
	public void testNormalizedUninitialized_insInteger_IntLVarUninitialized_IntLVarInitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x0 = new IntLVar("x0");
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = r.ins(x1).ins(x0).ins(1);
		IntLSet y = r.ins(x0).ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
				
	@Test // INSALL, VUOTO e ARRAY 
	public void testNormalizeInitializedWithInsAllEmpty_Array() {
		Integer [] x1={1,2,3};
		IntLSet x = IntLSet.empty().insAll(x1).ins(1);
		IntLSet y = IntLSet.empty().ins(1).ins(2).ins(3);
		assertEquals(y,(x.normalizeSet()));
	}
		
	@Test // INSALL, VUOTO VECTOR 
	public void testNormalizeInitializedWithInsAllEmpty_Vector() {
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		IntLSet x = IntLSet.empty().insAll(x1).ins(1);
		IntLSet y = IntLSet.empty().ins(1).ins(2).ins(3);
		assertEquals(y,(x.normalizeSet()));
	}
		
	@Test // INSALL, INTLSET non inizializzato e ARRAY 
	public void testNormalizeUnitializedWithInsAllRest_Array() {
		IntLSet r= new IntLSet ();
		Integer [] x1={1,2,3};
		IntLSet x = r.insAll(x1).ins(2);
		IntLSet y = r.ins(1).ins(2).ins(3);
		assertEquals(y,(x.normalizeSet()));
	}

	@Test // INSALL, INTLSET non inizializzato e ARRAY 
	public void testNormalizeUnitializedWithInsAllRest_Vector() {
		IntLSet r= new IntLSet ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		IntLSet x = r.insAll(x1).ins(1);   
		IntLSet y = r.ins(1).ins(2).ins(3);
		assertEquals(y,(x.normalizeSet()));
	}
		
	@Test // INSALL, IntLSET INIZIALIZZATO VECTOR 
	public void testNormalizedInitializedWithInsAll_Vector() {
		IntLSet r= IntLSet.empty().ins(1).ins(2).ins(1);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(3); x1.add(4);x1.add(5);
		IntLSet x = r.insAll(x1);
		IntLSet y = IntLSet.empty().ins(1).ins(2).ins(3).ins(4).ins(5);
		assertEquals(x,(x.normalizeSet()));
    }
	
	@Test // INSALL, INTLSET  inizializzato e ARRAY 
	public void testNormalizedInitializedWithInsAll_Array() {
		IntLSet r= IntLSet.empty().ins(1).ins(2).ins(1);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(3); x1.add(4);x1.add(5);
		IntLSet x = r.insAll(x1);   
		IntLSet y = IntLSet.empty().ins(1).ins(2).ins(3).ins(4).ins(5);
		assertEquals(y,(x.normalizeSet()));
	}		
			
	@Test //NEW INTLSET, VUOTO 
	public void testNormalizeInitializedWithNewIntLSetEmpty() {
		IntLSet x1=IntLSet.empty();
		IntLSet x= new IntLSet(x1);
		assertEquals(IntLSet.empty(),(x.normalizeSet()));
	}
	
	@Test //NEW INTLSET, INS
	public void testNormalizeInitializedWithNewIntLSetIns() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2).ins(1); 
		IntLSet x= new IntLSet(x1);
		IntLSet y = IntLSet.empty().ins(1).ins(2);
		assertEquals(y,(x.normalizeSet()));
	}
	
	@Test //NEW LSET, INTLSET NON INIZIALIZZATO
	public void testNormalizeInitializedWithNewIntLSetUninitialized() {
		IntLSet x1= new IntLSet();
		IntLSet x= new IntLSet(x1);
		assertEquals(x.getTail(),(x.normalizeSet()));
	}
			
	@Test //NEW SET, VUOTO 
	public void testNormalizeInitializedWithSetEmpty() {
		Set x1 = new HashSet();
		IntLSet x= new IntLSet(x1);
		assertEquals(LSet.empty(),(x.normalizeSet()));
	}

	@Test //NEW SET, INS
	public void testNormalizeInitializedWithSetAdd() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2); x1.add(2);
		IntLSet x= new IntLSet(x1);
		IntLSet y = IntLSet.empty().ins(2).ins(1);
		assertEquals(y,(x.normalizeSet()));
	}
			
	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%			
    //         TOSTRING
	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	//NON INIZIALIZZATO MODO DIRETTO
	@Test
	public void testToStringUninitialized() {
		IntLSet x = new IntLSet(); 
		assertEquals("_?",(x.toString()));
	}

	@Test //VUOTO
	public void testToStringInitializedEmpty() {
		IntLSet x = IntLSet.empty().setName("x"); 
		assertEquals("{}",(x.toString()));
	}
			
	@Test // INS, VUOTO , INTERO 
	public void testToStringInitializedWithInsEmpty() {
		IntLSet x = IntLSet.empty().ins(1);   
        assertEquals("{1}",(x.toString()));
	}
			
	@Test // INS,VUOTO , LVAR INIZIALIZZATO 
	public void testToStringInitializedWithInsIntLVarInitialized() {
		IntLVar x1 = new IntLVar("x1",1);   
		IntLSet x = IntLSet.empty().ins(x1);
        assertEquals("{1}",(x.toString()));
	}
			
	@Test // INS, VUOTO E LVAR NON INIZIALIZZATO 
	public void testToStringInitializedWithInsIntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1");   
		IntLSet x =IntLSet.empty().ins(x1);
        assertEquals("{_x1}",(x.toString()));
	}
			
	@Test // INS VUOTO E IntLVar INIZIALIZZATO, IntLVar NON INIZIALIZZATO E INTERO 
	public void testToStringInitializedWithInsInteger_IntLVarInitialized_IntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1",2);
		IntLVar x2 = new IntLVar("x2");   
		IntLSet x = IntLSet.empty().ins(1).ins(x1).ins(x2);
        assertEquals("{1..2, _x2}",(x.toString()));
	}
			
	@Test // INS, INTLSET NON INIZIALIZZATO E INTERO
	public void testToStringUninitialized_insInteger() {
		IntLSet r = new IntLSet("r");
		IntLSet x = r.ins(1);
		assertEquals("{1/_r}",(x.toString()));
	}
			
	@Test // INS, INTLSET NON INIZIALIZZATO E IntLVar NON INIZIALIZZATO
	public void testToStringUninitialized_insIntLVarUninitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = r.ins(x1);
		assertEquals("{_x1/_r}",(x.toString()));
	}
			
	@Test // INS, INTLSET NON INIZIALIZZATO E IntLVar INIZIALIZZATO 
	public void testToStringUninitialized_insIntLVarInitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = r.ins(x1);
		assertEquals("{1/_r}",(x.toString()));
	}
			
	@Test // INS, INTLSET NON INIZIALIZZATO INTERO, IntLVar NON INIZIALIZZATO, IntLVar INIZIALIZZATO 
	public void testToStringUninitialized_insInteger_IntLVarUninitialized_IntLVarInitialized() {
		IntLSet r = new IntLSet("r");
		IntLVar x0 = new IntLVar("x0");
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = r.ins(x1).ins(x0).ins(2);
		assertEquals("{1..2, _x0/_r}",(x.toString()));
	}
			
	@Test // INSALL, VUOTO e ARRAY 
	public void testToStringInitializedWithInsAllEmpty_Array() {
		Integer [] x1={1,2,3};
		IntLSet x = IntLSet.empty().insAll(x1);   
        assertEquals("{1..3}",(x.toString()));
	}
			
	@Test // INSALL, VUOTO VECTOR 
	public void testToStringInitializedWithInsAllEmpty_Vector() {
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		IntLSet x = IntLSet.empty().insAll(x1);
        assertEquals("{1..3}",(x.toString()));
	}
			
	@Test // INSALL, INTLSET non inizializzato e ARRAY 
	public void testToStringUnitializedWithInsAllRest_Array() {
		IntLSet r= new IntLSet ();
		Integer [] x1={1,2,3};
		IntLSet x = r.insAll(x1);   
        Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals("{1..3/_?}",(x.toString()));
	}

	@Test // INSALL, INTLSET non inizializzato e VECTOR 
	public void testToStringUnitializedWithInsAllRest_Vector() {
		IntLSet r= new IntLSet ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		IntLSet x = r.insAll(x1);   
        assertEquals("{1..3/_?}",(x.toString()));
	}
			
	@Test // INSALL, INTLSET INIZIALIZZATO E VECTOR 
	public void testToStringInitializedWithInsAllIntLSetInitialized_Vector() {
		IntLSet r= IntLSet.empty().ins(5);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		IntLSet x = r.insAll(x1);
        assertEquals("{1..3, 5}",(x.toString()));
	}

	@Test // INSALL, INTLSET INIZIALIZZATO e ARRAY 
	public void testToStringInitializedWithInsAllIntLSetInitialized_Array() {
		IntLSet r= IntLSet.empty().ins(5);
		Integer [] x1={1,2,3};
		IntLSet x= r.insAll(x1);   
        assertEquals("{1..3, 5}",(x.toString()));
	}

	
	@Test //NEW INTLSET, VUOTO 
	public void testToStringInitializedWithNewIntLSetEmpty() {
		IntLSet x1=IntLSet.empty();
		IntLSet x= new IntLSet(x1);
		assertEquals("{}",(x.toString()));
	}
	
	@Test //NEW INTLSET, INS
	public void testToStringInitializedWithNewLSetIns() {
		IntLSet x1 = IntLSet.empty().ins(1); 
		IntLSet x= new IntLSet(x1);
		assertEquals("{1}",(x.toString()));
	}
	
	@Test //NEW INTLSET, COSTRUTTORE NON INIZIALIZZATO
	public void testToStringInitializedWithNewIntLSetCostruttoriUninitialized() {
		IntLSet x1= new IntLSet();
		IntLSet x= new IntLSet(x1);
		assertEquals("_?",(x.toString()));
	}
				
	@Test //NEW SET, VUOTO 
	public void testToStringInitializedWithSetEmpty() {
		Set x1 = new HashSet();
		IntLSet x= new IntLSet(x1);
		assertEquals("{}",(x.toString()));
	}
	
	@Test //NEW SET, ADD
	public void testToStringInitializedWithSetAdd() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2); 
		IntLSet x= new IntLSet(x1);
		Set y = new HashSet();
		y.add(1);y.add(2);
		assertEquals("{1..2}",(x.toString()));
	}
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GET REST
	//%%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testgetRestInitializedEmpty() {
		IntLSet x = IntLSet.empty().setName("x"); 
		assertEquals(x,(x.getTail()));
	}
			
	@Test //LSET NON INIZIALIZZATO
	public void testgetRestUnitialized() {
		IntLSet x = new IntLSet(); 
		assertEquals(x,(x.getTail()));
	}
			
	@Test //INTLSET INIZIALIZZATO con INS CLOSED
	public void testgetRestIninializedWithInsClosed() {
		IntLSet x = IntLSet.empty().ins(1).ins(2); 
		IntLSet y = IntLSet.empty();
		assertEquals(y,(x.getTail()));
	}

	@Test //INTLSET INIZIALIZZATO con INS OPEN
	public void testgetRestIninializedWithInsOpen() {
		IntLSet r= new IntLSet();
		IntLSet x = r.ins(1).ins(2); 
		assertEquals(r,(x.getTail()));
	}
			
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI SET CLOSED
	public void testgetRestIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		IntLSet x = new IntLSet(x1);
		IntLSet y = IntLSet.empty();
		assertEquals(y,(x.getTail()));
	}
	
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTLSET CLOSED
	public void testgetRestIninializedWithConstructIntLSetClosed() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = new IntLSet(x1); 
		IntLSet y = IntLSet.empty();
		assertEquals(y,(x.getTail()));
	}
			
	@Test //LSET INIZIALIZZATO con COSTRUTTORI INTLSET OPEN
	public void testgetRestIninializedWithConstructIntLSetOpen() {
		IntLSet r = new IntLSet("r");
		IntLSet x1 = r.ins(1).ins(2);
		IntLSet x = new IntLSet(x1);
		assertEquals(r,(x.getTail()));
	}
				
	@Test //INTLSET INIZIALIZZATO con CLONE
	public void testgetRestIninializedWitClone() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = x1.clone();
		IntLSet y = IntLSet.empty();
		assertEquals(y,(x.getTail()));
	}
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISEMPTY	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsEmptyInitializedEmpty() {
		IntLSet x = IntLSet.empty().setName("x"); 
		assertTrue(x.isEmpty());
	}
				
	@Test //INTLSET NON INIZIALIZZATO
	public void testIsEmptyUnitialized() {
		IntLSet x = new IntLSet(); 
		try{
			 x.isEmpty();
			 fail("Eccezione Empty per intlset non inizializzato");
		 }
		 catch(NotInitLObjectException e){}
	}
	
	
				
	@Test //INTLSET INIZIALIZZATO con INS CLOSED
	
	public void testIsEmptyIninializedWithInsClosed() {
		IntLSet x = IntLSet.empty().ins(1).ins(2); 
		assertFalse(x.isEmpty());
	}
	
	@Test //INTLSET INIZIALIZZATO con INS OPEN
	public void testIsEmptyIninializedWithInsOpen() {
		IntLSet r= new IntLSet();
		IntLSet x = r.ins(1).ins(2); 
		assertFalse(x.isEmpty());
	}
				
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI SET CLOSED
	public void testIsEmptyIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		IntLSet x = new IntLSet(x1); 
		assertFalse(x.isEmpty());
	}

	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTSET CLOSED
	public void testIsEmptyIninializedWithConstructIntLSetClosed() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = new IntLSet(x1); 
		assertFalse(x.isEmpty());
	}
				
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTSET OPEN
	public void testIsEmptyIninializedWithConstructIntLSetOpen() {
		IntLSet r = new IntLSet("r");
		IntLSet x1 = r.ins(1).ins(2);
		IntLSet x = new IntLSet(x1);
		assertFalse(x.isEmpty());
	}
		
	@Test //INTLSET INIZIALIZZATO con CLONE
	public void testIsEmptytestisBoundIninializedWitClone() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = x1.clone();
		assertFalse(x.isEmpty());
	}
			
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISCLOSED	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsClosedInitializedEmpty() {
		IntLSet x = IntLSet.empty().setName("x"); 
		assertTrue(x.isClosed());
	}
				
	@Test //INTLSET NON INIZIALIZZATO
	public void testIsClosedUnitialized() {
		IntLSet x = new IntLSet(); 
		assertFalse(x.isClosed());
	}
				
	@Test //INTLSET INIZIALIZZATO con INS CLOSED
	
	public void testIsClosedIninializedWithInsClosed() {
		IntLSet x = IntLSet.empty().ins(1).ins(2); 
		assertTrue(x.isClosed());
	}
	
	@Test //LSET INIZIALIZZATO con INS OPEN
	public void testIsClosedIninializedWithInsOpen() {
		IntLSet r= new IntLSet();
		IntLSet x = r.ins(1).ins(2); 
		assertFalse(x.isClosed());
	}
				
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI SET CLOSED
	public void testIsClosedIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		IntLSet x = new IntLSet(x1); 
		assertTrue(x.isClosed());
	}

	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTSET CLOSED
	public void testIsClosedIninializedWithConstructIntLSetClosed() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = new IntLSet(x1); 
		assertTrue(x.isClosed());
	}
				
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTSET OPEN
	public void testIsClosedIninializedWithConstructIntLSetOpen() {
		IntLSet r = new IntLSet("r");
		IntLSet x1 = r.ins(1).ins(2);
		IntLSet x = new IntLSet(x1);
		assertFalse(x.isClosed());
	}
		
	@Test //INTLSET INIZIALIZZATO con CLONE
	public void testIsClosedtestisBoundIninializedWitClone() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = x1.clone();
		assertTrue(x.isClosed());
	}			
			
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISGROUND	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsGroundInitializedEmpty() {
		IntLSet x = IntLSet.empty().setName("x"); 
		assertTrue(x.isGround());
	}
				
	@Test //INTLSET NON INIZIALIZZATO
	public void testIsGroundUnitialized() {
		IntLSet x = new IntLSet();
		assertFalse(x.isGround());

	}
	
	
	@Test //INTLSET INIZIALIZZATO con INS CLOSED
	
	public void testIsGroundIninializedWithInsClosed() {
		IntLSet x = IntLSet.empty().ins(1).ins(2); 
		assertTrue(x.isGround());
	}
	
	@Test //LSET INIZIALIZZATO con INS OPEN
	public void testIsGroundIninializedWithInsOpen() {
		IntLSet r= new IntLSet();
		IntLSet x = r.ins(1).ins(2); 
		assertFalse(x.isGround());
	}
				
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI SET CLOSED
	public void testIsGroundIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		IntLSet x = new IntLSet(x1); 
		assertTrue(x.isGround());
	}

	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTSET CLOSED
	public void testIsGroundIninializedWithConstructIntLSetClosed() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = new IntLSet(x1); 
		assertTrue(x.isGround());
	}
				
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTSET OPEN
	public void testIsGroundIninializedWithConstructIntLSetOpen() {
		IntLSet r = new IntLSet("r");
		IntLSet x1 = r.ins(1).ins(2);
		IntLSet x = new IntLSet(x1);
		assertFalse(x.isGround());
	}
		
	@Test //INTLSET INIZIALIZZATO con CLONE
	public void testIsGroundtestisBoundIninializedWitClone() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = x1.clone();
		assertTrue(x.isGround());
	}			

	//%%%%%%%%%%%%%%%%%%%%%		
    //         isBound
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testisBoundInitializedEmpty() {
		IntLSet x = IntLSet.empty().setName("x"); 
		assertTrue(x.isBound());
	}
				
	@Test //INTLSET NON INIZIALIZZATO
	public void testisBoundUnitialized() {
		IntLSet x = new IntLSet(); 
		assertFalse(x.isBound());
	}
				
	@Test //INTLSET INIZIALIZZATO con INS CLOSED
	public void testisBoundIninializedWithInsClosed() {
		IntLSet x = IntLSet.empty().ins(1).ins(2); 
		assertTrue(x.isBound());
	}
	
	@Test //LSET INIZIALIZZATO con INS OPEN
	public void testisBoundIninializedWithInsOpen() {
		IntLSet r= new IntLSet();
		IntLSet x = r.ins(1).ins(2); 
		assertTrue(x.isBound());
	}
				
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI SET CLOSED
	public void testisBoundIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		IntLSet x = new IntLSet(x1); 
		assertTrue(x.isBound());
	}

	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTSET CLOSED
	public void testisBoundIninializedWithConstructIntLSetClosed() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = new IntLSet(x1); 
		assertTrue(x.isBound());
	}
				
	@Test //INTLSET INIZIALIZZATO con COSTRUTTORI INTSET OPEN
		public void testisBoundIninializedWithConstructIntLSetOpen() {
		IntLSet r = new IntLSet("r");
		IntLSet x1 = r.ins(1).ins(2);
		IntLSet x = new IntLSet(x1);
		assertTrue(x.isBound());
	}
		
	@Test //INTLSET INIZIALIZZATO con CLONE
	public void testisBoundIninializedWitClone() {
		IntLSet x1 = IntLSet.empty().ins(1).ins(2);
		IntLSet x = x1.clone();
		assertTrue(x.isBound());
	}
		
				
	//%%%%%%%%%%%%%%%%%%%%%		
    //         EQUALS(intlset)	
	//%%%%%%%%%%%%%%%%%%%%
	@Test // VUOTO --> VUOTO
	public void testEqualsEmpty_IntLSetEmpty() {
		IntLSet x = IntLSet.empty();
		IntLSet y = IntLSet.empty();
		assertTrue(x.equals(y));
	}
	
	@Test // VUOTO --> INTLSET NON INIZIALIZZATO
	public void testEqualsEmpty_IntLSetUninitilized() {
		IntLSet x = IntLSet.empty();
		IntLSet y = new IntLSet();
		assertFalse(x.equals(y));
	}
	
	@Test // VUOTO --> INTLSET VUOTO, INS INTERO 
	public void testEqualsEmpty_IntLSetEmptyInsInteger() {
		IntLSet x = IntLSet.empty();
		IntLSet y = IntLSet.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> INTLSET VUOTO, INS IntLVar UNINITIALIZEC
	public void testEqualsEmpty_IntLSetEmptyInsIntLVarUninitialized() {
		IntLSet x = IntLSet.empty();
		IntLVar y1= new IntLVar(); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> INTLSET VUOTO, INS IntLVar INITIALIZEC
	public void testEqualsEmpty_LSetEmptyInsLvarInitialized() {
		IntLSet x = IntLSet.empty();
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> INTLSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO
	public void testEqualsEmpty_IntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger() {
		IntLSet x = IntLSet.empty();
		IntLVar y0= new IntLVar();
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y =IntLSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test // VUOTO --> RESTO, INS INTERO
	public void testEqualsEmpty_LSetEmptyRestInsInteger() {
		IntLSet x = IntLSet.empty();
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
			
	@Test // INTLSET NON INIZIALIZZATO --> VUOTO
	public void testEqualsIntLSetUninitilized_IntLSetEmpty() {
		IntLSet x = new IntLSet();
		IntLSet y = IntLSet.empty();
		assertFalse(x.equals(y));
	}

	@Test // INTLSET NON INIZIALIZZATO --> STESSO INTLSET NON INIZIALIZZATO
	public void testEqualsIntLSetUninitilized_SameIntLSetUninitilized() {
		IntLSet x = new IntLSet();
		assertTrue(x.equals(x));
	}
	
	@Test // INTLSET NON INIZIALIZZATO --> INTLSET NON INIZIALIZZATO
	public void testEqualsIntLSetUninitilized_IntLSetUninitilized() {
		IntLSet x = new IntLSet();
		IntLSet y = new IntLSet();
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET NON INIZIALIZZATO --> INTLSET VUOTO, INS INTERO 
	public void testEqualsIntLSetUninitilized_IntLSetEmptyInsInteger() {
		IntLSet x = new IntLSet();
		IntLSet y = IntLSet.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET NON INIZIALIZZATO --> INTLSET VUOTO, INS IntLVar UNINITIALIZEC
	public void testEqualsIntLSetUninitilized_IntLSetEmptyInsIntLVarUninitialized() {
		IntLSet x = new IntLSet();
		IntLVar y1= new IntLVar(); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET NON INIZIALIZZATO--> INTLSET VUOTO, INS IntLVar INITIALIZEC
	public void testEqualsIntLSetUninitilized_IntLSetEmptyInsIntLVarInitialized() {
		IntLSet x = new IntLSet();
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET NON INIZIALIZZATO --> INTLSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO
	public void testEqualsIntLSetUninitilized_IntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger() {
		IntLSet x = new IntLSet();
		IntLVar y0= new IntLVar();
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET NON INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsIntLSetUninitilized_IntLSetEmptyRestInsInteger() {
		IntLSet x = new IntLSet();
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}				

	@Test //INTLSET VUOTO INS INTERO --> VUOTO
	public void testEqualsIntLSetEmptyInsInteger_IntLSetEmpty() {
		IntLSet x = IntLSet.empty().ins(1);
		IntLSet y = IntLSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET  VUOTO INS INTERO --> INTLSET NON INIZIALIZZATO
	public void testEqualsIntLSetEmptyInsInteger_IntLSetUninitilized() {
		IntLSet x = IntLSet.empty().ins(1);
		IntLSet y = new IntLSet();
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS INTERO --> INTLSET VUOTO, INS INTERO 
	public void testEqualsIntLSetEmptyInsInteger_IntLSetEmptyInsInteger() {
		IntLSet x = IntLSet.empty().ins(1);
		IntLSet y = IntLSet.empty().ins(1);
		assertTrue(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS INTERO --> INTLSET VUOTO, INS IntLVar UNINITIALIZEC
	public void testEqualsIntLSetEmptyInsInteger_IntLSetEmptyInsIntLVarUninitialized() {
		IntLSet x = IntLSet.empty().ins(1);
		IntLVar y1= new IntLVar(); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
			
	@Test //INTLSET VUOTO  INS INTERO--> INTLSET VUOTO, INS LVAR INITIALIZEC
	public void testEqualsIntLSetEmptyInsInteger_IntLSetEmptyInsLvarInitialized() {
		IntLSet x = IntLSet.empty().ins(1);
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertTrue(x.equals(y));
	}
				
				
	@Test //INTLSET VUOTO INS INTERO --> INTLSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO
	public void testEqualsIntLSetEmptyInsInteger_IntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger() {
		IntLSet x = IntLSet.empty().ins(1);
		IntLVar y0= new IntLVar();
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test //INTLSET VUOTO INS INTERO  --> RESTO, INS INTERO
	public void testEqualsIntLSetEmptyInsInteger_IntLSetEmptyRestInsInteger() {
		IntLSet x = IntLSet.empty().ins(1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}				
				
	@Test //INTLSET VUOTO INS IntLVar NON INIZIALIZZATO--> VUOTO
	public void testEqualsLSetEmptyInsIntLVarUninitialized_IntLSetEmpty() {
		IntLVar x1 = new IntLVar();
		IntLSet x = IntLSet.empty().ins(x1);
		IntLSet y = IntLSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar NON INIZIALIZZATO --> INTLSET NON INIZIALIZZATO
	public void testEqualsIntLSetEmptyInsIntLVarUninitialized_IntLSetUninitilized() {
		IntLVar x1 = new IntLVar();
		IntLSet x = IntLSet.empty().ins(x1);
		IntLSet y = new IntLSet();
		assertFalse(x.equals(y));
   }
				
	@Test //INTLSET VUOTO INS IntLVar NON INIZIALIZZATO --> INTLSET VUOTO, INS INTERO 
	public void testEqualsIntLSetEmptyInsIntLVarUninitialized_IntLSetEmptyInsInteger() {
		IntLVar x1 = new IntLVar();
		IntLSet x = IntLSet.empty().ins(x1);
		IntLSet y = IntLSet.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar NON INIZIALIZZATO --> INTLSET VUOTO, INS IntLVar UNINITIALIZEC
	public void testEqualsIntLSetEmptyInsIntLVarUninitialized_IntLSetEmptyInsIntLVarUninitialized() {
		IntLVar x1 = new IntLVar();
		IntLSet x = IntLSet.empty().ins(x1);
		IntLVar y1= new IntLVar(); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar NON INIZIALIZZATO--> INTLSET VUOTO, INS IntLVar INITIALIZEC
	public void testEqualsIntLSetEmptyInsIntLVarUninitialized_IntLSetEmptyInsIntLVarInitialized() {
		IntLVar x1 = new IntLVar();
		IntLSet x = IntLSet.empty().ins(x1);
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar NON INIZIALIZZATO--> INTLSET VUOTO, INS IntLVar INITIALIZATO,INT LVAR NON INIZIALIZZATO, INTERO
	public void testEqualsIntLSetEmptyInsIntLVarUninitialized_IntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger() {
		IntLVar x1 = new IntLVar();
		IntLSet x = IntLSet.empty().ins(x1);
		IntLVar y0= new IntLVar();
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar NON INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsIntLSetEmptyInsIntLVarUninitialized_IntLSetEmptyRestInsInteger() {
		IntLVar x1 = new IntLVar();
		IntLSet x = IntLSet.empty().ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}				

	@Test //INTLSET VUOTO INS IntLVar INIZIALIZZATO--> VUOTO
	public void testEqualsIntLSetEmptyInsIntLVarInitialized_IntLSetEmpty() {
		IntLVar x1 = new IntLVar("X1",1);
		IntLSet x = IntLSet.empty().ins(x1);
		IntLSet y = IntLSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar INIZIALIZZATO --> INTLSET NON INIZIALIZZATO
	public void testEqualsIntLSetEmptyInsIntLVarInitialized_IntLSetUninitilized() {
		IntLVar x1 = new IntLVar("X1",1);
		IntLSet x = IntLSet.empty().ins(x1);
		IntLSet y = new IntLSet();
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar INIZIALIZZATO --> INTLSET VUOTO, INS INTERO 
	public void testEqualsIntLSetEmptyInsIntLVarInitialized_IntLSetEmptyInsInteger() {
		IntLVar x1 = new IntLVar("X1",1);
		IntLSet x =IntLSet.empty().ins(x1);
		IntLSet y = IntLSet.empty().ins(1);
		assertTrue(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar INIZIALIZZATO --> INTLSET VUOTO, INS IntLVar UNINITIALIZEC
	public void testEqualsIntLSetEmptyInsIntLVarInitialized_IntLSetEmptyInsIntLVarUninitialized() {
		IntLVar x1 = new IntLVar("X1",1);
		IntLSet x = IntLSet.empty().ins(x1);
		IntLVar y1= new IntLVar(); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar  INIZIALIZZATO--> INTLSET VUOTO, INS IntLVar INITIALIZEC
	public void testEqualsIntLSetEmptyInsIntLVarInitialized_IntLSetEmptyInsIntLVarInitialized() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = IntLSet.empty().ins(x1);
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertTrue(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar  INIZIALIZZATO--> LSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO
	public void testEqualsIntLSetEmptyInsIntLVarInitialized_IntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = IntLSet.empty().ins(x1);
		IntLVar y0= new IntLVar();
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO INS IntLVar  INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsIntLSetEmptyInsIntLVarInitialized_IntLSetEmptyRestInsInteger() {
		IntLVar x1 = new IntLVar("X1",1);
		IntLSet x = IntLSet.empty().ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}	
				
				
	@Test //INTLSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO--> VUOTO
	public void testEqualsIntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger_IntLSetEmpty() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet y = IntLSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO --> INTLSET NON INIZIALIZZATO
	public void testEqualsIntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger_IntLSetUninitilized() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet y = new IntLSet();
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO --> INTLSET VUOTO, INS INTERO 
	public void testEqualsIntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger_IntLSetEmptyInsInteger() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet y = IntLSet.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO --> INTLSET VUOTO, INS LVAR UNINITIALIZEC
	public void testEqualsIntLSetEmptyInsIntLVarInitializedLIntvarUninitializedInteger_IntLSetEmptyInsLvarUninitialized() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLVar y1= new IntLVar(); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVAR INITIALIZATO, LVAR NON INIZIALIZZATO, INTERO --> LSET VUOTO, INS LVAR INITIALIZEC
	public void testEqualsIntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger_IntLSetEmptyInsIntLVarInitialized() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLVar y1= new IntLVar("y1",1); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVAR INITIALIZATO, LVAR NON INIZIALIZZATO, INTERO--> LSET VUOTO, INS LVAR INITIALIZATO, LVAR NON INIZIALIZZATO, INTERO
	public void testEqualsIntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger_IntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger() {
		IntLVar x0= new IntLVar("x0");
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLVar y0= new IntLVar("y0");
		IntLVar y1= new IntLVar("y1",2); 
		IntLSet y = IntLSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //INTLSET VUOTO, INS IntLVar INITIALIZATO, IntLVar NON INIZIALIZZATO, INTERO  --> RESTO, INS INTERO
	public void testEqualsIntLSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger_IntLSetEmptyRestInsInteger() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",1); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}	

	@Test //RESTO, INS INTERO--> VUOTO
	public void testEqualsLSetEmptyRestInsInteger_LSetEmpty() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> INTLSET NON INIZIALIZZATO
	public void testEqualsIntLSetEmptyRestInsInteger_IntLSetUninitilized() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> INTLSET VUOTO, INS INTERO 
	public void testEqualsIntIntLSetEmptyRestInsInteger_IntLSetEmptyInsInteger() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> INTLSET VUOTO, INS LVAR UNINITIALIZEC
	public void testEqualsIntLSetEmptyRestInsInteger_IntLSetEmptyInsLvarUninitialized() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LSET VUOTO, INS LVAR INITIALIZEC
	public void testEqualsIntLSetEmptyRestInsInteger_IntLSetEmptyInsLvarInitialized() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
				
	@Test //RESTO, INS INTERO--> LSET VUOTO, INS LVAR INITIALIZATO, LVAR NON INIZIALIZZATO, INTERO
	public void testEqualsIntLSetEmptyRestInsInteger_LSetEmptyInsIntLVarInitializedIntLVarUninitializedInteger() {
		IntLVar x0= new IntLVar("x0");
		IntLVar x1= new IntLVar("x1",2); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				

	@Test //RESTO, INS INTERO  --> RESTO, INS INTERO
	public void testEqualsIntLSetEmptyRestInsInteger_IntLSetEmptyRestInsInteger() {
		IntLVar x0= new IntLVar();
		IntLVar x1= new IntLVar("x1",1); 
		IntLSet x = IntLSet.empty().ins(1).ins(x0).ins(x1);
		IntLSet r= new IntLSet("r");
		IntLSet y = r.ins(1);
		assertFalse(x.equals(y));
	}

	//%%%%%%%%%%%%%%%%%%%%%		
    //         EQUALS(object)	
	//%%%%%%%%%%%%%%%%%%%%
	
	// INTLSET NON INIZIALIZZATO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetUninitialized_HashSetUninitialized() {
		IntLSet x = new IntLSet();
		Set s = new HashSet();
        assertFalse(x.equals(s));
	}
	
	// INTLSET NON INIZIALIZZATO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetUninitialized_HashSetInitialized() {
		IntLSet x = new IntLSet();
		Set s = new HashSet();
		s.add(2);s.add(3);
        assertFalse(x.equals(s));
	}
	
	// INTLSET NON INIZIALIZZATO ---> INTERO
	@Test
	public void testEqualsObjectIntLSetUninitialized_Integer() {
		IntLSet x = new IntLSet();
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// INTLSET NON INIZIALIZZATO ---> ARRAY
	@Test
	public void testEqualsObjectIntLSetUninitialized_Array() {
		IntLSet x = new IntLSet();
		Integer[] y= {4,3};
		assertFalse(x.equals(y));
	}
	

	// INTLSET VUOTO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetEmpty_HashSetUninitialized() {
		IntLSet x = IntLSet.empty();
		Set s = new HashSet();
        assertTrue(x.equals(s));
	}
	
	// INTLSET VUOTO ---> SET INIZIALIZZATO
	public void testEqualsObjectIntIntLSetEmpty_SetInitialized() {
		IntLSet x = IntLSet.empty();
		Set s = new HashSet();
		s.add(2);s.add(3);
        assertFalse(x.equals(s));
	}
	
	// INTLSET VUOTO ---> INTERO
	public void testEqualsObjectIntLSetEmpty_Integer() {
		IntLSet x = IntLSet.empty();
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// INTLSET VUOTO ---> ARRAY
	public void testEqualsObjectIntLSetEmpty_Array() {
		IntLSet x = IntLSet.empty();
		Integer[] y= {4,3};
		assertFalse(x.equals(y));
	}
	
	// INTLSET VUOTO INS INTERO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetInitializedWithInsInteger_HashSetUninitialized() {
		LSet x = LSet.empty().ins(1).ins(2);
		Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	
	// INTLSET VUOTO INS INTERO ---> SET INIZIALIZZATO UGUALE
	public void testEqualsObjectIntLSetInitializedWithInsInteger_HashSetInitializedSame() {
		IntLSet x = IntLSet.empty().ins(1).ins(2);
		Set s = new HashSet();
		s.add(1);s.add(2);
        assertTrue(x.equals(s));
	}
	
	// INTLSET VUOTO INS INTERO---> INTERO
	public void testEqualsObjectIntLSetInitializedWithInsInteger_Integer() {
		IntLSet x = IntLSet.empty().ins(1).ins(2);
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// INTLSET VUOTO INS INTERO ---> ARRAY 
	public void testEqualsObjectIntLSetInitializedWithInsInteger_ArraySame() {
		IntLSet x = IntLSet.empty().ins(1).ins(2);
		Integer[] y= {1,2};
		assertFalse(x.equals(y));
	}

	// INTLSET INIZIALIZZATO INS con IntLVar INIZIALIZZATO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetInitializedWithIntLVarInsInitialized_HashSetUninitialized() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = IntLSet.empty().ins(x1).setName("x");
	    Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	// INTLSET INIZIALIZZATO INS con  IntLVar INIZIALIZZATO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetInitializedWithIntLVarInsInitialized_HashSetInitialized() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = IntLSet.empty().ins(x1).setName("x");
	    Set s = new HashSet();
	    s.add(1);
		assertTrue(x.equals(s));
	}
	
	// INTLSET INIZIALIZZATO INS con IntLVar INIZIALIZZATO---> INTERO
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarInitialized_Integer() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = IntLSet.empty().ins(x1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	// INTLSET INIZIALIZZATO INS con IntLVar INIZIALIZZATO---> ARRAY UGUALE
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarInitialized_ArraySame() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLSet x = IntLSet.empty().ins(x1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}
	
	// INTLSET INIZIALIZZATO INS con IntLVar NON INIZIALIZZATO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarUnitialized_HashSetUninitialized() {
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = IntLSet.empty().ins(x1).setName("x");
	    Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	
	// INTLSET INIZIALIZZATO INS con IntLVar NON INIZIALIZZATO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarUnitialized_HashSetInitialized() {
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = IntLSet.empty().ins(x1).setName("x");
	    Set s = new HashSet();
	    s.add(1);
		assertFalse(x.equals(s));
	}
	
	// INTLSET INIZIALIZZATO INS con IntLVar NON INIZIALIZZATO---> INTERO
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarUnitialized_Integer() {
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = IntLSet.empty().ins(x1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	// INTLSET INIZIALIZZATO INS con IntLVar NON INIZIALIZZATO---> ARRAY UGUALE
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarUnitialized_ArraySame() {
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = IntLSet.empty().ins(x1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}
	

	

	// INTLSET INIZIALIZZATO INS con IntLVar NON INIZIALIZZATO, IntLVar INIZIALIZZATO, INTERO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarUnitializedIntLVarInitializedInteger_HashSetUninitialized() {
		IntLVar x0 = new IntLVar("x0",2);
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = IntLSet.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	
	// INTLSET INIZIALIZZATO INS con IntLVar NON INIZIALIZZATO, IntLVar INIZIALIZZATO, INTERO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarUnitializedIntLVarInitializedInteger_HashSetInitialized() {
		IntLVar x0 = new IntLVar("x0",2);
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = IntLSet.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Set s = new HashSet();
	    s.add(1);
		assertFalse(x.equals(s));
	}
	
	// INTLSET INIZIALIZZATO INS con IntLVar NON INIZIALIZZATO, IntLVar INIZIALIZZATO, INTERO---> INTERO
	public void testEqualsObjectIntLSetInitializedWithInsLvarUnitializedIntLVarInitializedInteger_Integer() {
		IntLVar x0 = new IntLVar("x0",2);
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = IntLSet.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	// INTLSET INIZIALIZZATO INS con IntLVar NON INIZIALIZZATO, IntLVar INIZIALIZZATO, INTERO---> ARRAY UGUALE
	public void testEqualsObjectIntLSetInitializedWithInsIntLVarUnitializedLvarInitializedInteger_ArraySame() {
		IntLVar x0 = new IntLVar("x0",2);
		IntLVar x1 = new IntLVar("x1");
		IntLSet x = IntLSet.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}
	
	// INTLSET NON INIZIALIZZATO INS INTERO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetUninitializedInsInteger_SetUninitialized() {
		IntLSet r = new IntLSet ("r");
		IntLSet x = r.ins(1).setName("x");
	    Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	
	// INTLSET NON INIZIALIZZATO INS INTERO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectIntLSetUninitializedInsInteger_SetInitialized() {
		IntLSet r = new IntLSet ("r");
		IntLSet x = r.ins(1).setName("x");
	    Set s = new HashSet();
	    s.add(1);
		assertTrue(x.equals(s));
	}
	
	// INTLSET NON INIZIALIZZATO INS INTERO---> INTERO
	public void testEqualsObjectIntLSetUninitializedInsInteger_Integer() {
		IntLSet r = new IntLSet ("r");
		IntLSet x = r.ins(1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	// INTLSET NON INIZIALIZZATO INS INTERO ---> ARRAY 
	public void testEqualsObjectIntLSetUninitializedInsInteger_Array() {
		IntLSet r = new IntLSet ("r");
		IntLSet x = r.ins(1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}

	public void testConstructorWithInvertedPQ(){
		IntLSet intLSet = new IntLSet(10,5);
		assertTrue(intLSet.isEmpty());
	}
		
}
