package jsetl.test;

import jsetl.test.utilities.comparing.ConstraintStringComparator;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import jsetl.*;

import junit.framework.TestCase;

import java.util.LinkedList;
import java.util.List;

public class IntLVarTest extends TestCase {

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}

	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETNAME	
	//%%%%%%%%%%%%%%%%%%%%
	@Test
	public void testGetNameWithName() {
		IntLVar a = new IntLVar("x");
		assertTrue("x".equals(a.getName()));
	}

	@Test
	public void testGetNameWithNoName() {
		IntLVar a = new IntLVar();
		assertTrue("?".equals(a.getName()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //        SETNAME	
	//%%%%%%%%%%%%%%%%%%%%
	@Test
	public void testSetNameWithName() {
		IntLVar x = new IntLVar("x"); 
        x.setName("x_new");
        assertTrue("x_new".equals(x.getName()));
	}

	@Test
	public void testSetNameWithNotName() {
		IntLVar x = new IntLVar();
        x.setName("x_new");
        assertTrue("x_new".equals(x.getName()));
	}

	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETVALUE	
	//%%%%%%%%%%%%%%%%%%%%
	// INTLVar NON INIZIALIZZATO
	@Test
	public void testGetValueUninitialized() {
		IntLVar x = new IntLVar("x");
		assertEquals(null,(x.getValue()));
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR NON INIZIALIZZATO
	@Test
	public void testGetValueNewIntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1");
		IntLVar x = new IntLVar(x1);
		assertEquals(null,(x.getValue()));
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR INIZIALIZZATO CON INTERO
	@Test
	public void testGetValueNewIntLVarInitializedWithInteger() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLVar x = new IntLVar(x1);
		Integer y = 1;
		assertEquals(y,(x.getValue()));
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR INIZIALIZZATO CON INTERVALLO
	@Test
	public void testGetValueNewIntLVarInitializedWithIntervall() {
		IntLVar x1 = new IntLVar("x1",1,10);
		IntLVar x = new IntLVar(x1);
		assertEquals(null,(x.getValue()));
	}

	//%%%%%%%%%%%%%%%%%%%%%		
    //         TOSTRING	
	//%%%%%%%%%%%%%%%%%%%%
	// INTLVar NON INIZIALIZZATO
	@Test
	public void testToStringUninitialized() {
		IntLVar x = new IntLVar("x");
		assertEquals("_x",(x.toString()));
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR NON INIZIALIZZATO
	@Test
	public void testToStringNewIntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1");
		IntLVar x = new IntLVar(x1);
		assertEquals("_x1",(x.toString()));
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR INIZIALIZZATO CON INTERO
	@Test
	public void testToStringNewIntLVarInitializedWithInteger() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLVar x = new IntLVar(x1);
		assertEquals("1",(x.toString()));
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR INIZIALIZZATO CON INTERVALLO
	@Test
	public void testToStringNewIntLVarInitializedWithIntervall() {
		IntLVar x1 = new IntLVar("x1",1,10);
		IntLVar x = new IntLVar(x1);
		assertEquals("_x1",(x.toString()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         isBound	
	//%%%%%%%%%%%%%%%%%%%%
	// INTLVar NON INIZIALIZZATO
	@Test
	public void testisBoundUninitialized() {
		IntLVar x = new IntLVar("x");
		assertFalse(x.isBound());
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR NON INIZIALIZZATO
	@Test
	public void testisBoundNewIntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1");
		IntLVar x = new IntLVar(x1);
		assertFalse(x.isBound());
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR INTLVar INIZIALIZZATO CON INTERO
	@Test
	public void testisBoundNewIntLVarInitializedWithInteger() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLVar x = new IntLVar(x1);
		assertTrue(x.isBound());
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR INTLVar INIZIALIZZATO CON INTERVALLO
	@Test
	public void testisBoundNewIntLVarInitializedWithIntervall() {
		IntLVar x1 = new IntLVar("x1",1,10);
		IntLVar x = new IntLVar(x1);
		assertFalse(x.isBound());
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETDOM	
	//%%%%%%%%%%%%%%%%%%%%
	// INTLVar NON INIZIALIZZATO
	@Test
	public void testGetDomUninitialized() {
		IntLVar x = new IntLVar("x");
		MultiInterval a = new MultiInterval(Interval.INF, Interval.SUP);

		assertEquals(a,(x.getDomain()));
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR NON INIZIALIZZATO
	@Test
	public void testGetDomNewIntLVarUninitialized() {
		IntLVar x1 = new IntLVar("x1");
		IntLVar x = new IntLVar(x1);
		MultiInterval a= new MultiInterval(Interval.INF, Interval.SUP);
		assertEquals(a,(x.getDomain()));
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVar INIZIALIZZATO CON INTERO
	@Test
	public void testGetDomNewIntLVarInitializedWithInteger() {
		IntLVar x1 = new IntLVar("x1",1);
		IntLVar x = new IntLVar(x1);
		MultiInterval a = new MultiInterval(1);
		assertEquals(a,(x.getDomain()));
		
	}
	
	//INTLVar INIZ CON COSTRUTTORE A CUI E' PASSATO UN INTLVAR INIZIALIZZATO CON INTERVALLO
	@Test
	public void testGetDomNewIntLVarInitializedWithIntervall() {
		IntLVar x1 = new IntLVar("x1",1,10);
		IntLVar x = new IntLVar(x1);
		MultiInterval a = new MultiInterval(1,10);
		assertEquals(a,(x.getDomain()));
	}
	
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         SUM	
	//%%%%%%%%%%%%%%%%%%%%
	// INTLVar NON INIZIALIZZATA -- > INTERO
	@Test
	public void testSUMIntLVarUninitialized_Integer() {
		IntLVar x= new IntLVar();
		IntLVar e= x.sum(1);
		
		Constraint c = new Constraint ("_sum",e,x,1);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar NON INIZIALIZZATA
	@Test
	public void testSUMIntLVarUninitializedx_IntLVarUninitializedy() {
		IntLVar x= new IntLVar();
		IntLVar y= new IntLVar();
		IntLVar e= x.sum(y);
				
		Constraint c = new Constraint ("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar NON INIZIALIZZATA
	@Test
	public void testSUMIntLVarUninitializedx_IntLVarUninitializedx() {
		IntLVar x= new IntLVar();
		IntLVar e= x.sum(x);
			
		Constraint c = new Constraint ("_sum",e,x,x);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar INIZIALIZZATA intero
	@Test
	public void testSUMIntLVarUninitialized_IntLVarInitializedInteger() {
		IntLVar x= new IntLVar();
		IntLVar y= new IntLVar(3);
		IntLVar e= x.sum(y);
				
		Constraint c = new Constraint ("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar INIZIALIZZATA CON INTLVAR
	@Test
	public void testSUMIntLVarUninitialized_IntLVarInitializedWithIntLVar() {
		IntLVar x= new IntLVar();
		IntLVar y1 = new IntLVar(2);
		IntLVar y= new IntLVar(y1);
		IntLVar e= x.sum(y);
				
		Constraint c = new Constraint ("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar INIZIALIZZATA con VINCOLO
	@Test
	public void testSUMIntLVarUninitialized_IntLVarIninitialized_Constraint() {		
		IntLVar x= new IntLVar();
		IntLVar y= new IntLVar();
		IntLVar z= new IntLVar();
		z=x.sub(1);
		y=(z.sum(5));
		IntLVar e= x.sum(y);
				
		Constraint c = 
			new Constraint("_sum",e,x,y).and(
		    new Constraint ("_sum",y,z,5).and(
		    new Constraint ("_sub",z,x,1)));
				
		assertEquals(c,e.getConstraint());   
	}
	
	// INTLVar INIZIALIZZATA INTERO -- > INTERO
	@Test
	public void testSUMIntLVarInitializedWithInteger_Integer() {
		IntLVar x= new IntLVar(3);
		IntLVar e= x.sum(1);
		
		Constraint c = new Constraint("_sum",e,x,1);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTERO -- > INTLVar NON INIZIALIZZATA
	@Test
	public void testSUMIntLVarInitializedWithInteger_IntLVarUninitialized() {
		IntLVar x= new IntLVar(3);
		IntLVar y= new IntLVar();
		IntLVar e= x.sum(y);
				
		Constraint c= new Constraint ("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTERO -- > INTLVar INIZIALIZZATA intero
	@Test
	public void testSUMIntLVarInitializedWithInteger_IntLVarInitializedInteger() {
		IntLVar x= new IntLVar(3);
		IntLVar y= new IntLVar(3);
		IntLVar e= x.sum(y);
				
		Constraint c = new Constraint("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTERO-- > INTLVar INIZIALIZZATA CON INTLVAR
	@Test
	public void testSUMIntLVarInitializedWithInteger_IntLVarInitializedWithIntLVar() {
		IntLVar x= new IntLVar(3);
		IntLVar y1 = new IntLVar(2);
		IntLVar y= new IntLVar(y1);
		IntLVar e= x.sum(y);
				
		Constraint c = new Constraint("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTERO -- > INTLVar INIZIALIZZATA con VINCOLO
	@Test
	public void testSUMIntLVarInitializedWithInteger_IntLVarIninitialized_Constraint() {		
		IntLVar x= new IntLVar(3);
		IntLVar y= new IntLVar();
		IntLVar z= new IntLVar();
		z=x.sub(1);
		y=(z.sum(5));
		IntLVar e= x.sum(y);
				
		Constraint c = 
			new Constraint("_sum",e,x,y).and(
		    new Constraint ("_sum",y,z,5).and(
		    new Constraint ("_sub",z,x,1)));

		assertEquals(c,e.getConstraint());  
	}
	
	// INTLVar INIZIALIZZATA INTLVAR  -- > INTERO
	@Test
	public void testSUMIntLVarInitializedWithIntLVar_Integer() {
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar e= x.sum(1);
		
		Constraint c = new Constraint("_sum",e,x,1);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTLVAR  -- > INTLVar NON INIZIALIZZATA
	@Test
	public void testSUMIntLVarInitializedWithIntLVar_IntLVarUninitialized() {
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar y= new IntLVar();
		IntLVar e= x.sum(y);
				
		Constraint c = new Constraint("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTLVAR  -- > INTLVar INIZIALIZZATA intero
	@Test
	public void testSUMIntLVarInitializedWithIntLVar_IntLVarInitializedInteger() {
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar y= new IntLVar(3);
		IntLVar e= x.sum(y);
				
		Constraint c = new Constraint("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTLVAR -- > INTLVar INIZIALIZZATA CON INTLVAR
	@Test
	public void testSUMIntLVarInitializedWithIntLVar_IntLVarInitializedWithIntLVar() {
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar y1 = new IntLVar(2);
		IntLVar y= new IntLVar(y1);
		IntLVar e= x.sum(y);
			
		Constraint c = new Constraint("_sum",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTLVAR  -- > INTLVar INIZIALIZZATA con VINCOLO
	@Test
	public void testSUMIntLVarInitializedWithIntLVar_IntLVarIninitialized_Constraint() {		
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar y= new IntLVar();
		IntLVar z= new IntLVar();
		z=x.sub(1);
		y=(z.sum(5));
		IntLVar e= x.sum(y);
				
		Constraint c = 
			new Constraint("_sum",e,x,y).and(
		    new Constraint ("_sum",y,z,5).and(
		    new Constraint ("_sub",z,x,1)));

		assertEquals(c,e.getConstraint());   
		
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //        MUL	
	//%%%%%%%%%%%%%%%%%%%%
	// INTLVar NON INIZIALIZZATA -- > INTERO
	@Test
	public void testMULIntLVarUninitialized_Integer() {
		IntLVar x= new IntLVar();
		IntLVar e= x.mul(1);
		
		Constraint c = new Constraint("_mul",e,x,1);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar NON INIZIALIZZATA
	@Test
	public void testMULIntLVarUninitialized_IntLVarUninitialized() {
		IntLVar x= new IntLVar();
		IntLVar y= new IntLVar();
		IntLVar e= x.mul(y);
				
		Constraint c= new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar INIZIALIZZATA intero
	@Test
	public void testMULIntLVarUninitialized_IntLVarInitializedInteger() {
		IntLVar x= new IntLVar();
		IntLVar y= new IntLVar(3);
		IntLVar e= x.mul(y);
				
		Constraint c = new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar INIZIALIZZATA CON INTLVAR
	@Test
	public void testMULIntLVarUninitialized_IntLVarInitializedWithIntLVar() {
		IntLVar x= new IntLVar();
		IntLVar y1 = new IntLVar(2);
		IntLVar y= new IntLVar(y1);
		IntLVar e= x.mul(y);
				
		Constraint c = new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar NON INIZIALIZZATA -- > INTLVar INIZIALIZZATA con VINCOLO
	@Test
	public void testMULIntLVarUninitialized_IntLVarIninitialized_Constraint() {		
		IntLVar x= new IntLVar();
		IntLVar y= new IntLVar();
		IntLVar z= new IntLVar();
		z=x.sub(1);
		y=(z.mul(5));
		IntLVar e= x.mul(y);		
		
		Constraint c = 
			new Constraint("_mul",e,x,y).and(
		    new Constraint ("_mul",y,z,5).and(
		    new Constraint ("_sub",z,x,1)));

		assertEquals(c,e.getConstraint());   
	}
	
	// INTLVar INIZIALIZZATA INTERO -- > INTERO
	@Test
	public void testMULIntLVarInitializedWithInteger_Integer() {
		IntLVar x= new IntLVar(3);
		IntLVar e= x.mul(1);
		
		Constraint c = new Constraint("_mul",e,x,1);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTERO -- > INTLVar NON INIZIALIZZATA
	@Test
	public void testMULIntLVarInitializedWithInteger_IntLVarUninitialized() {
		IntLVar x= new IntLVar(3);
		IntLVar y= new IntLVar();
		IntLVar e= x.mul(y);
				
		Constraint c = new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTERO -- > INTLVar INIZIALIZZATA intero
	@Test
	public void testMULIntLVarInitializedWithInteger_IntLVarInitializedInteger() {
		IntLVar x= new IntLVar(3);
		IntLVar y= new IntLVar(3);
		IntLVar e= x.mul(y);
				
		Constraint c = new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTERO-- > INTLVar INIZIALIZZATA CON INTLVAR
	@Test
	public void testMULIntLVarInitializedWithInteger_IntLVarInitializedWithIntLVar() {
		IntLVar x= new IntLVar(3);
		IntLVar y1 = new IntLVar(2);
		IntLVar y= new IntLVar(y1);
		IntLVar e= x.mul(y);
				
		Constraint c = new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTERO -- > INTLVar INIZIALIZZATA con VINCOLO
	@Test
	public void testMULIntLVarInitializedWithInteger_IntLVarIninitialized_Constraint() {		
		IntLVar x= new IntLVar(3);
		IntLVar y= new IntLVar();
		IntLVar z= new IntLVar();
		z=x.sub(1);
		y=(z.mul(5));
		IntLVar e= x.mul(y);
				
		Constraint c = 
			new Constraint("_mul",e,x,y).and(
		    new Constraint ("_mul",y,z,5).and(
		    new Constraint ("_sub",z,x,1)));

		assertEquals(c,e.getConstraint()); 
	}
	
	// INTLVar INIZIALIZZATA INTLVAR  -- > INTERO
	@Test
	public void testMULIntLVarInitializedWithIntLVar_Integer() {
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar e= x.mul(1);
		
		Constraint c = new Constraint("_mul",e,x,1);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTLVAR  -- > INTLVar NON INIZIALIZZATA
	@Test
	public void testMULIntLVarInitializedWithIntLVar_IntLVarUninitialized() {
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar y= new IntLVar();
		IntLVar e= x.mul(y);
				
		Constraint c = new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTLVAR  -- > INTLVar INIZIALIZZATA intero
	@Test
	public void testMULIntLVarInitializedWithIntLVar_IntLVarInitializedInteger() {
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar y= new IntLVar(3);
		IntLVar e= x.mul(y);
				
		Constraint c = new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTLVAR -- > INTLVar INIZIALIZZATA CON INTLVAR
	@Test
	public void testMULIntLVarInitializedWithIntLVar_IntLVarInitializedWithIntLVar() {
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar y1 = new IntLVar(2);
		IntLVar y= new IntLVar(y1);
		IntLVar e= x.mul(y);
				
		Constraint c = new Constraint("_mul",e,x,y);
		
		assertEquals(c,e.getConstraint());
	}
	
	// INTLVar INIZIALIZZATA INTLVAR  -- > INTLVar INIZIALIZZATA con VINCOLO
	@Test
	public void testMULIntLVarInitializedWithIntLVar_IntLVarIninitialized_Constraint() {		
		IntLVar x1= new IntLVar(2);
		IntLVar x= new IntLVar(x1);
		IntLVar y= new IntLVar();
		IntLVar z= new IntLVar();
		z=x.sub(1);
		y=(z.mul(5));
		IntLVar e= x.mul(y);
			
		Constraint c = 
			new Constraint("_mul",e,x,y).and(
		    new Constraint ("_mul",y,z,5).and(
		    new Constraint ("_sub",z,x,1)));

		assertEquals(c,e.getConstraint());  
	}

	@Test
	public void testCopyConstructor(){
		IntLVar x = new IntLVar(5);
		IntLVar y = new IntLVar(3);
		IntLVar z = x.mul(y); // 15
		IntLVar z_clone = new IntLVar(z);

		Solver solver = new Solver();
		solver.add(z.getConstraint());
		solver.add(z_clone.getConstraint());
		solver.add(z.neq(z_clone));
		assertEquals(false, solver.check());
	}

	@Test
	public void testEqLVarIntLVar(){
		IntLVar a = new IntLVar();
		LVar b = new LVar();
		assertEquals(false, a.lt(3).and(b.eq(a)).and(b.eq(3)).check());
	}

	@Test
	public void testConstraints01(){
		IntLVar x = new IntLVar(), y = new IntLVar("y");
		LSet d = new LSet("d");
		Ris ris = new Ris(x,d,Constraint.truec(),new LPair(x,x.mul(x)));
		Solver solver = new Solver();
		solver.check(new LPair(5,y).in(ris));
		assertEquals("[]", solver.getConstraint().toString());
	}

	@Test
    public void testConstraints02(){
        IntLVar x = new IntLVar("x");
        Constraint f = x.mod(2).eq(0);
        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator("_? = 0 AND _? = _x mod 2",f.toString());
        assertTrue(constraintStringComparator.areEqualInSameOrder());
    }

    @Test
	public void testOutput01(){
		java.io.ByteArrayOutputStream out = new java.io.ByteArrayOutputStream();
		System.setOut(new java.io.PrintStream(out));

		new IntLVar("a", new MultiInterval(MultiInterval.INF, 3)).output();
		new IntLVar("b", new MultiInterval(3, MultiInterval.SUP)).output();
		new IntLVar("c", new MultiInterval(3, 4)).output();
		new IntLVar("d").output();
		new IntLVar("e", 3).output();
		List<Interval> intervals = new LinkedList<>();
		intervals.add(new Interval(-10,10));
		intervals.add(new Interval(20,40));
		new IntLVar("f", new MultiInterval(intervals)).output();

		assertEquals("_a = unknown, _a =< 3\r\n" +
				"_b = unknown, _b >= 3\r\n" +
				"_c = unknown, 3 =< _c =< 4\r\n" +
				"_d = unknown\r\n" +
				"_e = 3\r\n"+
				"_f = unknown -- Domain: {-10..10, 20..40}\r\n", out.toString());



	}
}
