package jsetl.test;

import java.util.HashSet;
import java.util.TreeSet;

import jsetl.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import junit.framework.TestCase;

public class IntervalTest extends TestCase {
	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}

	@Test
	public void
	testEmptyIntervals() {
		int a = Interval.INF - 1;
		Interval i = new Interval(a);
		int b = Interval.SUP + 1;
		Interval j = new Interval(b);
		HashSet<Integer> set = new HashSet<Integer>();
		set.add(a);
		set.add(b);
		Interval k = new Interval(set);
		assertEquals(i, new Interval());
		assertEquals(i, j);
		assertEquals(j, k);
		assertEquals(i, j.intersect(k));
		assertEquals(i.sum(j), j.sub(k));
		assertEquals(k, k.opposite());
		assertFalse(i.contains(j.size()));
		assertTrue(j.subset(k));
		set.remove(a);
		set.remove(b);
		assertEquals(set, i.toSet());
		assertEquals(j.toString(), "{}");
	}
	
	@Test
	public void
	testConvexClosure() {
		TreeSet<Integer> set = new TreeSet<Integer>();
		set.add(Interval.SUP + 1);
		set.add(0);
		set.add(Interval.INF - 1);
		Interval i = new Interval(set);
		assertFalse(i.isUniverse());
		assertTrue(i.isSingleton());
		
		set.add(Interval.INF);
		set.add(8);
		i = new Interval(set);
		assertEquals(i.toString(), "[-1073741823..8]");
		
		set.add(Interval.SUP);
		i = new Interval(set);
		assertEquals(i, Interval.universe());
	}
	
	@Test
	public void
	testIntervalMultiplication() {
		Solver s = new Solver();
		IntLVar x = new IntLVar();
		IntLVar y = new IntLVar();
		IntLVar z = new IntLVar();
		s.add(z.eq(x.mul(y)));
		assertTrue(s.check());
		s.add(x.ge(0));
		assertTrue(s.check());
		s.add(x.le(0));
		assertTrue(s.check());
		assertEquals((int) z.getValue(), 0);
		assertEquals(x, z);
		
		IntLVar xx = new IntLVar();
		s.add(z.eq(xx.mul(y)).and(xx.le(0)));
		s.add(xx.eq(y.mul(5)));
		assertTrue(s.check());
		assertEquals(xx.getDomain().toString(), "[-1073741820..0]");
		
		IntLVar yy = new IntLVar(-1, 1);
		s.add(y.eq(yy.mul(-1)));
		assertTrue(s.check());
		assertEquals(y.getDomain().toString(), "[-1..0]");
	}
	
	@Test
	public void
	testIntervalDivision() {
		Solver s = new Solver();
		IntLVar x = new IntLVar();
		IntLVar y = new IntLVar();
		IntLVar z = new IntLVar();
		s.add(z.eq(x.div(y)));
		assertTrue(s.check());
		s.add(x.lt(0));
		s.add(y.dom(-1, 1));
		assertTrue(s.check());
	
		s.add(y.neq(-1));
		assertTrue(s.check());
		assertEquals((int) y.getValue(), 1);
	}
}
