package jsetl.test;

import jsetl.*;
import jsetl.exception.Failure;
import junit.framework.TestCase;
import org.junit.After;
import org.junit.Before;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;


public class LCollectionIteratorTest extends TestCase{

    @Before
    public void setUp() {
        LObject.USEJOLLYTOKEN = true;
    }

    @After
    public void tearDown(){LObject.USEJOLLYTOKEN = false;}

    public void testEmptyLList() throws Failure {
        LList llist = LList.empty();
        Iterator<?> iterator = llist.iterator();
        assertEquals(false, iterator.hasNext());
    }

    public void testEmptyLSet() {
        LSet llist = LSet.empty();
        Iterator<?> iterator = llist.iterator();
        assertEquals(false, iterator.hasNext());
    }

    public void testEmptyIntLSet() {
        IntLSet llist = IntLSet.empty();
        Iterator<?> iterator = llist.iterator();
        assertEquals(false, iterator.hasNext());
    }

    public void testEmptyRis()  {
        Ris ris = new Ris(new LVar(), LSet.empty(), Constraint.truec());
        assertFalse(ris.iterator().hasNext());
    }

    public void testVarLList() {
        LList llist = new LList();
        assertEquals(false, llist.iterator().hasNext());
    }

    public void testVarLSet() {
        LSet lset = new LSet();
        assertEquals(false, lset.iterator().hasNext());
    }

    public void testVarIntLSet() {
        LSet lset = new IntLSet();
        assertEquals(false, lset.iterator().hasNext());
    }

    public void testVarRis()  {
        Ris ris = new Ris(new LVar(), new LSet(), Constraint.truec());
        try{
            ris.iterator();
            fail();

        }catch (IllegalStateException illegalStateException){
            //DO NOTHING;
        }

    }

    public void testInitRisGround()  {
        Ris ris = new Ris(new LVar(), LSet.empty().ins(10).ins("a"), Constraint.truec());
        assertTrue(ris.iterator().hasNext());
    }

    public void testInitRisNotGround()  {
        Ris ris = new Ris(new LVar(), new LSet().ins(10).ins("a"), Constraint.truec());
        assertTrue(ris.iterator().hasNext());
    }
    public void testInitLListGround1(){
        LList llist = LList.empty().ins(5).ins(4).ins(3).ins(2).ins(1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 ", s);
    }

    public void testInitLListGround2(){
        Integer el1[] = {1,2,3};
        ArrayList<Integer> elems1 = new ArrayList<Integer>(Arrays.asList(el1));

        Integer el2[] = {4,5,6,7};

        ArrayList<Integer> elems2 = new ArrayList<Integer>(Arrays.asList(el2));

        LList llist = LList.empty().insAll(elems2).insAll(elems1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 6 7 ", s);
    }

    public void testInitLListNotGround1(){
        LList llist = new LList().ins(5).ins(4).ins(3).ins(2).ins(1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 ", s);
    }

    public void testInitLListNotGround2(){
        Integer el1[] = {1,2,3};
        ArrayList<Integer> elems1 = new ArrayList<Integer>(Arrays.asList(el1));

        Integer el2[] = {4,5,6,7};

        ArrayList<Integer> elems2 = new ArrayList<Integer>(Arrays.asList(el2));

        LList llist = new LList().insAll(elems2).insAll(elems1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 6 7 ", s);
    }

    public void testInitLSetGround1(){
        LSet llist = LSet.empty().ins(5).ins(4).ins(3).ins(2).ins(1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 ", s);
    }

    public void testInitLSetGround2(){
        Integer el1[] = {1,2,3};
        ArrayList<Integer> elems1 = new ArrayList<Integer>(Arrays.asList(el1));

        Integer el2[] = {4,5,6,7};

        ArrayList<Integer> elems2 = new ArrayList<Integer>(Arrays.asList(el2));

        LSet llist = LSet.empty().insAll(elems2).insAll(elems1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 6 7 ", s);
    }

    public void testInitLSetNotGround1(){
        LSet llist = new LSet().ins(5).ins(4).ins(3).ins(2).ins(1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 ", s);
    }

    public void testInitLSetNotGround2(){
        Integer el1[] = {1,2,3};
        ArrayList<Integer> elems1 = new ArrayList<Integer>(Arrays.asList(el1));

        Integer el2[] = {4,5,6,7};

        ArrayList<Integer> elems2 = new ArrayList<Integer>(Arrays.asList(el2));

        LSet llist = new LSet().insAll(elems2).insAll(elems1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 6 7 ", s);
    }

    public void testInitIntLSetGround1(){
        LSet llist = IntLSet.empty().ins(5).ins(4).ins(3).ins(2).ins(1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 ", s);
    }

    public void testInitIntLSetGround2(){
        Integer el1[] = {1,2,3};
        ArrayList<Integer> elems1 = new ArrayList<Integer>(Arrays.asList(el1));

        Integer el2[] = {4,5,6,7};

        ArrayList<Integer> elems2 = new ArrayList<Integer>(Arrays.asList(el2));

        LSet llist = LSet.empty().insAll(elems2).insAll(elems1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 6 7 ", s);
    }

    public void testInitIntLSetGround3(){
        LSet llist = new IntLSet(10,13).ins(5).ins(4).ins(3).ins(2).ins(1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 10 11 12 13 ", s);
    }

    public void testInitIntLSetNotGround1(){
        LSet llist = new LSet().ins(5).ins(4).ins(3).ins(2).ins(1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 ", s);
    }

    public void testInitIntLSetNotGround2(){
        Integer el1[] = {1,2,3};
        ArrayList<Integer> elems1 = new ArrayList<Integer>(Arrays.asList(el1));

        Integer el2[] = {4,5,6,7};

        ArrayList<Integer> elems2 = new ArrayList<Integer>(Arrays.asList(el2));

        LSet llist = new LSet().insAll(elems2).insAll(elems1);
        String s = "";
        for(Object elem: llist)
            s = s + elem + " ";

        assertEquals("1 2 3 4 5 6 7 ", s);
    }
}