package jsetl.test;

import jsetl.annotation.NotNull;
import jsetl.exception.NotInitLObjectException;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

import jsetl.*;

public class LListTest extends TestCase {
	static class LList1 extends jsetl.LList{
		public LList1(LList l){
			super(l);
		}
		@Override
		public boolean occurs(@NotNull Object object){
			return super.occurs(object);
		}
	}
	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         SETNAME
	//%%%%%%%%%%%%%%%%%%%%
	@Test
	public void testSetNameWithName() {
		LList x = LList.empty().ins(1).setName("x"); 
		assertTrue("x".equals(x.getName()));
    }
	
	@Test
	public void testSetNameNotName() {
		LList x = new LList(); 
		assertTrue("?".equals(x.getName()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETNAME	
	//%%%%%%%%%%%%%%%%%%%%
	@Test
	public void testGetNameWithName() {
		LList x = LList.empty().ins(1).setName("x"); 
		assertTrue("x".equals(x.getName()));
    }
	
	@Test
	public void testGetNameNotName() {
		LList x = new LList(); 
		assertTrue("?".equals(x.getName()));
	}
	
	@Test
	public void testGetNameEmpty() {
		LList x = LList.empty(); 
		assertTrue("emptyLList".equals(x.getName()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETVALUE	
	//%%%%%%%%%%%%%%%%%%%%
	//NON INIZIALIZZATO MODO DIRETTO
	//x.getValue()
	@Test
	public void testGetValueUninitialized() {
		LList x = new LList(); 
		assertEquals(null,(x.getValue()));
	}

	@Test //VUOTO
	//x=[], x.getValue()
	public void testGetValueInitializedEmpty() {
		LList x = LList.empty().setName("x"); 
		List y = new ArrayList();
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, VUOTO , INTERO 
	//x=[1], x.getValue()
	public void testGetValueInitializedWithInsEmpty() {
		LList x = LList.empty().ins(1);  
		List y = new ArrayList();
		y.add(1);
        assertEquals(y,(x.getValue()));
	}
	
	@Test // INS,VUOTO , LVar INIZIALIZZATO 
	//x1=1, x=[x1], x.getValue()
	public void testGetValueInitializedWithInsLVarInitialized() {
		LVar x1 = new LVar("x1",1);   
		LList x = LList.empty().ins(x1);
		List y = new ArrayList();
		y.add(1);
        assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, VUOTO E LVar NON INIZIALIZZATO 
	//x=[x1], x.getValue()
	public void testGetValueInitializedWithInsLVarUninitialized() {
		LVar x1 = new LVar("x1");   
		LList x = LList.empty().ins(x1);
		List y = new ArrayList();
		y.add(x1);
        assertEquals(y,(x.getValue()));
	}
	
	@Test // INS VUOTO E LVar INIZIALIZZATO, LVar NON INIZIALIZZATO E INTERO 
	public void testGetValueInitializedWithInsInteger_LVarInitialized_LVarUninitialized() {
		LVar x1 = new LVar("x1",2);
		LVar x2 = new LVar("x2");   
		LList x = LList.empty().ins(1).ins(x1).ins(x2);
		List y = new ArrayList();
		y.add(x2);y.add(x1);y.add(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, LLIST NON INIZIALIZZATO E INTERO
	//x=[1|r], x.getValue()
	public void testGetValueUninitialized_insInteger() {
		LList r = new LList("r");
//		LList x = r.ins(1);
		LList x = r.ins(new LVar());
		
		//System.out.println(x.getValue());
		
		List y = new ArrayList();
		y.add(1);
//		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, LLIST NON INIZIALIZZATO E LVar NON INIZIALIZZATO
	public void testGetValueUninitialized_insLVarUninitialized() {
		LList r = new LList("r");
		LVar x1 = new LVar("x1");
		LList x = r.ins(x1);
		List y = new ArrayList();
		y.add(x1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, LLIST NON INIZIALIZZATO E LVar INIZIALIZZATO 
	public void testGetValueUninitialized_insLVarInitialized() {
		LList r = new LList("r");
		LVar x1 = new LVar("x1",1);
		LList x = r.ins(x1);
		List y = new ArrayList();
		y.add(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INS, LLIST NON INIZIALIZZATO INTERO, LVar NON INIZIALIZZATO, LVar INIZIALIZZATO 
	public void testGetValueUninitialized_insInteger_LVarUninitialized_LVarInitialized() {
		LList r = new LList("r");
		LVar x0 = new LVar("x0");
		LVar x1 = new LVar("x1",1);
		LList x = r.ins(x1).ins(x0).ins(2);
		List y = new ArrayList();
		y.add(2);y.add(x0);y.add(x1);
		assertEquals(y,(x.getValue()));
	}
		
	@Test // INSALL, VUOTO e ARRAY 
	public void testGetValueInitializedWithInsAllEmpty_Array() {
		Integer [] x1={1,2,3};
		LList x = LList.empty().insAll(x1);  
		List y = new ArrayList();
		y.add(1);y.add(2);y.add(3);
        assertEquals(y,(x.getValue()));
	}
	
	@Test // INSALL, VUOTO VECTOR 
	public void testGetValueInitializedWithInsAllEmpty_Vector() {
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LList x = LList.empty().insAll(x1);
		List y = new ArrayList();
		y.add(1);y.add(2);y.add(3);
        assertEquals(y,(x.getValue()));
	}
	
	@Test // INSALL, LLIST non inizializzato e ARRAY 
	public void testGetValueUnitializedWithInsAllRest_Array() {
		LList r= new LList ();
		Integer [] x1={1,2,3};
		LList x = r.insAll(x1);   
		List y = new ArrayList();
		y.add(1);y.add(2);y.add(3);
        assertEquals(y,(x.getValue()));
	}
	@Test // INSALL, LLIST non inizializzato e VECTOR
	public void testGetValueUnitializedWithInsAllRest_Vector() {
		LList r= new LList ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LList x = r.insAll(x1);
		List y = new ArrayList();
		y.add(1);y.add(2);y.add(3);
        assertEquals(y,(x.getValue()));
	}
	
	@Test // INSALL, LLIST INIZIALIZZATO E VECTOR 
	public void testGetValueInitializedWithInsAllLSetInitialized_Vector() {
		LList r= LList.empty().ins(5);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LList x = r.insAll(x1);
		List y = new ArrayList();
		y.add(1);y.add(2);y.add(3);y.add(5);
        assertEquals(y,(x.getValue()));
	}

	@Test // INSALL, LLIST INIZIALIZZATO e ARRAY 
	public void testGetValueInitializedWithInsAllLSetInitialized_Array() {
		LList r= LList.empty().ins(5);
		Integer [] x1={1,2,3};
		LList x = r.insAll(x1);
		List y = new ArrayList();
		y.add(1);y.add(2);y.add(3);y.add(5);
        assertEquals(y,(x.getValue()));
	}
		
	@Test // INSALL, INIZIALIZZATO VECTOR 
	public void testGetValueInitializedWithInsAll_Vector() {
		LList r= new LList ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LList x = r.ins(x1);
		List y1 = new ArrayList();
		y1.add(1);y1.add(2);y1.add(3);
		List y = new ArrayList();
		y.add(y1);
        assertEquals(y,(x.getValue()));
	}
		
	@Test //NEW LLIST, VUOTO 
	public void testGetValueInitializedWithNewLSetEmpty() {
		LList x1=LList.empty();
		LList x= new LList(x1);
		List y = new ArrayList();
	    assertEquals(y,(x.getValue()));
	}
	
	@Test //NEW LLIST, INS
	public void testGetValueInitializedWithNewLSetIns() {
		LList x1 = LList.empty().ins(1); 
		LList x= new LList(x1);
		List y = new ArrayList();
		y.add(1);
       	assertEquals(y,(x.getValue()));
	}
	
	@Test //NEW LIST, VUOTO
	public void testGetValueInitializedWithNewListEmpty() {
		List x1 = new ArrayList();
		LList x= new LList(x1);
		assertEquals(x1,(x.getValue()));
    }
	
	@Test //NEW LIST, INIZIALIZZATA
	public void testGetValueInitializedWithNewLlistInitialized() {
		List x1 = new ArrayList();
		x1.add(1);x1.add(2);
		LList x= new LList(x1);
		assertEquals(x1,(x.getValue()));
    }
		
		
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         TOSTRING	
	//%%%%%%%%%%%%%%%%%%%%
	//NON INIZIALIZZATO MODO DIRETTO
	@Test
	public void testToStringUninitialized() {
		LList x = new LList(); 
		assertEquals("_?",(x.toString()));
	}

	@Test //VUOTO
	public void testToStringInitializedEmpty() {
		LList x = LList.empty().setName("x"); 
		assertEquals("[]",(x.toString()));
	}
			
	@Test // INS, VUOTO , INTERO 
	public void testToStringInitializedWithInsEmpty() {
		LList x = LList.empty().ins(1);   
        assertEquals("[1]",(x.toString()));
	}
			
	@Test // INS,VUOTO , LVar INIZIALIZZATO 
	public void testToStringInitializedWithInsLVarInitialized() {
		LVar x1 = new LVar("x1",1);   
		LList x = LList.empty().ins(x1);
        assertEquals("[1]",(x.toString()));
	}
			
	@Test // INS, VUOTO E LVar NON INIZIALIZZATO 
	public void testToStringInitializedWithInsLVarUninitialized() {
		LVar x1 = new LVar("x1");   
		LList x =LList.empty().ins(x1);
        assertEquals("[_x1]",(x.toString()));
	}
			
	@Test // INS VUOTO E LVar INIZIALIZZATO, LVar NON INIZIALIZZATO E INTERO 
	public void testToStringInitializedWithInsInteger_LVarInitialized_LVarUninitialized() {
		LVar x1 = new LVar("x1",2);
		LVar x2 = new LVar("x2");   
		LList x = LList.empty().ins(1).ins(x1).ins(x2);
        assertEquals("[_x2,2,1]",(x.toString()));
	}
			
	@Test // INS,LList NON INIZIALIZZATO E INTERO
	public void testToStringUninitialized_insInteger() {
		LList r = new LList("r");
		LList x = r.ins(1);
		assertEquals("[1|_r]",(x.toString()));
	}
		
	@Test // INS, LList NON INIZIALIZZATO E LVar NON INIZIALIZZATO
	public void testToStringUninitialized_insLVarUninitialized() {
		LList r = new LList("r");
		LVar x1 = new LVar("x1");
		LList x = r.ins(x1);
		LList y = LList.empty().ins(x1);
		assertEquals("[_x1|_r]",(x.toString()));
	}
			
	@Test // INS, LList NON INIZIALIZZATO E LVar INIZIALIZZATO 
	public void testToStringUninitialized_insLVarInitialized() {
		LList r = new LList("r");
		LVar x1 = new LVar("x1",1);
		LList x = r.ins(x1);
		assertEquals("[1|_r]",(x.toString()));
	}
			
	@Test // INS, LList NON INIZIALIZZATO INTERO, LVar NON INIZIALIZZATO, LVar INIZIALIZZATO 
	public void testToStringUninitialized_insInteger_LVarUninitialized_LVarInitialized() {
		LList r = new LList("r");
		LVar x0 = new LVar("x0");
		LVar x1 = new LVar("x1",1);
		LList x = r.ins(x1).ins(x0).ins(2);
		assertEquals("[2,_x0,1|_r]",(x.toString()));
	}
			
	@Test // INSALL, VUOTO e ARRAY 
	public void testToStringInitializedWithInsAllEmpty_Array() {
		Integer [] x1={1,2,3};
		LList x = LList.empty().insAll(x1);   
        assertEquals("[1,2,3]",(x.toString()));
	}
			
	@Test // INSALL, VUOTO VECTOR 
	public void testToStringInitializedWithInsAllEmpty_Vector() {
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LList x = LList.empty().insAll(x1);
		assertEquals("[1,2,3]",(x.toString()));
	}
			
	@Test // INSALL, LList non inizializzato e ARRAY 
	public void testToStringUnitializedWithInsAllRest_Array() {
		LList r= new LList ();
		Integer [] x1={1,2,3};
		LList x = r.insAll(x1);   
        Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals("[1,2,3|_?]",(x.toString()));
	}

	@Test // INSALL, LList non inizializzato e VECTOR 
	public void testToStringUnitializedWithInsAllRest_Vector() {
		LList r= new LList ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LList x = r.insAll(x1);   
        assertEquals("[1,2,3|_?]",(x.toString()));
	}
			
	@Test // INSALL, LList INIZIALIZZATO E VECTOR 
	public void testToStringInitializedWithInsAllLListInitialized_Vector() {
		LList r= LList.empty().ins(5);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LList x = r.insAll(x1);
        assertEquals("[1,2,3,5]",(x.toString()));
	}

	@Test // INSALL, LList INIZIALIZZATO e ARRAY 
	public void testToStringInitializedWithInsAllLListInitialized_Array() {
		LList r= LList.empty().ins(5);
		Integer [] x1={1,2,3};
		LList x = r.insAll(x1);   
	    assertEquals("[1,2,3,5]",(x.toString()));
	}
				
				
	@Test //NEW LLIST, VUOTO 
	public void testToStringInitializedWithNewLListEmpty() {
		LList x1=LList.empty();
		LList x= new LList(x1);
		assertEquals("[]",(x.toString()));
	}

	@Test //NEW LLIST, INS
	public void testToStringInitializedWithNewLListIns() {
		LList x1 = LList.empty().ins(1); 
		LList x= new LList(x1);
		assertEquals("[1]",(x.toString()));
	}

	@Test //NEW LLIST, COSTRUTTORE NON INIZIALIZZATO
	public void testToStringInitializedWithNewLSetCostruttoriUninitialized() {
		LList x1= new LList();
		LList x= new LList(x1);
		assertEquals("_?",(x.toString()));
	}
				
	@Test //NEW LIST, VUOTO
	public void testToStringInitializedWithNewListEmpty() {
		List x1 = new ArrayList();
		LList x= new LList(x1);
		assertEquals("[]",(x.toString()));
    }
				
	@Test //NEW LIST, INIZIALIZZATA
	public void testToStringInitializedWithNewLlistInitialized() {
		List x1 = new ArrayList();
		x1.add(1);x1.add(2);
		LList x= new LList(x1);
		assertEquals("[1,2]",(x.toString()));
    }
			
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETREST	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testgetRestInitializedEmpty() {
		LList x = LList.empty().setName("x"); 
		assertEquals(x,(x.getTail()));
	}
			
	@Test //LList NON INIZIALIZZATO
	public void testgetRestUnitialized() {
		LList x = new LList(); 
		assertEquals(x,(x.getTail()));
	}
			
	@Test //LList INIZIALIZZATO con INS CLOSED
	public void testgetRestIninializedWithInsClosed() {
		LList x = LList.empty().ins(1).ins(2); 
		LList y = LList.empty();
		assertEquals(y,(x.getTail()));
	}

	@Test //LList INIZIALIZZATO con INS OPEN
	public void testgetRestIninializedWithInsOpen() {
		LList r= new LList();
		LList x = r.ins(1).ins(2); 
		assertEquals(r,(x.getTail()));
	}
		
	@Test //LSET INIZIALIZZATO con COSTRUTTORI LIST CLOSED
	public void testgetRestIninializedWithConstructListClosed() {
		List x1 = new ArrayList();
		x1.add(1);x1.add(2);
        LList x = new LList(x1);
		LList y = LList.empty();
		assertEquals(y,(x.getTail()));
	}
	
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST CLOSED
	public void testgetRestIninializedWithConstructLListClosed() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = new LList(x1); 
		LList y = LList.empty();
		assertEquals(y,(x.getTail()));
	}
			
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST OPEN
	public void testgetRestIninializedWithConstructLListOpen() {
		LList r = new LList("r");
		LList x1 = r.ins(1).ins(2);
		LList x = new LList(x1);
		assertEquals(r,(x.getTail()));
	}
	
	@Test //LList INIZIALIZZATO con CLONE
	public void testgetRestIninializedWitClone() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = x1.clone();
		LList y = LList.empty();
		assertEquals(y,(x.getTail()));
	}
		
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISEMPTY	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsEmptyInitializedEmpty() {
		LList x = LList.empty().setName("x"); 
		assertTrue(x.isEmpty());
	}
			
	   @Test //LList NON INIZIALIZZATO
	     public void testIsEmptyUnitialized() {
         LList x = new LList(); 
         try{
	    	   x.isEmpty();
	    	   fail("Eccezione isEmpty per LList non inizializzato");
	             }
	            catch(NotInitLObjectException e){}
	     }

	
			
	@Test //LList INIZIALIZZATO con INS CLOSED
	public void testIsEmptyIninializedWithInsClosed() {
		LList x = LList.empty().ins(1).ins(2); 
		LList y = LList.empty();
		assertFalse(x.isEmpty());
	}

	@Test //LList INIZIALIZZATO con INS OPEN
	public void testIsEmptyIninializedWithInsOpen() {
		LList r= new LList();
		LList x = r.ins(1).ins(2); 
		assertFalse(x.isEmpty());
	}
		
	@Test //LSET INIZIALIZZATO con COSTRUTTORI LIST CLOSED
	public void testIsEmptyIninializedWithConstructListClosed() {
		List x1 = new ArrayList();
		x1.add(1);x1.add(2);
        LList x = new LList(x1);
		LList y = LList.empty();
		assertFalse(x.isEmpty());
	}
	
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST CLOSED
	public void testIsEmptyIninializedWithConstructLListClosed() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = new LList(x1); 
		LList y = LList.empty();
		assertFalse(x.isEmpty());
	}
			
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST OPEN
	public void testIsEmptyIninializedWithConstructLListOpen() {
		LList r = new LList("r");
		LList x1 = r.ins(1).ins(2);
		LList x = new LList(x1);
		assertFalse(x.isEmpty());
	}
	
	@Test //LList INIZIALIZZATO con CLONE
	public void testIsEmptyIninializedWitClone() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = x1.clone();
		LList y = LList.empty();
		assertFalse(x.isEmpty());
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISCLOSED	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsClosedInitializedEmpty() {
		LList x = LList.empty().setName("x"); 
		assertTrue(x.isClosed());
	}
			
	@Test //LList NON INIZIALIZZATO
	public void testIsClosedUnitialized() {
		LList x = new LList(); 
		assertFalse(x.isClosed());
	}
			
	@Test //LList INIZIALIZZATO con INS CLOSED
	public void testIsClosedIninializedWithInsClosed() {
		LList x = LList.empty().ins(1).ins(2); 
		LList y = LList.empty();
		assertTrue(x.isClosed());
	}

	@Test //LList INIZIALIZZATO con INS OPEN
	public void testIsClosedIninializedWithInsOpen() {
		LList r= new LList();
		LList x = r.ins(1).ins(2); 
		assertFalse(x.isClosed());
	}
		
	@Test //LSET INIZIALIZZATO con COSTRUTTORI LIST CLOSED
	public void testIsClosedIninializedWithConstructListClosed() {
		List x1 = new ArrayList();
		x1.add(1);x1.add(2);
        LList x = new LList(x1);
		LList y = LList.empty();
		assertTrue(x.isClosed());
	}
	
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST CLOSED
	public void testIsClosedIninializedWithConstructLListClosed() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = new LList(x1); 
		LList y = LList.empty();
		assertTrue(x.isClosed());
	}
			
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST OPEN
	public void testIsClosedIninializedWithConstructLListOpen() {
		LList r = new LList("r");
		LList x1 = r.ins(1).ins(2);
		LList x = new LList(x1);
		assertFalse(x.isClosed());
	}
	
	@Test //LList INIZIALIZZATO con CLONE
	public void testIsClosedIninializedWitClone() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = x1.clone();
		LList y = LList.empty();
		assertTrue(x.isClosed());
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISGROUND	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsGroundInitializedEmpty() {
		LList x = LList.empty().setName("x"); 
		assertTrue(x.isGround());
	}
			
	@Test //LList NON INIZIALIZZATO
	public void testIsGroundUnitialized() {
		LList x = new LList(); 
		assertFalse(x.isGround());
	}
	
				
	@Test //LList INIZIALIZZATO con INS CLOSED
	public void testIsGroundIninializedWithInsClosed() {
		LList x = LList.empty().ins(1).ins(2); 
		LList y = LList.empty();
		assertTrue(x.isGround());
	}

	@Test //LList INIZIALIZZATO con INS OPEN
	public void testIsGroundIninializedWithInsOpen() {
		LList r= new LList();
		LList x = r.ins(1).ins(2); 
		assertFalse(x.isGround());
	}
		
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LIST CLOSED
	public void testIsGroundIninializedWithConstructListClosed() {
		List x1 = new ArrayList();
		x1.add(1);x1.add(2);
        LList x = new LList(x1);
		LList y = LList.empty();
		assertTrue(x.isGround());
	}
	
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST CLOSED
	public void testIsGroundIninializedWithConstructLListClosed() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = new LList(x1); 
		LList y = LList.empty();
		assertTrue(x.isGround());
	}
			
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST OPEN
	public void testIsGroundIninializedWithConstructLListOpen() {
		LList r = new LList("r");
		LList x1 = r.ins(1).ins(2);
		LList x = new LList(x1);
		assertFalse(x.isGround());
	}
	
	@Test //LList INIZIALIZZATO con CLONE
	public void testIsGroundIninializedWitClone() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = x1.clone();
		LList y = LList.empty();
		assertTrue(x.isGround());
	}
	
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         isBound	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testisBoundInitializedEmpty() {
		LList x = LList.empty().setName("x"); 
		assertTrue(x.isBound());
	}
			
	@Test //LList NON INIZIALIZZATO
	public void testisBoundUnitialized() {
		LList x = new LList(); 
		assertFalse(x.isBound());
	}
			
	@Test //LList INIZIALIZZATO con INS CLOSED
	public void testisBoundIninializedWithInsClosed() {
		LList x = LList.empty().ins(1).ins(2); 
		assertTrue(x.isBound());
	}

	@Test //LList INIZIALIZZATO con INS OPEN
	public void testisBoundIninializedWithInsOpen() {
		LList r= new LList();
		LList x = r.ins(1).ins(2); 
		assertTrue(x.isBound());
	}
		
	@Test //LSET INIZIALIZZATO con COSTRUTTORI LIST CLOSED
	public void testisBoundIninializedWithConstructListClosed() {
		List x1 = new ArrayList();
		x1.add(1);x1.add(2);
        LList x = new LList(x1);
		LList y = LList.empty();
		assertTrue(x.isBound());
	}
	
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST CLOSED
	public void testisBoundIninializedWithConstructLListClosed() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = new LList(x1); 
		LList y = LList.empty();
		assertTrue(x.isBound());
	}
			
	@Test //LLIST INIZIALIZZATO con COSTRUTTORI LLIST OPEN
	public void testisBoundIninializedWithConstructLListOpen() {
		LList r = new LList("r");
		LList x1 = r.ins(1).ins(2);
		LList x = new LList(x1);
		assertTrue(x.isBound());
	}
	
	@Test //LList INIZIALIZZATO con CLONE
	public void testisBoundIninializedWitClone() {
		LList x1 = LList.empty().ins(1).ins(2);
		LList x = x1.clone();
		LList y = LList.empty();
		assertTrue(x.isBound());
	}
	//%%%%%%%%%%%%%%%%%%%%%		
    //         EQUALS(llist)	
	//%%%%%%%%%%%%%%%%%%%%
	@Test // VUOTO --> VUOTO
	public void testEqualsEmpty_LListEmpty() {
		LList x = LList.empty();
		LList y = LList.empty();
		assertTrue(x.equals(y));
	}

	@Test // VUOTO --> LList NON INIZIALIZZATO
	public void testEqualsEmpty_LListUninitilized() {
		LList x = LList.empty();
		LList y = new LList();
		assertFalse(x.equals(y));
	}
		
	@Test // VUOTO --> LList VUOTO, INS INTERO 
	public void testEqualsEmpty_LListEmptyInsInteger() {
		LList x = LList.empty();
		LList y = LList.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> LList VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsEmpty_LListEmptyInsLVarUninitialized() {
		LList x = LList.empty();
		LVar y1= new LVar(); 
		LList y = LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> LList VUOTO, INS LVar INITIALIZEC
	public void testEqualsEmpty_LListEmptyInsLVarInitialized() {
		LList x = LList.empty();
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsEmpty_LSetEmptyInsLVarInitializedLVarUninitializedInteger() {
		LList x = LList.empty();
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test // VUOTO --> RESTO, INS INTERO
	public void testEqualsEmpty_LListEmptyRestInsInteger() {
		LList x = LList.empty();
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}
	
	
	@Test // LList NON INIZIALIZZATO --> VUOTO
	public void testEqualsLListUninitilized_LListEmpty() {
		LList x = new LList();
		LList y = LList.empty();
		assertFalse(x.equals(y));
	}
				
	@Test // LList NON INIZIALIZZATO x --> LList NON INIZIALIZZATO x
	public void testEqualsLListUninitilized_LListUninitilizedSame() {
		LList x = new LList();
		assertTrue(x.equals(x));
	}
		
	@Test // LList NON INIZIALIZZATO x --> LList NON INIZIALIZZATO y
	public void testEqualsLListUninitilized_LListUninitilized() {
		LList x = new LList();
		LList y = new LList();
		assertFalse(x.equals(y));
	}
	@Test //LList NON INIZIALIZZATO --> LListVUOTO, INS INTERO 
	public void testEqualsLListUninitilized_LListEmptyInsInteger() {
		LList x = new LList();
		LList y = LList.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //LListNON INIZIALIZZATO --> LListVUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLSetUninitilized_LListEmptyInsLVarUninitialized() {
		LList x = new LList();
		LVar y1= new LVar(); 
		LList y = LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LList NON INIZIALIZZATO--> LList VUOTO, INS LVar INITIALIZEC
	public void testEqualsLLListUninitilized_LListEmptyInsLVarInitialized() {
		LList x = new LList();
		LVar y1= new LVar("y1",1); 
		LList y =LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
				
	@Test //LList NON INIZIALIZZATO --> LListVUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLSetUninitilized_LListEmptyInsLVarInitializedLVarUninitializedInteger() {
		LList x = new LList();
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LList NON INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsLListUninitilized_LListEmptyRestInsInteger() {
		LList x = new LList();
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}				

	
	@Test //LListVUOTO INS INTERO --> VUOTO
	public void testEqualsLListEmptyInsInteger_LListEmpty() {
		LSet x = LSet.empty().ins(1);
		LSet y = LSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //LList  VUOTO INS INTERO --> LList NON INIZIALIZZATO
	public void testEqualsLListEmptyInsInteger_LListUninitilized() {
		LList x = LList.empty().ins(1);
		LList y = new LList ();
		assertFalse(x.equals(y));
	}
				
	@Test //LList VUOTO INS INTERO --> LList VUOTO, INS INTERO 
	public void testEqualsLListEmptyInsInteger_LListEmptyInsInteger() {
		LList x = LList.empty().ins(1);
		LList y = LList.empty().ins(1);
		assertTrue(x.equals(y));
	}
				
	@Test //LListVUOTO INS INTERO --> LList VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLListEmptyInsInteger_LListEmptyInsLVarUninitialized() {
		LList x = LList.empty().ins(1);
		LVar y1= new LVar(); 
		LList y = LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LList VUOTO  INS INTERO--> LList VUOTO, INS LVar INITIALIZEC
	public void testEqualsLListEmptyInsInteger_LListEmptyInsLVarInitialized() {
		LList x = LList.empty().ins(1);
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(y1);
		assertTrue(x.equals(y));
	}
				
	@Test //LList VUOTO INS INTERO --> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLListEmptyInsInteger_LListEmptyInsLVarInitializedLVarUninitializedInteger() {
		LList x = LList.empty().ins(1);
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test //LList VUOTO INS INTERO  --> RESTO, INS INTERO
	public void testEqualsLListEmptyInsInteger_LListEmptyRestInsInteger() {
		LList x = LList.empty().ins(1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}				
				
	@Test //LList VUOTO INS LVar NON INIZIALIZZATO--> VUOTO
	public void testEqualsLListEmptyInsLVarUninitialized_LListEmpty() {
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1);
		LList y = LList.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //LList VUOTO INS LVar NON INIZIALIZZATO --> LList NON INIZIALIZZATO
	public void testEqualLListEmptyInsLVarUninitialized_LListUninitilized() {
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1);
		LList y = new LList();
		assertFalse(x.equals(y));
	}
				
	@Test //LListVUOTO INS LVar NON INIZIALIZZATO --> LList VUOTO, INS INTERO 
	public void testEqualsLListEmptyInsLVarUninitialized_LListEmptyInsInteger() {
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1);
		LList y =LList.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //LList VUOTO INS LVar NON INIZIALIZZATO --> LListVUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLListEmptyInsLVarUninitialized_LListEmptyInsLVarUninitialized() {
		LVar x1 = new LVar();
		LList x =LList.empty().ins(x1);
		LVar y1= new LVar(); 
		LList y = LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LList VUOTO INS LVar NON INIZIALIZZATO--> LList VUOTO, INS LVar INITIALIZEC
	public void testEqualsLListEmptyInsLVarUninitialized_LListEmptyInsLVarInitialized() {
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1);
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
				
	@Test //LList VUOTO INS LVar NON INIZIALIZZATO--> LListVUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLListEmptyInsLVarUninitialized_LListEmptyInsLVarInitializedLVarUninitializedInteger() {
		LVar x1 = new LVar();
		LList x =LList.empty().ins(x1);
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test //LList VUOTO INS LVar NON INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsLListEmptyInsLVarUninitialized_LListEmptyRestInsInteger() {
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}				
	
	@Test //LList VUOTO INS LVar INIZIALIZZATO--> VUOTO
	public void testEqualsLListEmptyInsLVarInitialized_LListEmpty() {
		LVar x1 = new LVar("X1",1);
		LList x =LList.empty().ins(x1);
		LList y = LList.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //LListVUOTO INS LVar INIZIALIZZATO --> LList NON INIZIALIZZATO
	public void testEqualsLListEmptyInsLVarInitialized_LListUninitilized() {
		LVar x1 = new LVar("X1",1);
		LList x = LList.empty().ins(x1);
		LList y = new LList();
		assertFalse(x.equals(y));
	}
				
	@Test //LList VUOTO INS LVar INIZIALIZZATO --> LList VUOTO, INS INTERO 
	public void LListEqualsLSetEmptyInsLVarInitialized_LListEmptyInsInteger() {
		LVar x1 = new LVar("X1",1);
		LList x = LList.empty().ins(x1);
		LList y = LList.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //LLListVUOTO INS LVar INIZIALIZZATO --> LList VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLListEmptyInsLVarInitialized_LListEmptyInsLVarUninitialized() {
		LVar x1 = new LVar("X1",1);
		LList x = LList.empty().ins(x1);
		LVar y1= new LVar(); 
		LList y = LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LList VUOTO INS LVar  INIZIALIZZATO--> LList VUOTO, INS LVar INITIALIZEC
	public void testEqualsLListEmptyInsLVarInitialized_LListEmptyInsLVarInitialized() {
		LVar x1 = new LVar("X1",1);
		LList x = LList.empty().ins(x1);
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(y1);
		assertTrue(x.equals(y));
	}
				
				
	@Test //LList VUOTO INS LVar  INIZIALIZZATO--> LList VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLListEmptyInsLVarInitialized_LListEmptyInsLVarInitializedLVarUninitializedInteger() {
		LVar x1 = new LVar("X1",1);
		LList x = LList.empty().ins(x1);
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LList y = LList.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test //LList VUOTO INS LVar  INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsLListEmptyInsLVarInitialized_LListEmptyRestInsInteger() {
		LVar x1 = new LVar("X1",1);
		LList x = LList.empty().ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}	
				
				
	@Test //LList VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO--> VUOTO
		public void testEqualsLListEmptyInsLVarInitializedLVarUninitializedInteger_LListEmpty() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList y = LList.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO --> LList NON INIZIALIZZATO
	public void testEqualsLListEmptyInsLVarInitializedLVarUninitializedInteger_LListUninitilized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList y = new LList();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO --> LList VUOTO, INS INTERO 
	public void testEqualsLListEmptyInsLVarInitializedLVarUninitializedInteger_LListEmptyInsInteger() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList y =LList.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO --> LList VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLSetEmptyInsLVarInitializedLVarUninitializedInteger_LListEmptyInsLVarUninitialized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LVar y1= new LVar(); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LList VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO --> LListVUOTO, INS LVar INITIALIZEC
		public void testEqualsLListEmptyInsLVarInitializedLVarUninitializedInteger_LListEmptyInsLVarInitialized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LVar y1= new LVar("y1",1); 
		LList y =LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
		
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO--> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLListEmptyInsLVarInitializedLVarUninitializedInteger_LListEmptyInsLVarInitializedLVarUninitializedInteger() {
		LVar x0= new LVar("x0");
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LVar y0= new LVar("y0");
		LVar y1= new LVar("y1",2); 
		LList y = LList.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test //LList VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO  --> RESTO, INS INTERO
	public void testEqualsLListEmptyInsLVarInitializedLVarUninitializedInteger_LListEmptyRestInsInteger() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",1); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}	
	
	
	@Test //RESTO, INS INTERO--> VUOTO
	public void testEqualsLListEmptyRestInsInteger_LListEmpty() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LList NON INIZIALIZZATO
	public void testEqualsLListEmptyRestInsInteger_LListUninitilized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LList VUOTO, INS INTERO 
	public void testEqualsLListEmptyRestInsInteger_LListEmptyInsInteger() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LList VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLListEmptyRestInsInteger_LListEmptyInsLVarUninitialized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LList VUOTO, INS LVar INITIALIZEC
	public void testEqualsLListEmptyRestInsInteger_LListEmptyInsLVarInitialized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
				
	@Test //RESTO, INS INTERO--> LList VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLListEmptyRestInsInteger_LListEmptyInsLVarInitializedLVarUninitializedInteger() {
		LVar x0= new LVar("x0");
		LVar x1= new LVar("x1",2); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}
				

	@Test //RESTO, INS INTERO  --> RESTO, INS INTERO
	public void testEqualsLListEmptyRestInsInteger_LListEmptyRestInsInteger() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",1); 
		LList x = LList.empty().ins(1).ins(x0).ins(x1);
		LList r= new LList("r");
		LList y = r.ins(1);
		assertFalse(x.equals(y));
	}

	//%%%%%%%%%%%%%%%%%%%%%		
    //         EQUALS(object)	
	//%%%%%%%%%%%%%%%%%%%%
	
	// LList NON INIZIALIZZATO ---> LIST NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLListUninitialized_SetUninitialized() {
		LList x = new LList();
		List s = new ArrayList();
        assertFalse(x.equals(s));
	}
	
	// LListNON INIZIALIZZATO ---> LIST INIZIALIZZATO
	public void testEqualsObjectLListUninitialized_SetInitialized() {
		LList x = new LList();
		List s = new ArrayList();
		s.add(2);s.add(3);
        assertFalse(x.equals(s));
	}
	
	// LList NON INIZIALIZZATO ---> INTERO
	public void testEqualsObjectLListUninitialized_Integer() {
		LList x = new LList();
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LList NON INIZIALIZZATO ---> ARRAY
	public void testEqualsObjectLListUninitialized_Array() {
		LList x = new LList();
		Integer[] y= {4,3};
		assertFalse(x.equals(y));
	}
	

	// LList VUOTO ---> LIST NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLListEmpty_HashSetUninitialized() {
		LList x = LList.empty();
		List s = new ArrayList();
        assertTrue(x.equals(s));
	}
	
	// LList VUOTO ---> LIST INIZIALIZZATO
	public void testEqualsObjectLListEmpty_HashSetInitialized() {
		LList x = LList.empty();
		List s = new ArrayList();
		s.add(2);s.add(3);
        assertFalse(x.equals(s));
	}
	
	// LList VUOTO ---> INTERO
	public void testEqualsObjectLListEmpty_Integer() {
		LList x = LList.empty();
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LList VUOTO ---> ARRAY
	public void testEqualsObjectLListEmpty_Array() {
		LList x = LList.empty();
		Integer[] y= {4,3};
		assertFalse(x.equals(y));
	}
	
	// LList VUOTO INS INTERO ---> LIST NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLListInitializedWithInsInteger_HashSetUninitialized() {
		LList x = LList.empty().ins(1).ins(2);
		List s = new ArrayList();
		assertFalse(x.equals(s));
	}
	
	//LList VUOTO INS INTERO ---> LIST INIZIALIZZATO 
	public void testEqualsObjectLListInitializedWithInsInteger_HashSetInitializedSame() {
		LList x = LList.empty().ins(1).ins(2);
		List s = new ArrayList();
		s.add(1);s.add(2);
        assertFalse(x.equals(s));
	}
	
	// LList VUOTO INS INTERO---> INTERO
	public void testEqualsObjectLListInitializedWithInsInteger_Integer() {
		LList x = LList.empty().ins(1).ins(2);
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LSET VUOTO INS INTERO ---> ARRAY 
	public void testEqualsObjectLListInitializedWithInsInteger_ArraySame() {
		LList x = LList.empty().ins(1).ins(2);
		Integer[] y= {1,2};
		assertFalse(x.equals(y));
	}

	// LList INIZIALIZZATO INS con LVar INIZIALIZZATO ---> LIST NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLListInitializedWithLVarInsInitialized_HashSetUninitialized() {
		LVar x1 = new LVar("x1",1);
		LList x = LList.empty().ins(x1).setName("x");
		List s = new ArrayList();
		assertFalse(x.equals(s));
	}
	// LList INIZIALIZZATO INS con LVar INIZIALIZZATO ---> LIST INIZIALIZZATO
	@Test
	public void testEqualsObjectLListInitializedWithLVarInsInitialized_HashSetInitialized() {
		LVar x1 = new LVar("x1",1);
		LList x = LList.empty().ins(x1).setName("x");
		List s = new ArrayList();
	    s.add(1);
		assertTrue(x.equals(s));
	}
	
	// LList INIZIALIZZATO INS con LVar INIZIALIZZATO---> INTERO
	public void testEqualsObjectLListInitializedWithInsLVarInitialized_Integer() {
		LVar x1 = new LVar("x1",1);
		LList x = LList.empty().ins(x1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	
	// LList INIZIALIZZATO INS con LVar INIZIALIZZATO---> ARRAY UGUALE
	public void testEqualsObjectLListInitializedWithInsLVarInitialized_ArraySame() {
		LVar x1 = new LVar("x1",1);
		LList x = LList.empty().ins(x1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}
	
	// LList INIZIALIZZATO INS con LVar NON INIZIALIZZATO ---> LIST NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLListInitializedWithInsLVarUnitialized_SetUninitialized() {
		LVar x1 = new LVar("x1");
		LList x =LList.empty().ins(x1).setName("x");
		List s = new ArrayList();
		assertFalse(x.equals(s));
	}
	
	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO ---> LIST INIZIALIZZATO
	@Test
	public void testEqualsObjectLListInitializedWithInsLVarUnitialized_SetInitialized() {
		LVar x1 = new LVar("x1");
		LList x = LList.empty().ins(x1).setName("x");
		List s = new ArrayList();
	    s.add(1);
		assertFalse(x.equals(s));
	}
	
	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO---> INTERO
	public void testEqualsObjectLListInitializedWithInsLVarUnitialized_Integer() {
		LVar x1 = new LVar("x1");
		LList x = LList.empty().ins(x1).setName("x");
	    Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LList INIZIALIZZATO INS con LVar NON INIZIALIZZATO---> ARRAY 
	public void testEqualsObjectLListInitializedWithInsLVarUnitialized_Array() {
		LVar x1 = new LVar("x1");
		LList x = LList.empty().ins(x1).setName("x");
	    Integer[] y= {1,2};
		assertFalse(x.equals(y));
	}
	

	

	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO, LVar INIZIALIZZATO, INTERO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLListInitializedWithInsLVarUnitializedLVarInitializedInteger_SetUninitialized() {
		LVar x0 = new LVar("x0",2);
		LVar x1 = new LVar("x1");
		LList x = LList.empty().ins(x0).ins(x1).ins(1).setName("x");
		List s = new ArrayList();
		assertFalse(x.equals(s));
	}
	
	// LListINIZIALIZZATO INS con LVar NON INIZIALIZZATO, LVar INIZIALIZZATO, INTERO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectLListInitializedWithInsLVarUnitializedLVarInitializedInteger_SetInitialized() {
		LVar x0 = new LVar("x0",2);
		LVar x1 = new LVar("x1");
		LList x = LList.empty().ins(x0).ins(x1).ins(1).setName("x");
		List s = new ArrayList();
	    s.add(1);
		assertFalse(x.equals(s));
	}
	
	// LList INIZIALIZZATO INS con LVar NON INIZIALIZZATO, LVar INIZIALIZZATO, INTERO---> INTERO
	public void testEqualsObjectLListInitializedWithInsLVarUnitializedLVarInitializedInteger_Integer() {
		LVar x0 = new LVar("x0",2);
		LVar x1 = new LVar("x1");
		LList x = LList.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LList INIZIALIZZATO INS con LVar NON INIZIALIZZATO, LVar INIZIALIZZATO, INTERO---> ARRAY UGUALE
	public void testEqualsObjectLListInitializedWithInsLVarUnitializedLVarInitializedInteger_ArraySame() {
		LVar x0 = new LVar("x0",2);
		LVar x1 = new LVar("x1");
		LList x =LList.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Integer[] y= {1,2};
		assertFalse(x.equals(y));
	}
	
	// LList NON INIZIALIZZATO INS INTERO ---> LIST NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLListUninitializedInsInteger_SetUninitialized() {
		LList r = new LList ("r");
		LList x = r.ins(1).setName("x");
		List s = new ArrayList();
		assertFalse(x.equals(s));
	}
	
	// LList NON INIZIALIZZATO INS INTERO ---> LIST INIZIALIZZATO
	@Test
	public void testEqualsObjectLListUninitializedInsInteger_SetInitialized() {
		LList r = new LList ("r");
		LList x = r.ins(1).setName("x");
		List s = new ArrayList();
	    s.add(1);
		assertTrue(x.equals(s));
	}
	
	// LList NON INIZIALIZZATO INS INTERO---> INTERO
	public void testEqualsObjectLListUninitializedInsInteger_Integer() {
		LList r = new LList ("r");
		LList x = r.ins(1).setName("x");
	    Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LLList NON INIZIALIZZATO INS INTERO ---> ARRAY 
	public void testEqualsObjectLListUninitializedInsInteger_Array() {
		LList r = new LList("r");
		LList x = r.ins(1).setName("x");
	    Integer[] y= {1,2};
		assertFalse(x.equals(y));
	}

	public void testIteratorLListUnbound(){
		LList ll = new LList();
		for(Object o: ll)
			fail();
	}

	@Test
	public void testOccurCheck01(){
		LList tail = new LList();
		LVar x = new LVar("x");
		LVar y = new LVar("y");
		LVar z = new LVar(y);
		LList1 lList = new LList1(tail.ins(x).ins(y));
		assertFalse(lList.occurs(lList));
		assertTrue(lList.occurs(tail));
		assertTrue(lList.occurs(x));
		assertTrue(lList.occurs(y));
		assertTrue(lList.occurs(z));
		x.setValue(lList);
		assertTrue(lList.occurs(lList));

		LVar lVar = new LVar();

		assertFalse(lList.occurs(lVar));

		LList lList2 = new LList();
		assertFalse(lList.occurs(lList2));
	}

	@Test
	public void testOccurCheck02(){
		LList tail = new LList();
		LVar x = new LVar("x");
		LVar y = new LVar("y", x);
		LVar z = new LVar("z");

		LList1 lList = new LList1(tail.ins(y).ins(z));
		assertTrue(lList.occurs(x));
		assertTrue(lList.occurs(y));
		assertTrue(lList.occurs(z));
		assertTrue(lList.occurs(tail));
		assertFalse(lList.occurs(lList));

		LSet k = new LSet("k");
		LList nestedLList = new LList().ins(1).ins(k).ins(LList.empty().ins(1).ins(lList).ins(2));
		y.setValue(nestedLList);
		assertTrue(lList.occurs(x));
		assertTrue(lList.occurs(y));
		assertTrue(lList.occurs(z));
		assertTrue(lList.occurs(tail));
		assertTrue(lList.occurs(k));
		assertTrue(lList.occurs(1));
		assertTrue(lList.occurs(2));
		assertTrue(lList.occurs(lList));
	}


		

}
