package jsetl.test;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import junit.framework.TestCase;

import jsetl.*;

import jsetl.LVar;

/**
 * @author Lucia Guglielmetti
 *
 */
public class LVarTest extends TestCase {

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}

	/**
	 * Test method for {@link jsetl.LVar#getName()}.
	 */
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETNAME	
	//%%%%%%%%%%%%%%%%%%%%
	@Test
	public void testGetNameWithName() {
		LVar a = new LVar("x");
		assertTrue("x".equals(a.getName()));
	}

	@Test
	public void testGetNameWithNoName() {
		LVar a = new LVar(1);
		assertTrue("?".equals(a.getName()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         SETNAME	
	//%%%%%%%%%%%%%%%%%%%%
	@Test
	public void testSetNameWithName() {
		LVar x = new LVar("x"); 
        x.setName("x_new");
        assertTrue("x_new".equals(x.getName()));
	}

	@Test
	public void testSetNameWithNotName() {
		LVar x = new LVar(1);
        x.setName("x_new");
        assertTrue("x_new".equals(x.getName()));
	}

	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETVALUE	
	//%%%%%%%%%%%%%%%%%%%%
	//LVar NON INIZIALIZZATO
	@Test
	public void testGetValueUninitialized() {
		 LVar x = new LVar();
		 assertEquals(null,(x.getValue()));
	}
	
	//LVar INIZIALIZZATO CON TIPO PRIMITIVO PRECISAMENTE INTERO
	@Test
	public void testGetValueInitialized() {
		LVar x = new LVar("x",1);
		assertEquals(1,(x.getValue()));
	}

	//LVar INIZIALIZZATO CON OGGETTO PRECISAMENTE STRINGA
	@Test
	public void testGetValueInitializedWithString() {
		 String x1 = "Stringa";
		 LVar x = new LVar("x",x1); 
	     assertEquals(x1,(x.getValue()));
	}

	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO
	@Test
	public void testGetValueInitializedWithLVarInitialized() {
		 LVar y = new LVar("y",3);
	     LVar x = new LVar(y); 
	     assertEquals(3,(x.getValue()));
	}
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO
	@Test
	public void testGetValueInitializedWithLVarUninitialized() {
		 LVar y = new LVar();
	     LVar x = new LVar(y); 
	     assertEquals(null,(x.getValue()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         TOSTRING	
	//%%%%%%%%%%%%%%%%%%%%
	//LVar NON INIZIALIZZATO
	@Test
	public void testToStringUninitialized() {
		 LVar x = new LVar("x");
		 assertEquals("_x",(x.toString()));
	}
	
	//LVar INIZIALIZZATO CON TIPO PRIMITIVO PRECISAMENTE INTERO
	@Test
	public void testToStringInitialized() {
		LVar x = new LVar("x",1);
		assertEquals("1",(x.toString()));
	}
	

	//LVar INIZIALIZZATO CON OGGETTO PRECISAMENTE STRINGA
	@Test
	public void testToStringInitializedWithString() {
		 String x1 = "Stringa";
		 LVar x = new LVar("x",x1); 
	     assertEquals(x1,(x.toString()));
	}
	
	//LVar INIZIALIZZATO CON LVar INIZIALIZZATA
	@Test
	public void testToStringInitializedWithLVarInitialized() {
		 LVar y = new LVar("y",1);
	     LVar x = new LVar(y); 
	     assertEquals("1",(x.toString()));
	}
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATA
	@Test
	public void testToStringInitializedWithLVarUninitialized() {
		 LVar y = new LVar();
	     LVar x = new LVar(y); 
	     assertEquals("_?",(x.toString()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         isBound	
	//%%%%%%%%%%%%%%%%%%%%
	//LVar INIZIALIZZATA
	@Test
	public void testisBoundInitialized() {
		LVar x = new LVar("x",1);
		assertTrue(x.isBound());
	}
		
	//LVar NON INIZIALIZZATA
	@Test
	public void testisBoundUninitialized() {
		 LVar x = new LVar("x");
		 assertFalse(x.isBound());
	}
		
	//LVar CLONATA
	@Test
	public void testisBoundClone() {
		 LVar x1 = new LVar("x1",1);
		 LVar x = x1.clone();
	     assertTrue(x.isBound());
	}
		
	//%%%%%%%%%%%%%%%%%%%%%		
    //         EQUALS(lvar)	
	//%%%%%%%%%%%%%%%%%%%%

	// LVar NON INIZIALIZZATA x --> LVar NON INIZIALIZZATA x
	@Test
	public void testEqualsLVarUninitialized_SameLVarUninitialized() {
		LVar x = new LVar("x");  
		assertTrue(x.equals(x));
	}
	
	// LVar NON INIZIALIZZATA x --> LVar NON INIZIALIZZATA y
	@Test
	public void testEqualsLVarUninitialized_LVarUninitialized() {
		LVar x = new LVar("x");  
		LVar y = new LVar("y");  
		assertFalse(x.equals(y));
	}
	
	// LVar NON INIZIALIZZATA --> LVar INIZIALIZZATA CON INTERO
	@Test
	public void testEqualsLVarUninitialized_LVarInitialized() {
		LVar x = new LVar("x");  
		LVar y = new LVar("y",1);
		assertFalse(x.equals(y));
	}

	// LVar NON INIZIALIZZATA --> LVar INIZIALIZZATO STRINGA
	@Test
	public void testEqualsLVarUninitialized_LVarInitializedWithString() {
		LVar x = new LVar("x");    
		String y1 = "Prova";
		LVar y = new LVar("y",y1); 
		assertFalse(x.equals(y));
	}
	
	// LVar NON INIZIALIZZATA --> LVar INIZIALIZZATO CON LVar INIZIALIZZATO
	@Test
	public void testEqualsLVarUninitialized_LVarInitializedWithLVarInitialized() {
		LVar x = new LVar("x");    
		LVar y1 = new LVar ("y1",1);
		LVar y = new LVar("y",y1); 
		assertFalse(x.equals(y));
	}
	
	// LVar NON INIZIALIZZATA --> LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarUninitialized_LVarInitializedWithLVarUninitialized() {
		LVar x = new LVar("x");   
		LVar y1 = new LVar ("y1");
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}
	
	
	//LVar INIZIALIZZATA CON INTERO --> LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarInitialized_LVarUninitialized() {
		LVar x = new LVar("x",1);   
		LVar y = new LVar("y");   
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATA CON INTERO --> LVar INIZIALIZZATO CON INTERO 
	@Test
	public void testEqualsLVarInitialized_LVarInitialized() {
		LVar x = new LVar("x",1);
		LVar y = new LVar("y",1); 
		assertTrue(x.equals(y));
	}
	
	//LVar INIZIALIZZATA CON INTERO --> LVar INIZIALIZZATO CON STRINGA 
	@Test
	public void testEqualsLVarInitialized_LVarInitializedWithString() {
		LVar x = new LVar("x",1);
		String y1 = "Prova";   
		LVar y = new LVar("y",y1);   
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATA CON INTERO --> LVar INIZIALIZZATO CON LVar INIZIALIZZATO
	@Test
	public void testEqualsLVarInitialized_LVarInitializedWithLVarInitialized() {
		LVar x = new LVar("x",1);
		LVar y1 = new LVar ("y1",1);
		LVar y = new LVar("y",y1);
		assertTrue(x.equals(y));
	}
	
	//LVar INIZIALIZZATA CON INTERO --> LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithInteger_LVarInitializedWithLVarUninitialized() {
		LVar x = new LVar("x",1); 
		LVar y1 = new LVar ("y1");
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}
	
		
	//LVar INIZIALIZZATO CON STRINGA --> LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithString_LVarUninitialized() {
		String x1 = "Prova";
		LVar x = new LVar("x",x1); 
		LVar y = new LVar ("y");
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON STRINGA --> LVar INIZIALIZZATO CON INTERO
	@Test
	public void testEqualsLVarInitializedWithString_LVarInitialized() {
		String x1 = "Prova";
		LVar x = new LVar("x",x1); 
		LVar y = new LVar("y",1);
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON STRINGA --> LVar INIZIALIZZATO CON STRINGA
	@Test
	public void testEqualsLVarInitializedWithString_LVarInitializedWithString() {
		String x1 = "Prova";
		LVar x = new LVar("x",x1); 
		String y1 = "Prova";
		LVar y = new LVar("y",y1);
		assertTrue(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON STRINGA --> LVar INIZIALIZZATO CON LVar INIZIALIZZATO 
	@Test
	public void testEqualsLVarInitializedWithString_LVarInitializedWithLVarInitialized() {
		String x1 = "Prova";
		LVar x = new LVar("x",x1); 
		LVar y1 = new LVar ("y1",1);
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}
	//LVar INIZIALIZZATO CON STRINGA --> LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithString_LVarInitializedWithLVarUninitialized() {
		String x1 = "Prova";
		LVar x = new LVar("x",x1); 
		LVar y1 = new LVar ("y1");
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}
	
		
	
	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO --> LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithLVarInitialized_LVarUninitialized() {
		LVar x1 = new LVar("x1",1);
		LVar x = new LVar("x",x1); 
		LVar y = new LVar ("y");
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO --> LVar INIZIALIZZATO CON INTERO 
	@Test
	public void testEqualsLVarInitializedWithLVarInitialized_LVarInitialized() {
		LVar x1 = new LVar("x1",1);
		LVar x = new LVar("x",x1); 
		LVar y = new LVar ("y",1);
		assertTrue(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO --> LVar INIZIALIZZATO CON STRING 
	@Test
	public void testEqualsLVarInitializedWithLVarInitialized_LVarInitializedWithString() {
		LVar x1 = new LVar("x1",1);
		LVar x = new LVar("x",x1); 
		String y1 = "Prova";
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}

	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO --> LVar INIZIALIZZATO CON LVar INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithLVarInitialized_LVarInitializedWithLVarInitialized() {
		LVar x1 = new LVar("x1",1);
		LVar x = new LVar("x",x1); 
		LVar y1 = new LVar ("y1",1);
		LVar y = new LVar("y",y1);
		assertTrue(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO --> LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithLVarInitialized_LVarInitializedWithLVarUninitialized() {
		LVar x1 = new LVar("x1",1);
		LVar x = new LVar("x",x1); 
		LVar y1 = new LVar ("y1");
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}
	
		
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO -->LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithLVarUninitialized_LVarUninitialized() {
		LVar x1 = new LVar("x1");
		LVar x = new LVar("x",x1); 
		LVar y = new LVar("y");   
		assertFalse(x.equals(y));
	}
	
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO -->LVar  INIZIALIZZATA CON INTERO
	@Test
	public void testEqualsLVarInitializedWithLVarUninitialized_LVarInitializedWithInteger() {
		LVar x1 = new LVar("x1");
		LVar x = new LVar("x",x1); 
		LVar y = new LVar("y",1); 
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO -->LVar  INIZIALIZZATO STRING
	@Test
	public void testEqualsLVarInitializedWithLVarUninitialized_LVarInitializedWithString() {
		LVar x1 = new LVar("x1");
		LVar x = new LVar("x",x1); 
		String y1 = "Prova";
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO -->LVar  INIZIALIZZATO CON LVar  INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithLVarUninitialized_LVarInitializedWithLVarInitialized() {
		LVar x1 = new LVar("x1");
		LVar x = new LVar("x",x1); 
		LVar y1 = new LVar ("y1",1);
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO -->LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO
	@Test
	public void testEqualsLVarInitializedWithLVarUninitialized_LVarInitializedWithLVarUninitialized() {
		LVar x1 = new LVar("x1");
		LVar x = new LVar("x",x1); 
		LVar y1 = new LVar ("y1");
		LVar y = new LVar("y",y1);
		assertFalse(x.equals(y));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         EQUALS(object)
	//%%%%%%%%%%%%%%%%%%%%
	//LVar NON INIZIALIZZATO --> INTERO
	@Test
	public void testEqualsObjectLVarUninitialized_Integer() {
		LVar x = new LVar("x");  
    	Integer k=5;
    	assertFalse(x.equals(k));
	}
	
	//LVar NON INIZIALIZZATO --> LSET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarUninitialized_LSetUninitialized() {
		LVar x = new LVar("x");    
		LSet y = new LSet();
    	assertFalse(x.equals(y));
	}
	
	//LVar NON INIZIALIZZATO --> LSET INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarUninitialized_LSetInitialized() {
		LVar x = new LVar("x");   
		LSet y = LSet.empty().ins(1).ins(2).setName("s");
    	assertFalse(x.equals(y));
	}
	
	//&%%&%&%&%&%&%&%&%&%&&%&%&%&
	//LVar INIZIALIZZATO CON INTERO-->  INTERO 
	@Test
	public void testEqualsObjectLVarInitialized_Integer() {
		LVar x = new LVar("x",5);  
    	Integer k=5;
    	assertTrue(x.equals(k));
	}
	
	//LVar INIZIALIZZATO CON INTERO --> LSET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarInitialized_LSetUninitialized() {
		LVar x = new LVar("x",5);
		LSet y = new LSet();
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON INTERO --> LSET INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarInitializedWithInteger_LSetInitialized() {
		LVar x = new LVar("x",5);
		LSet y = LSet.empty().ins(5).setName("y");
		assertFalse(x.equals(y));
	}
	//&%&%&%&%&&%&%&%&%&%&%&%&%&%

	//LVar INIZIALIZZATO CON STRINGA --> INTERO 
	@Test
	public void testEqualsObjectLVarInitializedWithString_Integer() {
		String x1 = "Prova";
		LVar x = new LVar("x",x1);
    	Integer k=3;
    	assertFalse(x.equals(k));
	}
	
	//LVar INIZIALIZZATO CON STRINGA --> LSET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarInitializedWithString_LSetUninitialized() {
		String x1 = "Prova";
		LVar x = new LVar("x",x1);
    	LSet y = new LSet();
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON STRINGA --> LSET INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarInitializedWithString_LSetInitialized() {
		String x1 = "Prova";
		LVar x = new LVar("x",x1);
    	LSet y = LSet.empty().ins(2);
		assertFalse(x.equals(y));
	}
	
	//&%&%&%&%&%&%&%&%&%&%&%
	

	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO --> INTERO 
	@Test
	public void testEqualsObjectLVarIntializedWithLVarInitialized_Integer() {
		LVar x1 = new LVar("x1",5);
		LVar x = new LVar("x1",x1);
    	Integer k=5;
    	assertTrue(x.equals(k));
	}


	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO-->LSET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarInitializedWithLVarInitialized_LSetUninitialized() {
		LVar x1 = new LVar("x1",3);
		LVar x = new LVar("x",x1);
    	LSet y = new LSet();
		assertFalse(x.equals(y));
	}
	//LVar INIZIALIZZATO CON LVar INIZIALIZZATO-->LSET  INIZIALIZZATO 
	@Test
	public void testEqualsObjectLVarInitializedWithLVarInitialized_LSetInitialized() {
		LVar x1 = new LVar("x1",3);
		LVar x = new LVar("x",x1);
		LSet y = LSet.empty().ins(3).setName("y");
		assertFalse(x.equals(y));
	}

   //&%%&%&%&%&%&%&%&%&%&%&%&%&%&%%&&%&%&%&%%&&%&%
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO --> INTERO 
	@Test
	public void testEqualsObjectLVarUnitializedWithLVarInitialized_Integer() {
		LVar x = new LVar("x");
    	Integer k=5;
    	assertFalse(x.equals(k));
	}
	
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO --> LSET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarInitializedWithLVarUninitialized_LSetUninitialized() {
		LVar x1 = new LVar("x1");
		LVar x = new LVar("x",x1);
    	LSet y = new LSet();
		assertFalse(x.equals(y));
	}
	
	//LVar INIZIALIZZATO CON LVar NON INIZIALIZZATO --> LSET INIZIALIZZATO
	@Test
	public void testEqualsObjectLVarInitializedWithLVarUninitialized_LSetInitialized() {
		LVar x1 = new LVar("x1");
		LVar x = new LVar("x",x1);
		LSet y = LSet.empty().ins(1).ins(2).setName("s");
		assertFalse(x.equals(y));
	}

	@Test
	public void testNameNoName(){
		LObject.USEJOLLYTOKEN = false;
		LVar x = new LVar();
		LVar y = new LVar();
		LVar z = new LVar();

		assertTrue(x.getName().startsWith("N"));
		assertTrue(y.getName().startsWith("N"));
		assertTrue(z.getName().startsWith("N"));

		int idx = Integer.parseInt(x.getName().substring(1));
		int idy = Integer.parseInt(y.getName().substring(1));
		int idz = Integer.parseInt(z.getName().substring(1));

		assertTrue(idy == idx + 1 && idz == idy + 1);


	}
}
