package jsetl.test;

import java.util.*;

import jsetl.*;

import org.junit.Test;
import junit.framework.TestCase;

public class MultiIntervalTest extends TestCase {
	
	@Test
	public void
	testConvexMultiIntervals() {
		MultiInterval x = new MultiInterval(1,10);
        MultiInterval y = new MultiInterval(5);
        MultiInterval z = MultiInterval.universe();
        MultiInterval w = x.clone();
        assertEquals(z, new MultiInterval(Interval.INF, 
        								  Interval.SUP));
        assertEquals(w, x);
        assertEquals(x.size(), 10);       
        assertEquals(y.size(), 1);       
        assertTrue(x.contains(5));       
        assertFalse(x.contains(11));       
        assertEquals(x.getGlb() + x.getLub(), 11);
        
        x = x.intersect(new MultiInterval(4, 8));
        MultiInterval v = new MultiInterval(0,5); 
        assertEquals(x.intersect(v), new MultiInterval(4, 5));       
        MultiInterval v1 = new MultiInterval(7,9);
        assertEquals(x.intersect(v1), new MultiInterval(7, 8));       
        MultiInterval v2 = new MultiInterval(5,7);
        assertEquals(x.intersect(v2), v2);       
        MultiInterval v3 = new MultiInterval(1,10);
        assertEquals(x.intersect(v3), new MultiInterval(4, 8));       
        MultiInterval v4 = new MultiInterval(10,20);
        assertEquals(x.intersect(v4), new MultiInterval());
		
        Iterator<Integer> x_it = x.iterator(); 
		while(x_it.hasNext()) {
			int next = x_it.next();
			assertTrue(4 <= next && next <= 8);
		}
		
		Iterator<Integer> y_it = y.iterator(); 
		while(y_it.hasNext())  
			assertEquals((int) y_it.next(), 5);
		
		Iterator<Integer> w_it = w.iterator();
		while(w_it.hasNext()) {
			int next = w_it.next();
			assertTrue(1 <= next && next <= 10);
		} 
        
		Iterator<Integer> v_it = x.intersect(v).iterator(); 
		while(v_it.hasNext()) {
			int next = v_it.next();
			assertTrue(4 <= next && next <= 5);
		}     
        
		Iterator<Integer> v1_it = x.intersect(v1).iterator(); 
		while(v1_it.hasNext()) {
			int next = v1_it.next();
			assertTrue(7 <= next && next <= 8);
		}    
        
		Iterator<Integer> v2_it = x.intersect(v2).iterator(); 
		while(v2_it.hasNext()) {
			int next = v2_it.next();
			assertTrue(5 <= next && next <= 7);
		}    
        
		Iterator<Integer> v3_it = x.intersect(v3).iterator(); 
		while(v3_it.hasNext()) {
			int next = v3_it.next();
			assertTrue(4 <= next && next <= 8);
		}
		
		MultiInterval i = new MultiInterval(1, 0);
		assertTrue(!i.contains(0) && i.contains(new Interval()));
		assertNull(i.getGlb());
	}
	
	@Test
	public void 
	testUtilityMethods() {
		List<Interval> l = new LinkedList<Interval>();
    	l.add(new Interval(-1, 0));
        l.add(new Interval(10, 13));
        l.add(new Interval(4, 7));
        l.add(new Interval(6, 11));
        l.add(new Interval(15, 18));
        l.add(new Interval(2, 5));
        l.add(new Interval(16, 17));
        MultiInterval mi = new MultiInterval(l);
        assertFalse(mi.contains(1));
        assertFalse(mi.contains(14));
        assertEquals(mi.convexClosure(), new MultiInterval(-1, 18));
        assertEquals(mi.size(), 18);
        assertFalse(mi.contains(1));
        assertTrue(mi.contains(new Interval(4, 9)));
        
        mi.add(25);
        mi.remove(8);
        assertFalse(mi.contains(20));
        assertFalse(mi.contains(8));
        assertEquals(mi.size(), 18);
        l.clear();
        l.add(new Interval(100, 200));
        l.add(new Interval(15, 18));
        l.add(new Interval(-2, 3));
        l.add(new Interval(10, 12));
        l.add(new Interval(-10, -5));
        l.add(new Interval(25, 30));
        MultiInterval mi1 = new MultiInterval(l);
        assertEquals(mi1.size(), 126);
        MultiInterval mi2 = mi.intersect(mi1);
        assertEquals(mi2.size(), 12);
        assertEquals(mi2.intersect(mi), mi2.intersect(mi1));
        
        mi.clear();
        assertTrue(mi.isEmpty());
        mi2 = mi.intersect(mi1);
        assertTrue(mi2.isEmpty());
        List<Integer> l1 = new LinkedList<Integer>();
        int i;
        for (i = -5; i < 6; i += 2)
        	l1.add(i);
        mi.addAll(l1);
        for (i = -4; i < 5; i += 2)
        	assertFalse(mi.contains(i));
        assertTrue(mi.contains((Object)3));
        assertFalse(mi.contains((Object)new Interval(-1, 1)));
        mi1.removeAll(l1);
        assertEquals(mi1.size(), 122);
        Object[] l2 = mi.toArray();
        assertEquals(l2.length, 6);
        
        List<Integer> l3 = new LinkedList<Integer>();
        for (i = 10; i > 0; --i)
        	l3.add(i);
        mi1.retainAll(l3);
        assertEquals(mi1, new MultiInterval(2).union(new MultiInterval(10)));
        Integer[] l4 = new Integer[2];
        mi1.toArray(l4);
        assertEquals(l4.length, 2);
        
        Set<Integer> s = new HashSet<Integer>();
        s.add(Interval.INF - 1);
        s.add(-2);
        s.add(0);
        s.add(7);
        s.add(-1);
        s.add(Interval.SUP + 1);
        mi2 = new MultiInterval(s);
        MultiInterval m = new MultiInterval(-2, 0);
        assertEquals(mi2, m.union(new MultiInterval(7)));
        s.remove(-1);
        mi1 = new MultiInterval(s);
        assertFalse(mi1.contains(-1));
        for (i = 1; i < 7; ++i)
          s.add(i);
        mi = new MultiInterval(s);
        assertTrue(mi.contains(new Interval(0, 7)));
	}
	
	@Test
	public void 
	testFDConstraints() {
		List<Interval> l = new LinkedList<Interval>();
    	l.add(new Interval(0));
        l.add(new Interval(1, 2));
        l.add(new Interval(15));
        MultiInterval mi = new MultiInterval(l);
        assertEquals(mi.toString(), "{0..2, 15}");
        Solver s = new Solver();
        IntLVar X = new IntLVar("X");
        IntLVar Y = new IntLVar("Y");       
        s.add(X.dom(mi).and(X.neq(15)));
        int i = 0;
        if (s.check(X.label(ValHeuristic.MID_MOST)))
        	do {
        		i++;
        	} while (s.nextSolution());
        assertEquals(i, 3);
        
        // FIXME: s.clearStore() removes all the constraints in the c.store, 
        // but not the equality relations, previously created, between logical 
        // variables. In this case, X is still initialized.
        s.clearStore();
        assertTrue(s.check());
        s.add(Y.neq(0));
        s.add(Y.neq(100));
        s.add(Y.neq(15));
        s.add(Y.neq(5));
        assertTrue(s.check());
        assertEquals((int) X.getValue(), 2);
        assertFalse(Y.getDomain().contains(0));
        s.add(X.neq(Y).and(Y.lt(X.sum(X))));
        assertTrue(s.check());
        assertFalse(Y.getDomain().contains(2));
        IntLVar Z = new IntLVar("Z");
        l.add(new Interval(-1000));
        l.add(new Interval(-100, -50));
        mi = new MultiInterval(l);
        s.add(Z.dom(mi));
        l.clear();
        List<Integer> li = new LinkedList<Integer>();
        li.add(1);
        li.add(2);
        li.add(0);
        assertTrue(mi.containsAll(li));
        assertEquals(new MultiInterval(-1000, 1),
        		     mi.intersect(Y.getDomain()).convexClosure());
        s.add(Z.le(Y));
        assertTrue(s.check());
        assertFalse(Y.getDomain().contains(2));
        assertFalse(Z.getDomain().contains(-1));
        IntLVar A = new IntLVar("A");
        s.add(A.eq(Y.sum(Z.sub(X))));
        assertTrue(s.check());
        assertEquals(A.getDomain(), new Interval(-2002, 3));
        l.clear();
        l.add(new Interval(1, 3));
        l.add(new Interval(50, 52));
        mi = new MultiInterval(l);
        l.clear();
        l.add(new Interval(200, 210));
        l.add(new Interval(30, 31));
        mi = mi.sum(new MultiInterval(l));
        assertEquals(mi.toString(), 
        			"{31..34, 80..83, 201..213, 250..262}");
        assertEquals(A.getDomain().sub(mi), new Interval(-2264, -28));
	}
	
	@Test
	public void 
	testSetOperations() {
		ArrayList<Interval> v = new ArrayList<Interval>();
    	v.add(new Interval(0));
        v.add(new Interval(1, 2));
        v.add(new Interval(15));
        MultiInterval mi = new MultiInterval(v);
        assertTrue(mi.contains(new Interval(0, 2)));
        MultiInterval mi1 = mi.complement();
        assertFalse(mi1.contains(15));
        assertEquals(mi1.size(), 2147483643);
        assertEquals(mi1.complement(), mi);
        
        HashSet<Integer> s = new HashSet<Integer>();
        s.add(-1);
        s.add(2);
        s.add(5);
        s.add(16);
        mi1 = new MultiInterval(s);
        assertEquals(mi1.toString(), "{-1, 2, 5, 16}");
        MultiInterval mi2 = mi1.union(mi);
        assertFalse(mi2.contains(new MultiInterval(3, 4)));
        assertFalse(mi2.contains(new Interval(6, 14)));
        assertTrue(mi.subset(mi2));
        
        mi2 = mi2.complement();
        assertEquals(mi2.size(), 2147483640);
        assertEquals(mi2, mi1.complement().intersect(mi.complement()));
        mi2 = mi1.diff(mi);
        assertEquals(mi2.toString(), "{-1, 5, 16}");
        mi2 = mi.diff(mi1);
        assertEquals(mi2.toString(), "{0..1, 15}");
	}
	
}
