package jsetl.test;

import jsetl.*;

import java.io.IOException;
import java.util.*;

import jsetl.exception.Failure;
import jsetl.test.utilities.comparing.ConstraintStringComparator;
import junit.framework.TestCase;
import org.junit.Test;

public class RisIntegrationTest  extends TestCase {

    private <T> Set<T> arrayToSet(T[] array){
        Vector v = new Vector(Arrays.asList(array));
        return new HashSet<T>(v);
    }

    private Object[] lSetToArray(LSet set) {
        return set.getValue().toArray();
    }




    private Integer[] arbitraryFunction(Integer[] array, Ris ris) throws Failure {

        LSet domain = new LSet(arrayToSet(array));
        ris.getDomain().eq(domain).solve();
        Object[] results =  ris.expand().toArray();
        System.out.println(ris.expand().toArray());
        Integer[] res = new Integer[results.length];
        for(int i = 0; i < res.length; ++i)
            res[i] = ((IntLVar) results[i]).getValue();
        return res;
    }

    private <T> String arrayToString(T[] array) {
        ArrayList<T> vector = new ArrayList<T>();
        for(int i = 0; i < array.length; ++i)
            vector.add(array[i]);
        return vector.toString();
    }

    //@Test
    public void testRisAsFunction01() throws Failure{

        LSet domain = new LSet("domain");
        IntLVar x = new IntLVar("x");
        Ris ris = new Ris(x, domain, x.eq(x), x.mul(2));

        Integer[] array = {1,2,3,4};

        Integer[] result = arbitraryFunction(array, ris);

        assertEquals("[2, 4, 6, 8]",arrayToString(result));

    }

    //@Test
    public void testExample01A() {

        //S = { x : [ − 2 , 2] | x mod 2 = 0 • x } = {− 2 , 0 , 2 }
        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet(-2,2);

        Ris S = new Ris(x, domain, x.mod(2).eq(0), x);
        LSet solution = LSet.empty().ins(-2).ins(0).ins(2);

        Solver solver = new Solver();
        solver.add(S.eq(solution));

        assertEquals(solver.check(), true);
    }

    //@Test
    public void testExample01B() throws Failure{
        // (5 ,y ) in { x : D | x > 0 • ( x,x ∗ x ) } , where D is a variable

        IntLVar y = new IntLVar();

        LList p = LList.empty().ins(y).ins(5);
        LVar pp = new LVar("pp",p);

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D");

        LList px = LList.empty().ins(x.mul(x)).ins(x);

        Ris ris = new Ris(x,domain, x.gt(0), px);


        Solver solver = new Solver();
        solver.add(pp.in(ris));

        assertEquals(solver.check(), true);

        int yval = y.getValue();

        assertEquals(yval, 5*5);

    }

    //@Test
    public void testExample01C() {
        //(5 , 0) not in { ( x,y ) : { P u R }| y != 0 • ( x,y ) } , where P and R are variables.
        LList pair = LList.empty().ins(0).ins(5);


        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        LList p = new LList("p");

        LSet domain = new LSet().ins(p);

        LList pairxy = LList.empty().ins(y).ins(x);
        Ris ris = new Ris(pairxy, domain, y.neq(0));

        LSet sol = new LSet();

        Solver solver = new Solver();
        solver.add(ris.ncontains(pair));
        solver.add(sol.eq(ris));

        assertEquals(true, solver.check());




    }

    //@Test
    public void testExample01Cbis() {
        //(5 , 0) in { ( x,y ) : { P u R }| y != 0 • ( x,y ) } , where P and R are variables. -> False!
        LList pair = LList.empty().ins(0).ins(5);


        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        LList p = new LList("p");

        LSet domain = new LSet().ins(p);

        LList pairxy = LList.empty().ins(y).ins(x);
        Ris ris = new Ris(pairxy, domain, y.neq(0));
/*
		System.out.println(ris.F(LList.empty().ins(0).ins(5)).check());
		Constraint cc = new Constraint();
		System.out.println(ris.P(new LPair(1,2),cc));
		System.out.println(cc);*/


        Solver solver = new Solver();
        solver.add(ris.contains(pair));
        assertEquals(false, solver.check());




    }

    //@Test
    public void testExample01Ctris() {
        //(5 , 0) not in { ( x,y ) : { P u R }| y != 0 • ( x,y ) } , where P and R are variables.
        LList pair = LList.empty().ins(1).ins(5);


        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        LList p = new LList("p");

        LSet domain = new LSet().ins(p);

        LList pairxy = LList.empty().ins(y).ins(x);
        Ris ris = new Ris(pairxy, domain, y.neq(0));

        LSet sol = new LSet();

        Solver solver = new Solver();
        solver.add(ris.ncontains(pair));
        solver.add(sol.eq(ris));

        assertEquals(true, solver.check());




    }

    //@Test
    public void testExample01Ctetra() {
        //(5 , 0) not in { ( x,y ) : { P u R }| y != 0 • ( x,y ) } , where P and R are variables.
        LList pair = LList.empty().ins(1).ins(5);


        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        LList p = new LList("p");

        LSet domain = new LSet().ins(p);

        LList pairxy = LList.empty().ins(y).ins(x);
        Ris ris = new Ris(pairxy, domain, y.neq(0));

        LSet sol = new LSet();

        Solver solver = new Solver();
        solver.add(ris.contains(pair));
        solver.add(sol.eq(ris));

        assertEquals(true, solver.check());




    }

    //@Test
    public void testExample02() throws Failure{
        //y in S and S = { x : S | y ≤ x }
        // S = { 2 , 4 , 1 , 6 }
        // y = 1
        IntLVar y = new IntLVar("y");
        IntLVar x = new IntLVar("x");
        IntLSet ss = IntLSet.empty().ins(2).ins(4).ins(1).ins(6);
        Ris s = new Ris(x, ss, y.le(x));

        Solver solver = new Solver();
        solver.add(s.eq(ss));
        solver.add(y.in(ss));

        solver.solve();

        assertEquals((Integer)1,y.getValue());
    }

    //@Test
    public void testExample03A() throws Failure{
        //(5 ,y ) in { x : D @ ( x,x ∗ x ) }
        IntLVar y = new IntLVar("y");
        IntLVar x = new IntLVar("x");
        LList pair = LList.empty().ins(y).ins(5);
        LList pairxxx = LList.empty().ins(x.mul(x)).ins(x);

        IntLSet domain = new IntLSet(2,6);

        Ris ris = new Ris(x, domain, x.eq(x),pairxxx);

        Solver solver = new Solver();
        solver.add(ris.contains(pair));

        solver.solve();

        assertEquals((Integer)(5*5), y.getValue());

    }

    //@Test
    public void testExample03B() throws Failure{
        //(y ,36 ) in { x : D @ ( x,x ∗ x ) }
        IntLVar y = new IntLVar("y");
        IntLVar x = new IntLVar("x");
        LList pair = LList.empty().ins(36).ins(y);
        LList pairxxx = LList.empty().ins(x.mul(x)).ins(x);

        IntLSet domain = new IntLSet(-6,6);

        Ris ris = new Ris(x, domain, x.eq(x),pairxxx);

        Solver solver = new Solver();
        solver.add(ris.contains(pair));

        solver.solve();

        Set<Integer> solutions = new HashSet<Integer>();
        solutions.add(y.getValue());

        solver.nextSolution();

        solutions.add(y.getValue());

        //assert: only 2 solutions
        assertEquals(false, solver.nextSolution());

        //assert: -6 and 6 are solutions
        assertEquals(true, solutions.contains(-6));
        assertEquals(true, solutions.contains(6));
    }

    //@Test
    public void testExample03C() throws Failure{
        //f ( x ) = x^2 + 8, f(5):
        // (5 ,y ) in { x : D • ( x,x ∗ x ) } and
        //( y,z ) in { e : E • ( e,e + 8) }
        // y = 25 and z = 33
        IntLVar x= new IntLVar("x");
        IntLVar y = new IntLVar("y");
        IntLVar z = new IntLVar("z");
        IntLVar e = new IntLVar("e");

        LList pair5y = LList.empty().ins(y).ins(5);
        IntLSet domainD = new IntLSet();
        LList pairxxx = LList.empty().ins(x.mul(x)).ins(x);
        Ris risx = new Ris(x, domainD, x.eq(x), pairxxx);

        LList pairyz = LList.empty().ins(z).ins(y);
        IntLSet domainE = new IntLSet();
        LList paireesum8 = LList.empty().ins(e.sum(8)).ins(e);
        Ris rise = new Ris(e, domainE, e.eq(e), paireesum8);

        Solver solver = new Solver();
        solver.add(risx.contains(pair5y));
        solver.add(rise.contains(pairyz));

        solver.solve();

        assertEquals((Integer)25, y.getValue());
        assertEquals((Integer)33, z.getValue());
        assertEquals(false, solver.nextSolution());

    }

    //@Test
    public void testExample04Adifferent() {
        //X = { x : D | x != 0 } ( X and D may be the same variable)
        //case 1: X and D ARE NOT the same variable

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, x.neq(0));
        IntLSet X = new IntLSet("X");

        Solver solveCase1 = new Solver();
        solveCase1.add(X.eq(ris));

        assertEquals(true, solveCase1.check());
    }

    //@Test
    public void testExample04Asame() {
        //X = { x : D | x != 0 } ( X and D may be the same variable)
        //case 2: X and D ARE the same variable

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, x.neq(0));


        Solver solveCase1 = new Solver();
        solveCase1.add(domain.eq(ris));

        assertEquals(true, solveCase1.check());
    }

    //@Test
    public void testExample04B() {
        // 1 not in { x : D | x != 0 }

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet();
        Ris ris = new Ris(x, domain, x.neq(0));


        Solver solver = new Solver();
        solver.add(ris.ncontains(1));

        assertEquals(true, solver.check());
    }

    //@Test
    public void testExample04C() {
        // { x : D_1 | x mod 2 = 0 @ ( x,x ) } = { x : D_2 | x > 0 @ ( x,x + 2) }

        IntLVar x = new IntLVar("x");
        IntLSet domain1 = new IntLSet();
        Ris ris1 = new Ris(x, domain1, x.mod(2).eq(0), LList.empty().ins(x).ins(x));

        IntLSet domain2 = new IntLSet();
        IntLVar y = new IntLVar("y");
        Ris ris2 = new Ris(y, domain2, y.gt(0), LList.empty().ins(y.sum(2)).ins(y));


        Solver solver = new Solver();
        solver.add(ris1.eq(ris2));

        assertEquals(true, solver.check());
    }

    //@Test
    public void testExample05() {
        // { 5 } in { x : { y u D } | x!= {} and  5 not in x @ x }

        IntLSet xset = new IntLSet("x");

        IntLSet y = new IntLSet();
        LSet domain = new LSet();
        domain =domain.ins(y);

        Ris ris = new Ris(xset, domain, xset.neq(LSet.empty()).and(xset.ncontains(5)));

        Solver solver = new Solver();
        solver.add(ris.contains(IntLSet.empty().ins(5)));
        System.out.println(domain);
        assertEquals(false, solver.check());
    }

    //@Test
    public void testExample05bis() {
        // { 5 } in { x : { y u D } | x!= {} and  5 in x @ x }

        IntLSet xset = new IntLSet("x");

        IntLSet y = new IntLSet();
        LSet domain = new LSet();
        domain =domain.ins(y);



        Ris ris = new Ris(xset, domain, xset.neq(LSet.empty()).and(xset.contains(5)));

        Solver solver = new Solver();
        solver.add(ris.contains(IntLSet.empty().ins(5)));
        System.out.println(domain);
        assertEquals(true, solver.check());
    }

    //@Test
    public void testExample06A() {
        // multiples of N: ris ( X in D,  X mod N = 0 )
        IntLVar N = new IntLVar(3);
        IntLSet domain = new IntLSet(-10,10);
        IntLVar x = new IntLVar("x");
        Ris ris = new Ris(x, domain, x.mod(N).eq(0));


        Solver solver = new Solver();
        IntLSet multiples = new IntLSet();
        solver.add(multiples.eq(ris));

        assertEquals(true, solver.check());

        Integer[] sols = {-9,-6,-3,0,3,6,9};
        ArrayList<Object> solutionVector = new ArrayList<Object>();
        for(Object o : multiples)
            solutionVector.add(o);
        Set<Integer> solutions = new HashSet<Integer>();
        for(Object o:solutionVector)
            solutions.add((Integer)((LVar)o).getValue());
        System.out.println(solutionVector);
        for(Integer s:sols)
            assertEquals(true, solutions.contains(s));
        assertEquals(false, solver.nextSolution());
    }

    //@Test
    public void testExample06B() {
        //ris ( S in D, subset ( A,S ))
        LSet s = new LSet();
        LSet d = LSet.empty()
                .ins(LSet.empty().ins(5).ins(9).ins("ciao"))
                .ins(LSet.empty().ins(5).ins(3).ins("ciao"))
                .ins(LSet.empty().ins(9));
        LSet a = LSet.empty().ins(5).ins("ciao");

        LSet sol = new LSet();
        Ris ris = new Ris(s,d,a.subset(s));

        Solver solver = new Solver();
        solver.add(sol.eq(ris));
        assertEquals(true, solver.check());
        assertEquals("{{ciao,3,5},{ciao,9,5}}", sol.toString());
    }

    //@Test
    public void testExample06C() throws Failure {
        //ris ([ X,Y ] in D,Y is X * X )
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList pair = LList.empty().ins(y).ins(x);
        LSet domain = LSet.empty()
                .ins(LList.empty().ins(9).ins(3))
                .ins(LList.empty().ins(5).ins(4))
                .ins(LList.empty().ins(100).ins(-10))
                .ins(LList.empty().ins(3).ins(6))
                .ins(LList.empty().ins(1).ins(1))
                .ins(LList.empty().ins(0).ins(0));
        Ris ris = new Ris(pair, domain, y.eq(x.mul(x)));
        LList onesol =LList.empty().ins(new IntLVar()).ins(new IntLVar());

        Solver solver = new Solver();
        solver.add(ris.contains(onesol));

        assertEquals(true, solver.check());
        assertEquals("[0,0]", onesol.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("[1,1]", onesol.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("[-10,100]", onesol.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("[3,9]", onesol.toString());
        assertEquals(false, solver.nextSolution());

    }

    //@Test
    public void testExample06Cbis() throws Failure {
        //ris ([ X,Y ] in D,Y is X * X )
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList pair = LList.empty().ins(y).ins(x);
        LSet domain = LSet.empty()
                .ins(LList.empty().ins(9).ins(3))
                .ins(LList.empty().ins(5).ins(4))
                .ins(LList.empty().ins(100).ins(-10))
                .ins(LList.empty().ins(3).ins(6))
                .ins(LList.empty().ins(1).ins(1))
                .ins(LList.empty().ins(0).ins(0));
        Ris ris = new Ris(pair, domain, y.eq(x.mul(x)));
        LSet sol = LSet.empty()
                .ins(LList.empty().ins(9).ins(3))
                .ins(LList.empty().ins(100).ins(-10))
                .ins(LList.empty().ins(1).ins(1))
                .ins(LList.empty().ins(0).ins(0));
        Solver solver = new Solver();
        solver.add(sol.eq(ris));

        assertEquals(true, solver.check());
        assertEquals("{[0,0],[1,1],[-10,100],[3,9]}", sol.toString());
    }

    //@Test
    public void testExample06D() {
        //ris ( X in { 2 , 4 /M } , 2 * X ) = { 2 , 4 , 6 , 8 }
        IntLVar x = new IntLVar("x");
        IntLSet domain = IntLSet.mkIntSet(4);
        Ris ris = new Ris(x, domain, x.eq(x), x.mul(2));

        IntLSet sol = IntLSet.empty().ins(8).ins(6).ins(4).ins(2);

        Solver solver = new Solver();
        solver.add(sol.eq(ris));
        assertEquals(true, solver.check());
        assertEquals("{1..4}", domain.toString());
    }

    //@Test
    public void testExample07() {
        //inters ( A,B,C ) ∧ D = ris ( X in A,X in B ) ∧ C neq D -> false
        LSet A = new LSet("A");
        LSet B = new LSet("B");
        LSet C = new LSet("C");
        LVar x = new LVar();
        Ris D = new Ris(x, A, x.in(B));

        Solver solver = new Solver();
        solver.add(A.inters(B,C));
        solver.add(C.neq(D));

        assertEquals(false,solver.check());
    }

    //@Test
    public void testDisjRisVoid() {
        // {x:D | x = x @ x} || void
        LSet lvoid = LSet.empty();
        IntLVar x = new IntLVar("x");
        LSet d = new LSet("d");
        Ris ris = new Ris(x, d, x.eq(x));

        Solver solver = new Solver();
        solver.add(ris.disj(lvoid));

        assertEquals(true, solver.check());
    }

    //@Test
    public void testDisjVoidRis() {
        // {x:D | x = x @ x} || void
        LSet lvoid = LSet.empty();
        IntLVar x = new IntLVar("x");
        LSet d = new LSet("d");
        Ris ris = new Ris(x, d, x.eq(x));

        Solver solver = new Solver();
        solver.add(lvoid.disj(ris));

        assertEquals(true, solver.check());
    }

    //@Test
    public void  testDisjRisNotVoidFalse01() {
        // {x:D | x = x @ x} || {y/R}
        IntLVar y = new IntLVar(5);
        LSet notvoid = new LSet().ins(y);
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet(1,10);
        Ris ris = new Ris(x, d, x.eq(x));

        Solver solver = new Solver();
        solver.add(ris.disj(notvoid));
        assertEquals(false, solver.check());
    }
    //@Test
    public void  testDisjRisNotVoidFalse02() {
        // {x:D | x = x @ x} || {y/R}
        IntLVar y = new IntLVar(15);
        LSet notvoid = new LSet().ins(y).ins(new IntLVar(9));
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet(1,10);
        Ris ris = new Ris(x, d, x.eq(x));

        Solver solver = new Solver();
        solver.add(ris.disj(notvoid));
        assertEquals(false, solver.check());
    }
    //@Test
    public void  testDisjRisNotVoidTrue01() {
        // {x:D | x = x @ x} || {y/R}
        IntLVar y = new IntLVar(15);
        LSet notvoid = new LSet().ins(y);
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet(1,10);
        Ris ris = new Ris(x, d, x.eq(x));

        Solver solver = new Solver();
        solver.add(ris.disj(notvoid));
        assertEquals(true, solver.check());
    }
    //@Test
    public void  testDisjRisNotVoidTrue02() {
        // {x:D | x = x @ x} || {y/R}
        IntLVar y = new IntLVar(15);
        LSet notvoid = new LSet().ins(y).ins(new IntLVar(22));
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet(1,10);
        Ris ris = new Ris(x, d, x.eq(x));

        Solver solver = new Solver();
        solver.add(ris.disj(notvoid));
        assertEquals(true, solver.check());
    }
    //@Test
    public void  testDisjRisNotInitRisNotInitTrue() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet();
        Ris ris = new Ris(x, d, x.eq(x));

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet();

        Ris ris2 = new Ris(y, b, y.eq(y), y);

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(true, solver.check());
    }

    //@Test
    public void  testDisjRisInitRisInitTrue01() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet(1,10);
        Ris ris = new Ris(x, d, x.eq(x));

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet(11,20);

        Ris ris2 = new Ris(y, b, y.eq(y), y);

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(true, solver.check());
    }

    //@Test
    public void  testDisjRisInitRisInitTrue02() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        LSet d = new LSet().ins(4).ins(-2);
        Ris ris = new Ris(x, d, x.eq(x));

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet(11,20);

        Ris ris2 = new Ris(y, b, y.eq(y), y);

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(true, solver.check());
    }

    //@Test
    public void  testDisjRisInitRisInitFalse01() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet(1,9);
        Ris ris = new Ris(x, d, x.eq(x));
        assertEquals(true, ris.isGround());

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet(5,7);

        Ris ris2 = new Ris(y, b, y.eq(y), y);

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(false, solver.check());
    }

    //@Test
    public void  testDisjRisInitRisInitFalse02() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(9).ins(1);
        Ris ris = new Ris(x, d, x.eq(x));

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet(8,20);

        Ris ris2 = new Ris(y, b, y.eq(y), y);

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(false, solver.check());
    }

    //@Test
    public void  testDisjRisInitRisInitFalse03() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        LSet d = new LSet().ins(2).ins(1);
        Ris ris = new Ris(x, d, x.eq(x));

        IntLVar y = new IntLVar("y");
        LSet b = LSet.empty().ins(8).ins(9).ins(20);

        Ris ris2 = new Ris(y, b, y.eq(y), y);

        Solver solver = new Solver();
        solver.add(d.contains(20));
        solver.add(ris.disj(ris2));
        assertEquals(false, solver.check());
    }

    //@Test
    public void  testDisjRisNotInitRisInitTrue() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet();
        Ris ris = new Ris(x, d, x.eq(x));

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet(8,20);

        Ris ris2 = new Ris(y, b, y.eq(y), y);

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(true, solver.check());
    }

    //@Test
    public void  testDisjRisInitRisNotInitTrue() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet();
        Ris ris = new Ris(x, d, x.eq(x));

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet();

        Ris ris2 = new Ris(y, b, y.eq(y), y);

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(true, solver.check());
    }

    //@Test
    public void  testDisjRisInitRisInitTrue() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(3).ins(2).ins(1);
        Ris ris = new Ris(x, d, x.mod(2).eq(1), x.sum(1));

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet().ins(3);
        Ris ris2 = new Ris(y, b, y.mod(2).eq(0), y.sum(1));

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(true, solver.check());
    }

    //@Test
    public void  testDisjRisInitRisInitFALSE() {
        // {x:D | x = x @ x} || {y:B | y= y @ y}

        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet().ins(3).ins(2).ins(1);
        Ris ris = new Ris(x, d, x.mod(2).eq(1), x.sum(2));

        IntLVar y = new IntLVar("y");
        IntLSet b = new IntLSet().ins(2);
        Ris ris2 = new Ris(y, b, y.mod(2).eq(0), y.sum(1));

        Solver solver = new Solver();
        solver.add(ris.disj(ris2));
        assertEquals(false, solver.check());
    }

    //@Test
    public void test01() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(-10,10);
        Ris r1 = new Ris(x, new IntLSet(), x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(1));
        Solver solver = new Solver();

        solver.add(r2.eq(r1));

        assertEquals(false, solver.check());
    }

    //@Test
    public void test02() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet().ins(1).ins(5).ins(9);
        Ris r1 = new Ris(x, new IntLSet("C"), x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(2).eq(0));
        Solver solver = new Solver();

        solver.add(r1.eq(r2));

        assertEquals(true, solver.check());
    }

    //@Test
    public void test03() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet().ins(1).ins(5).ins(9);
        Ris r1 = new Ris(x, new IntLSet("C"), x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(2).eq(1));
        Solver solver = new Solver();

        solver.add(r1.eq(r2));

        assertEquals(false, solver.check());
    }

    //@Test
    public void testLListControlTermUnbounded01() {
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain = new LSet()
                .ins(LList.empty().ins(5).ins(3))
                .ins(LList.empty().ins(2).ins(1));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(LList.empty().ins(5).ins(3));
        solver.add(sol.eq(ris));
        assertEquals(false, solver.check());

    }

    //@Test
    public void testLListControlTermUnbounded02() {
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain = new LSet()
                .ins(LList.empty().ins(5).ins(3))
                .ins(LList.empty().ins(2).ins(1))
                .ins(LList.empty())
                .ins(LList.empty().ins(1).ins(1));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(LList.empty().ins(2).ins(1));
        System.out.println(ris.expand());
        solver.add(sol.eq(ris));
        assertEquals(true, solver.check());

    }

    //@Test
    public void testLListControlTermUnbounded03() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain = new LSet()
                .ins(LList.empty().ins(5).ins(3))
                .ins(LList.empty().ins(5).ins(4))
                .ins(LList.empty().ins(2).ins(2));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(LList.empty().ins(5).ins(4));
        solver.add(sol.eq(ris));
        assertEquals(true, solver.check());

    }

    //@Test
    public void testLListControlTermUnbounded04() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain = new LSet()
                .ins(new LList().ins(5).ins(3))
                .ins(new LList().ins(5).ins(4))
                .ins(new LList().ins(3).ins(2));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(new LList().ins(5).ins(4)).ins(LList.empty().ins(3).ins(2));
        solver.add(sol.eq(ris));
        assertEquals(true, solver.check());

    }

    //@Test
    public void testLListControlTermUnbounded05() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain = new LSet()
                .ins(new LList().ins(5).ins(3))
                .ins(new LList().ins(5).ins(4))
                .ins(new LList().ins(3).ins(2));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(new LList().ins(5).ins(4));
        solver.add(sol.eq(ris));
        assertEquals(false, solver.check());

    }

    //@Test
    public void testLListControlTermUnbounded06() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain = new LSet()
                .ins(new LList().ins(5).ins(3))
                .ins(new LList().ins(5).ins(4))
                .ins(new LList().ins(3).ins(2));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(new LList().ins(5).ins(4)).ins(new LList().ins(1).ins(1));
        solver.add(sol.eq(ris));
        assertEquals(false, solver.check());

    }

    //@Test
    public void testLListControlTermUnbounded07() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain = new LSet()
                .ins(new LList().ins(5).ins(3))
                .ins(new LList().ins(5).ins(4));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(new LList().ins(5).ins(4)).ins(new LList().ins(2).ins(1));
        solver.add(sol.eq(ris));
        assertEquals(true, solver.check());

    }

    //@Test
    public void testLListControlTermUnbounded08() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain = new LSet("domain")
                .ins(LList.empty().ins(5).ins(3))
                .ins(LList.empty().ins(5).ins(4));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(LList.empty().ins(5).ins(4)).ins(LList.empty().ins(2).ins(1));
        solver.add(sol.eq(ris));
        assertEquals(true, solver.check());

    }

    //@Test
    public void testLListControlTermUnbounded09() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LList list = new LList().ins(y).ins(x);

        LSet domain =  LSet.empty()
                .ins(new LList().ins(5).ins(3))
                .ins(new LList().ins(5).ins(4));


        Ris ris = new Ris(list, domain, y.eq(x.sum(1)));

        Solver solver = new Solver();
        LSet sol = LSet.empty().ins(new LList().ins(5).ins(4)).ins(new LList().ins(2).ins(1));
        solver.add(sol.eq(ris));
        assertEquals(false, solver.check());

    }

    //@Test
    public void testGuessDomain01() {
        IntLVar x = new IntLVar("x");
        LSet domain = new LSet("domain");
        Ris ris = new Ris(x,domain, x.eq(x), x.mul(2));
        LSet sol = LSet.empty().ins(4).ins(2);

        assertEquals(true, sol.eq(ris).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{1,2/_N31114}", domain.toString());
        assertTrue(comparator.areEqualInSameOrder());
    }

    //@Test
    public void testGuessDomain02() {
        IntLVar x = new IntLVar("x");
        LSet domain = new LSet("domain");
        Ris ris = new Ris(x,domain, x.eq(x), x.mul(2));
        LSet sol = LSet.empty().ins(3).ins(2);

        assertEquals(false, sol.eq(ris).check());
        assertEquals("_domain", domain.toString());
    }

    //@Test
    public void testGuessDomain03() {
        IntLVar x = new IntLVar("x");
        LSet domain = new LSet("domain").ins(2);
        Ris ris = new Ris(x,domain, x.eq(x), x.mul(2));
        LSet sol = LSet.empty().ins(4).ins(2);

        assertEquals(true, sol.eq(ris).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{2,1/_N31162}", domain.toString());
        assertTrue(comparator.areEqualInSameOrder());
    }

    //@Test
    public void test() {

        //{ _x : {_z} | 3 <= _x @ _x } = {3|_?}

        IntLVar x = new IntLVar();
        IntLVar z = new IntLVar();
        LSet domain = LSet.empty().ins(z);

        IntLSet sol = new IntLSet().ins(3);
        Ris ris = new Ris(x, domain, x.ge(3));
        Solver solver = new Solver();
        solver.add(ris.eq(sol));
        assertEquals(true,  solver.check());
    }

    ////@Test
    public void test111A() {
        //%Check whether N is prime or not {log}=> N = 101 & N>1 &
        //MD is N div 2 & ris(X in int(2,MD),0 is N mod X) = {}.
        //N = 101, MD = 50
        Solver solver = new Solver();
        IntLVar N = new IntLVar("N",101);
        solver.add(N.gt(1));
        IntLVar MD = new IntLVar("MD", N.getValue()/2);

        IntLVar X = new IntLVar("X");
        IntLSet domain = new IntLSet(2, MD.getValue());

        Ris ris = new Ris(X, domain, N.mod(X).eq(0));
        LSet empty = LSet.empty();

        solver.add(ris.eq(empty));

        assertEquals(true, solver.check());
    }

    ////@Test
    public void test111B() {
        //%Check whether N is prime or not {log}=> N = 101 & N>1 &
        //MD is N div 2 & ris(X in int(2,MD),0 is N mod X) = {}.
        //N = 101, MD = 50
        Solver solver = new Solver();
        IntLVar N = new IntLVar("N",20);
        solver.add(N.gt(1));
        IntLVar MD = new IntLVar("MD", N.getValue()/2);

        IntLVar X = new IntLVar("X");
        IntLSet domain = new IntLSet(2, MD.getValue());

        Ris ris = new Ris(X, domain, N.mod(X).eq(0));
        LSet empty = LSet.empty();

        solver.add(ris.eq(empty));

        assertEquals(false, solver.check());
    }



    ////@Test
    public void test112A1() {
        //%Minimo di un insieme (non completamente specificato)
        //{log}=> S={3,2,Z} & Y in S & S=ris(X in S,Y =< X).
        IntLVar z = new IntLVar("z");
        IntLVar x = new IntLVar("x");

        IntLSet s = IntLSet.empty().ins(z).ins(2).ins(3);
        IntLVar y = new IntLVar("y");

        Ris ris = new Ris(x,s, y.le(x));


        Solver solver = new Solver();
        solver.add(y.in(s));
        solver.add(ris.eq(s));

        //fail("NON TERMINAVA!");
        assertEquals(true, solver.check());

        assertEquals((Integer)2, y.getValue());

    }

    //@Test
    public void test112A2() {
        //%Minimo di un insieme (non completamente specificato)
        //{log}=> S={3,2,Z} & Y in S & S=ris(X in S,Y =< X).
        IntLVar z = new IntLVar("z");
        IntLVar x = new IntLVar("x");

        IntLSet s = IntLSet.empty().ins(z).ins(3).ins(2);
        IntLVar y = new IntLVar("y");

        Ris ris = new Ris(x,s, y.le(x));


        Solver solver = new Solver();
        solver.add(y.in(s));
        solver.add(ris.eq(s));

        //fail("NON TERMINAVA!");
        assertEquals(true, solver.check());

        assertEquals((Integer)2, y.getValue());

    }

    //@Test
    public void test112A3() {
        //%Minimo di un insieme (non completamente specificato)
        //{log}=> S={3,2,Z} & Y in S & S=ris(X in S,Y =< X).
        IntLVar z = new IntLVar("z");
        IntLVar x = new IntLVar("x");

        IntLSet s = IntLSet.empty().ins(2).ins(z).ins(3);
        IntLVar y = new IntLVar("y");

        Ris ris = new Ris(x,s, y.le(x));


        Solver solver = new Solver();
        solver.add(y.in(s));
        solver.add(ris.eq(s));

        assertEquals(true, solver.check());
        assertEquals("[-1073741823..2]", y.getDomain().toString());
        assertEquals("[-1073741823..2]", z.getDomain().toString());
        assertEquals(y, z);

        boolean flag = false;
        while(solver.nextSolution())
            if(y.getDomain().toString().equals("{2}") &&
                    z.getDomain().toString().equals("{2}")
                    && y.equals(z))
                flag = true;
        assertTrue(flag);


    }

    //@Test
    public void test112A4() {
        //%Minimo di un insieme (non completamente specificato)
        //{log}=> S={3,2,Z} & Y in S & S=ris(X in S,Y =< X).
        IntLVar z = new IntLVar("z");
        IntLVar x = new IntLVar("x");

        IntLSet s = IntLSet.empty().ins(2).ins(3).ins(z);
        IntLVar y = new IntLVar("y");

        Ris ris = new Ris(x,s, y.le(x));


        Solver solver = new Solver();
        solver.add(y.in(s));
        solver.add(ris.eq(s));

        assertEquals(true, solver.check());
        assertEquals("[-1073741823..2]", y.getDomain().toString());
        assertEquals("[-1073741823..2]", z.getDomain().toString());
        assertEquals(y, z);

        boolean flag = false;
        while(solver.nextSolution())
            if(y.getDomain().toString().equals("{2}") &&
                    z.getDomain().toString().equals("{2}")
                    && y.equals(z))
                flag = true;
        assertTrue(flag);

    }

    //@Test
    public void test112A5() {
        //%Minimo di un insieme (non completamente specificato)
        //{log}=> S={3,2,Z} & Y in S & S=ris(X in S,Y =< X).
        IntLVar z = new IntLVar("z");
        IntLVar x = new IntLVar("x");

        IntLSet s = IntLSet.empty().ins(3).ins(z).ins(2);
        IntLVar y = new IntLVar("y");

        Ris ris = new Ris(x,s, y.le(x));


        Solver solver = new Solver();
        solver.add(y.in(s));
        solver.add(ris.eq(s));

        //fail("NON TERMINAVA!");
        assertEquals(true, solver.check());

        assertEquals((Integer)2, y.getValue());

    }

    //@Test
    public void test112A6() {
        //%Minimo di un insieme (non completamente specificato)
        //{log}=> S={3,2,Z} & Y in S & S=ris(X in S,Y =< X).
        IntLVar z = new IntLVar("z");
        IntLVar x = new IntLVar("x");

        IntLSet s = IntLSet.empty().ins(3).ins(2).ins(z);
        IntLVar y = new IntLVar("y");

        Ris ris = new Ris(x,s, y.le(x));


        Solver solver = new Solver();
        solver.add(y.in(s));
        solver.add(ris.eq(s));

        assertEquals(true, solver.check());
        assertEquals("[-1073741823..2]", y.getDomain().toString());
        assertEquals("[-1073741823..2]", z.getDomain().toString());
        assertEquals(y, z);

        boolean flag = false;
        while(solver.nextSolution())
            if(y.getDomain().toString().equals("{2}") &&
                    z.getDomain().toString().equals("{2}")
                    && y.equals(z))
                flag = true;
        assertTrue(flag);


    }

    //@Test
    public void test112B() {
        //%powerset of a s S
        //{log}=> S={a,b} & {b} in ris(SS in D, subset(SS,S)).
        //S = {a,b}, D = {{b}/_N1}, Constraint: s(_N1), s(SS)
        LVar a = new LVar("a");
        LVar b = new LVar("b");
        LSet S = LSet.empty().ins(a).ins(b);
        LSet s = LSet.empty().ins(b);

        LSet SS = new LSet();
        LSet D = new LSet();
        Ris ris = new Ris(SS, D, SS.subset(S));

        assertEquals(true, ris.contains(s).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{{_b}/_N8600}", D.toString());
        assertTrue(comparator.areEqualInSameOrder());
    }

    //@Test
/*	public void test112C() {
		//%e� anche possibile calcolare l�insieme potenza dato che e� nota la sua cardinalit�
		//{log}=> S={a,b} & solve(size(PS,4)) & call(PS= ris(SS in D, subset(SS,S))).
		//S = {a,b}, PS = {{},{b},{a},{a,b}},
		//D = {{},{b},{a},{a,b}/_N1}
		//Constraint: {}nin _N1, {b}nin _N1, {a}nin _N1, {a,b}nin _N1,
		//ris(SS in _N1,[],subset(SS,{a,b}),SS,true)={}, s(SS)
		LVar a = new LVar("a");
		LVar b = new LVar("b");
		LSet S = LSet.empty().ins(a).ins(b);


		LSet PS = LSet.mkSet(4); //se creo un LSet generico non termina?!

		LSet SS = new LSet();
		LSet D = LSet.mkSet(4);
		Ris ris = new Ris(SS, D, SS.subset(S));



		solver solver = new solver();
		solver.add(PS.eq(ris));
		solver.add(PS.size(4));
		//solver.add(Constraint.allDifferent(PS.toArray());

		assertEquals(true, solver.check());
		solver.showStoreAll();
		assertEquals("{{},{b},{a},{a,b}}", D.toString()); // so che non va

	}*/

    //@Test
    public void test121A() {
        //{log}=> ris(X in int(1,4),true,2*X) = {2,4,6,8} yes
        IntLSet in = new IntLSet(1,4);
        IntLVar x = new IntLVar("x");
        Ris ris = new Ris(x, in, x.eq(x), x.mul(2));

        Integer[] elems = {2,4,6,8};
        LSet sol = LSet.empty().insAll(elems);

        assertEquals(true, sol.eq(ris).check());
    }

    //@Test
    public void test121B() throws Failure {
        //%The s of even numbers in a given interval
        //{log}=> S is ris(X in int(1,10),true,2*X).
        //S = {20,2,4,6,8,10,12,14,16,18}
        IntLSet in = new IntLSet(1,10);
        IntLVar x = new IntLVar("x");
        Ris ris = new Ris(x, in, x.eq(x), x.mul(2));

        Integer[] elems = {20,2,4,6,8,10,12,14,16,18};
        LSet sol = LSet.empty().insAll(elems);

        assertEquals(true, ris.expand().eq(sol).check());
    }

    //@Test
    public void test121Bbis() throws Failure {
        //%The s of even numbers in a given interval
        //{log}=> S is ris(X in int(1,10),true,2*X).
        //S = {20,2,4,6,8,11,12,14,16,18} NO
        IntLSet in = new IntLSet(1,10);
        IntLVar x = new IntLVar("x");
        Ris ris = new Ris(x, in, x.eq(x), x.mul(2));

        Integer[] elems = {20,2,4,6,8,11,12,14,16,18};
        LSet sol = LSet.empty().insAll(elems);

        assertEquals(false, ris.expand().eq(sol).check());
    }

    //@Test
    public void test121C() throws Failure {
        //{log}=> 9 in ris(X in int(1,100),true,2*X). NO

        IntLSet in = new IntLSet(1,100);
        IntLVar x = new IntLVar("x");
        Ris ris = new Ris(x, in, x.eq(x), x.mul(2));



        assertEquals(false, ris.contains(9).check());
    }

    //@Test
    public void test121Cbis() throws Failure {
        //{log}=> 50 in ris(X in int(1,100),true,2*X). yes

        IntLSet in = new IntLSet(1,100);
        IntLVar x = new IntLVar("x");
        Ris ris = new Ris(x, in, x.eq(x), x.mul(2));



        assertEquals(true, ris.contains(50).check());
    }

    //@Test
    public void test121D() throws Failure {
        //{log}=> 9 in ris(X in int(1,100),true,2*X). NO

        IntLSet in = new IntLSet(1,100);
        IntLVar x = new IntLVar("x");
        Ris ris = new Ris(x, in, x.eq(x), x.mul(2));



        assertEquals(true, ris.ncontains(9).check());
    }

    //@Test
    public void test122A() throws Failure{
        //{log}=> {[2,4],[1,1],[-2,4]}= ris(X in D,[Z],X<3,[X,Z],Z is X*X).
        //D = {2,1,-2/_?}

        LSet sol = LSet.empty()
                .ins(LList.empty().ins(4).ins(2))
                .ins(LList.empty().ins(1).ins(1))
                .ins(LList.empty().ins(4).ins(-2));

        IntLVar x = new IntLVar("x");
        LSet d = new IntLSet("d");
        IntLVar z = new IntLVar("z"); //existential variable
        //z.getConstraint().add(z.eq(x.mul(x))); //add constraint to variable
        Ris ris = new Ris(x, d,
                z.eq(x.mul(x)).and(x.le(3)),
                LList.empty().ins(z).ins(x),
                z);

        assertEquals(true, sol.eq(ris).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{-2, 1..2/_N9782}", d.toString());
        assertTrue(comparator.areEqualInSameOrder());
    }

    //@Test
    public void test122AAA() throws Failure{
        //{log}=> {[2,4],[1,1],[-2,4]}= ris(X in D,[Z],X<3,[X,Z],Z is X*X).
        //D = {2,1,-2/_?}

        LSet sol = LSet.empty()
                .ins(LList.empty().ins(4).ins(2))
                .ins(LList.empty().ins(1).ins(1))
                .ins(LList.empty().ins(4).ins(-2));

        IntLVar x = new IntLVar("x");
        LSet d = new IntLSet("d");
        IntLVar z = new IntLVar("z");
        z.getConstraint().add(z.label());
        Ris ris = new Ris(x, d, x.le(3), LList.empty().ins(z).ins(x),z);

        assertEquals(true, sol.eq(ris).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{-2, 1..2/_N20481}",d.toString());
        assertTrue(comparator.areEqualInSameOrder());
    }

    //@Test
    public void test122Abis() throws Failure {
        //{log}=> {[2,4],[1,1],[-2,4]}= ris(X in D,[Z],X<3,[X,Z],Z is X*X).
        //D = {2,1,-2/_?}

        LSet sol = LSet.empty()
                .ins(LList.empty().ins(4).ins(2))
                .ins(LList.empty().ins(1).ins(1))
                .ins(LList.empty().ins(4).ins(-2));

        IntLVar x = new IntLVar("x");
        LSet d = new IntLSet("d");

        Ris ris = new Ris(x, d, x.le(3), LList.empty().ins(x.mul(x)).ins(x));

        assertEquals(true, sol.eq(ris).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{-2, 1..2/_N13942}", d.toString());
        assertTrue(comparator.areEqualInSameOrder());
    }

    //@Test
    public void test131A() {
//		%If R is a s of ordered pairs and D is a s, then the subset of R where all first component
//		%belong to D is:
//		{log}=> D={2} & R={[2,a],[1,b],[2,c]} & {[2,a],[2,c]} = ris([X,Y] in R,X in D).
//		D = {2}, R = {[2,a],[1,b],[2,c]}
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        IntLVar a = new IntLVar("a");
        IntLVar b = new IntLVar("b");
        IntLVar c = new IntLVar("c");
        LSet D = LSet.empty().ins(2);
        LSet R = LSet.empty()
                .ins(new LPair(2, a))
                .ins(new LPair(1, b))
                .ins(new LPair(2, c));

        LSet sol = LSet.empty()
                .ins(new LPair(2, a))
                .ins(new LPair(2, c));

        Ris ris = new Ris(new LPair(x, y), R, x.in(D));

        assertEquals(true, sol.eq(ris).check());
    }

    //@Test
    public void test131Abis() {
//		%If R is a s of ordered pairs and D is a s, then the subset of R where all first component
//		%belong to D is:
//		{log}=> D={2} & R={[2,a],[1,b],[2,c]} & {[2,a],[2,c]} = ris([X,Y] in R,X in D).
//		D = {2}, R = {[2,a],[1,b],[2,c]}
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        IntLVar a = new IntLVar("a");
        IntLVar b = new IntLVar("b");
        IntLVar c = new IntLVar("c");
        LSet D = LSet.empty().ins(2);
        LSet R = new LSet();

        LSet sol = LSet.empty()
                .ins(new LPair(2, a))
                .ins(new LPair(2, c));

        Ris ris = new Ris(new LPair(x, y), R, x.in(D));

        assertEquals(true, sol.eq(ris).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{(2,_c),(2,_a)/_N13980}", R.toString());
        assertTrue(comparator.areEqualInSameOrder());
    }

    //@Test
    public void test132A() {
//		{log}=> S={[1,2],[2,3]} & S=ris([X,Y] in S,Y neq X). (N.B. domain=RIS)
//				S = {[1,2],[2,3]}
        LSet S = LSet.empty()
                .ins(new LPair(1, 2))
                .ins(new LPair(2, 3));

        IntLVar x = new IntLVar();
        IntLVar y = new IntLVar();

        Ris ris = new Ris(new LPair(x, y), S, y.neq(x));

        assertEquals(true, S.eq(ris).check());

    }

    //@Test
    public void test132B() {
//		{log}=> S=ris([X,Y] in S,Y neq X) & [Z,Z] in S. (N.B. domain=RIS)
//				no

        LSet S = new LSet("S");
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        IntLVar z = new IntLVar("z");

        Ris ris = new Ris(new LPair(x, y), S, y.neq(x));

        Solver solver = new Solver();
        solver.add(S.eq(ris));
        solver.add(S.contains(new LPair(z, z)));

        assertEquals(false, solver.check());
    }

    //@Test
    public void test132Bbis() {
//		{log}=> S=ris([X,Y] in S,Y neq X) & [Z,Z] in S. (N.B. domain=RIS)
//				no

        LSet S = new LSet("S");
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        IntLVar z = new IntLVar("z");

        Ris ris = new Ris(new LPair(x, y), S, y.neq(x));

        Solver solver = new Solver();
        solver.add(S.eq(ris));
        solver.add(S.contains(new LPair(z, z.sum(1))));

        assertEquals(true, solver.check());
    }

    //@Test
    public void test132C() {
//		%A function that maps sets to their cardinalities provided they are greater than one
//		{log}=> [{a,b,c},N] in ris([S,C] in D,size(S,C) & C > 1).
//		N = 3, D = {[{a,b,c},3]/_N1}, Constraint: s(_N1), s(S), integer(C)
        IntLVar C = new IntLVar("C");

        LSet set = LSet.empty().ins("a").ins("b").ins("c");

        IntLVar N = new IntLVar("N");

        LSet D = new LSet("D");
        LSet S = new LSet("S");
        Ris ris = new Ris(new LPair(S, C), D, S.size(C).and(C.gt(1)));

        assertEquals(true, ris.contains(new LPair(set, N)).check());
        assertEquals((Integer)3, N.getValue());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{({c,b,a},3)/_N9936}", D.toString());
        assertTrue(comparator.areEqualInSameOrder());

    }

    //@Test
    public void test132D() {
        //{log}=> [{a},N] in ris([S,C] in D,size(S,C) & C > 1). no
        IntLVar a = new IntLVar("a");
        IntLVar N = new IntLVar("N");
        LList pair = new LPair(LSet.empty().ins(a), N);

        IntLVar C = new IntLVar("C");
        LSet S = new LSet("S");
        LList pairsc =  new LPair(S, C);
        LSet D = new LSet();

        Ris ris = new Ris(pairsc, D, S.size(C).and(C.gt(1)));

        assertEquals(false, ris.contains(pair).check());
    }

    //@Test
    public void test132Dbis() {
        //{log}=> [{a, b},N] in ris([S,C] in D,size(S,C) & C > 1). yes
        IntLVar a = new IntLVar("a");
        IntLVar b = new IntLVar("b");

        IntLVar N = new IntLVar("N");
        LList pair = new LPair(LSet.empty().ins(a).ins(b), N);

        IntLVar C = new IntLVar("C");
        LSet S = new LSet("S");
        LList pairsc =  new LPair(S, C);
        LSet D = new LSet();

        Ris ris = new Ris(pairsc, D, S.size(C).and(C.gt(1)));

        assertEquals(true, ris.contains(pair).check());
        assertEquals((Integer)2, N.getValue());
    }


    //@Test
    public void test132E() {
        //%cross product of sets A and B
        //{log}=> A = {a,b} & B = {1,2} & [a,2] in ris([X,Y] in D,X in A & Y in B).
        //A = {a,b}, B = {1,2}, D = {[a,2]/_N1}, Constraint: s(_N1)
        IntLVar a = new IntLVar("a");
        IntLVar b = new IntLVar("b");
        LSet A = LSet.empty().ins(a).ins(b);
        LSet B = LSet.empty().ins(1).ins(2);

        LVar x = new LVar("x");
        LVar y = new LVar("y");

        LSet D = new LSet("D");
        Ris ris = new Ris(new LPair(x, y), D, x.in(A).and(y.in(B)));

        assertEquals(true, ris.contains(new LPair(a, 2)).and(A.size(2)).check()); //senza a.size(2) deduce che la coppia [_b,2] sta nel dominio invece di [_a,2], credo perch� suppone a=b
        ConstraintStringComparator comparator = new ConstraintStringComparator("{(_a,2)/_N10000}", D.toString());
        assertTrue(comparator.areEqualInSameOrder());

    }

    //@Test
    public void test132Ebis() {
        //%cross product of sets A and B
        //{log}=> A = {a,b} & B = {1,2} & [a,2] in ris([X,Y] in D,X in A & Y in B).
        //A = {a,b}, B = {1,2}, D = {[a,2]/_N1}, Constraint: s(_N1)
        LVar a = new LVar("a", "a");
        LVar b = new LVar("b", "b");
        LSet A = LSet.empty().ins(a).ins(b);
        LSet B = LSet.empty().ins(1).ins(2);

        LVar x = new LVar("x");
        LVar y = new LVar("y");

        LSet D = new LSet("D");
        Ris ris = new Ris(new LPair(x, y), D, x.in(A).and(y.in(B)));
        assertEquals(true, ris.contains(new LPair("a", 2)).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{(a,2)/_N14063}", D.toString());
        assertTrue(comparator.areEqualInSameOrder());

    }

    //@Test
    public void test132Etris() {
        //%cross product of sets A and B
        //{log}=> A = {a,b} & B = {1,2} & [a,2] in ris([X,Y] in D,X in A & Y in B).
        //A = {a,b}, B = {1,2}, D = {[a,2]/_N1}, Constraint: s(_N1)
        LVar a = new LVar("a", "a");
        LVar b = new LVar("b", "b");
        LSet A = LSet.empty().ins(a).ins(b);
        LSet B = LSet.empty().ins(1).ins(2);

        LVar x = new LVar("x");
        LVar y = new LVar("y");

        LSet D = new LSet("D");
        Ris ris = new Ris(new LPair(x, y), D, x.in(A).and(y.in(B)));
        assertEquals(false, ris.contains(new LPair("c", 2)).check());

    }

    //@Test
    public void test132F() {
        //%cross product of sets A and B
        //{log}=> [a,2] in ris([X,Y] in D,X in A & Y in B).
        //D = {[a,2]/_N3}, A = {a/_N2}, B = {2/_N1}, Constraint: s(_N3), s(_N2), s(_N1)

        LSet A = new LSet("A");
        LSet B = new LSet("B");

        LVar x = new LVar("x");
        LVar y = new LVar("y");

        LSet D = new LSet("D");
        Ris ris = new Ris(new LPair(x, y), D, x.in(A).and(y.in(B)));

        assertEquals(true, ris.contains(new LPair("a", 2)).check()); //senza a.size(2) deduce che la coppia [_b,2] sta nel dominio invece di [_a,2], credo perch� suppone a=b
        ConstraintStringComparator c1 = new ConstraintStringComparator("{(a,2)/_N10035}",D.toString());
        assertTrue(c1.areEqualInSameOrder());
        ConstraintStringComparator c2 = new ConstraintStringComparator("{a/_N10027}", A.toString());
        ConstraintStringComparator c3 = new ConstraintStringComparator("{2/_N10028}",B.toString());
        assertTrue(c2.areEqualInSameOrder());
        assertTrue(c3.areEqualInSameOrder());

    }

    //@Test
    public void test21A() {
//		% Ex. 6.d art. CADE 2017 [testExample06D in jsetl � OK]
//				{log}=> ris(X in {2,4/M},true,2*X) = {2,4,6,8}.
//				M = {1,3/_N1}, Constraint: 1 nin _N1, 3 nin _N1, ris(X in _N1,[],true,2*X,true)={}
//				Another solution? (y/n)
//				M = {1,3,4/_N1}, Constraint: 1 nin _N1, 3 nin _N1, 4 nin _N1, ris(X in _N1,[],true,2*X,true)={}
//				Another solution? (y/n)
//				M = {1,2,3/_N1}, Constraint: 1 nin _N1, 2 nin _N1, 3 nin _N1, ris(X in _N1,[],true,2*X,true)={}
//				Another solution? (y/n)
//				M = {1,2,3,4/_N1}, Constraint: 1 nin _N1, 2 nin _N1, 3 nin _N1, 4 nin _N1, ris(X in _N1,[],true,2*X,true)={}
//				Another solution? (y/n)
//				no ==> OK

        IntLVar x = new IntLVar("x");
        Integer[] elems = {2,4,6,8};
        IntLSet sol = IntLSet.empty().insAll(elems);
        IntLSet dom = new IntLSet().ins(4).ins(2);
        Ris ris = new Ris(x, dom, x.eq(x), x.mul(2));

        assertEquals(true, sol.eq(ris).check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{1..4/_N4484}", dom.toString());
        assertTrue(comparator.areEqualInSameOrder());



    }

    //@Test
    public void test21B() {
        //{log}=> D={1,2,3} & S is ris(X in D,X<3,X*X).
        //		D = {1,2,3}, S = {4,1}
        IntLSet dom = IntLSet.empty().ins(3).ins(2).ins(1);
        IntLVar x = new IntLVar("x");

        Ris ris = new Ris(x, dom, x.lt(3), x.mul(x));
        IntLSet s = new IntLSet("s").ins(1).ins(4);
        assertEquals(true, s.eq(ris).check());
    }

    //@Test
    public void test21C() {
        IntLSet s = IntLSet.empty().ins(1).ins(4);
        IntLVar x = new IntLVar("x");
        IntLSet d = new IntLSet();

        Ris ris = new Ris(x,d,x.lt(3), x.mul(x));
        Solver solver = new Solver();
        solver.add(s.eq(ris));
        assertEquals(true, solver.check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{_N4538,_N4546/_N4545}", d.toString());
        assertTrue(comparator.areEqualInSameOrder());
        Object[] lvars = d.getValue().toArray();
        IntLVar lvar0 = (IntLVar) lvars[0];
        IntLVar lvar1 = (IntLVar) lvars[1];
        if(lvar0.getDomain().toString().equals("{-1, 1}"))
            assertEquals("{-2, 2}", lvar1.getDomain().toString());
        else if(lvar0.getDomain().toString().equals("{-2, 2}"))
            assertEquals("{-1, 1}", lvar1.getDomain().toString());
        else
            fail();


        assertEquals(false, solver.nextSolution());

    }


    //@Test
    public void test21D() throws Failure{
//		%If R is a s of ordered pairs and D is a s, then the subset of R where all first component
//		%belong to D is:
//		{log}=> D={2} & R={[2,a],[1,b],[2,c]} & {[2,a],[2,c]} = ris(X in D,[Y],[X,Y] in R,[X,Y]).

        LList apair = new LPair(2, "a");
        LList bpair = new LPair(1, "b");
        LList cpair = new LPair(2, "c");
        LSet R = LSet.empty().ins(apair).ins(bpair).ins(cpair);
        IntLSet D = IntLSet.empty().ins(2);

        LSet sol = LSet.empty().ins(cpair).ins(apair);

        IntLVar x = new IntLVar();
        LVar y = new LVar();
        Solver solver = new Solver();
        LList pairxy = new LPair(x, y);
        Ris ris = new Ris(x, D, R.contains(pairxy),pairxy,y);


        solver.add(sol.eq(ris));
        solver.showStoreAll();

        assertEquals(true, solver.check());


    }

    //@Test
    public void test21Dbis() {
//		%If R is a s of ordered pairs and D is a s, then the subset of R where all first component
//		%belong to D is:
//		{log}=> D={2} & R={[2,a],[1,b],[2,c]} & {[2,a],[2,c]} = ris(X in D,[Y],[X,Y] in R,[X,Y]).

        LList apair = new LPair(2, "a");
        LList bpair = new LPair(1, "b");
        LList cpair = new LPair(2, "c");
        LSet R = LSet.empty().ins(apair).ins(bpair).ins(cpair);
        IntLSet D = IntLSet.empty().ins(2);

        LSet sol = LSet.empty().ins(cpair).ins(apair);

        IntLVar x = new IntLVar();
        LVar y = new LVar("y");
        Solver solver = new Solver();
        LList pairxy = new LPair(x, y);
        Ris ris = new Ris(pairxy, R, D.contains(x));
        //solver.add(sol.eq(ris));
        //solver.add(y.label());
        solver.add(ris.contains(apair));

        assertEquals(true, solver.check());


    }

    //@Test
    public void test21Dtris() {
//		%If R is a s of ordered pairs and D is a s, then the subset of R where all first component
//		%belong to D is:
//		{log}=> D={2} & R={[2,a],[1,b],[2,c]} & {[2,a],[2,c]} = ris(X in D,[Y],[X,Y] in R,[X,Y]).

        LList apair = new LPair(2, "a");
        LList bpair = new LPair(1, "b");
        LList cpair = new LPair(2, "c");
        LSet R = LSet.empty().ins(apair).ins(bpair).ins(cpair);
        IntLSet D = IntLSet.empty().ins(2);

        LSet sol = LSet.empty().ins(cpair).ins(bpair);

        IntLVar x = new IntLVar();
        LVar y = new IntLVar();
        Solver solver = new Solver();
        LList pairxy = new LPair(x, y);
        Ris ris = new Ris(pairxy, R, D.contains(x));

        solver.add(sol.eq(ris));
        //solver.add(y.label());

        assertEquals(false, solver.check());


    }

    //@Test
    public void test21Dfalse() throws Failure{
//		%If R is a s of ordered pairs and D is a s, then the subset of R where all first component
//		%belong to D is:
//		{log}=> D={2} & R={[2,a],[1,b],[2,c]} & {[2,a],[2,c]} = ris(X in D,[Y],[X,Y] in R,[X,Y]).

        LList apair = new LPair(3, "a");
        LList bpair = new LPair(1, "b");
        LList cpair = new LPair(3, "c");
        LSet R = LSet.empty().ins(apair).ins(bpair).ins(cpair);
        IntLSet D = IntLSet.empty().ins(2);

        LSet sol = LSet.empty().ins(cpair).ins(apair);

        IntLVar x = new IntLVar();
        LVar y = new LVar();
        Solver solver = new Solver();
        LList pairxy = new LPair(x, y);
        Ris ris = new Ris(x, D, R.contains(pairxy).and(y.eq(y)),pairxy,y);
        System.out.println("FILTER " + ris.getFilter());
        Constraint cc = new Constraint();

        Constraint ccc = new Constraint();

        System.out.println("FILTER "+ris.getFilter());


        solver.add(sol.eq(ris));
        solver.showStoreAll();
        System.out.println(ris.expand());
        assertEquals(false, solver.check());


    }

    //@Test
    public void test21Dfalse2() throws Failure{
//		%If R is a s of ordered pairs and D is a s, then the subset of R where all first component
//		%belong to D is:
//		{log}=> D={2} & R={[2,a],[1,b],[2,c]} & {[2,a],[2,c]} = ris(X in D,[Y],[X,Y] in R,[X,Y]).

        LList apair = new LPair(3, "a");
        LList bpair = new LPair(1, "b");
        LList cpair = new LPair(3, "c");
        LSet R = LSet.empty().ins(apair).ins(bpair).ins(cpair);
        IntLSet D = IntLSet.empty().ins(2);

        LSet sol = LSet.empty().ins(cpair).ins(apair);

        IntLVar x = new IntLVar();
        LVar y = new LVar();
        Solver solver = new Solver();
        LList pairxy = new LPair(x, y);
        Ris ris = new Ris(x, D, R.contains(pairxy).and(y.eq(y)),pairxy,y);
        Constraint cc = new Constraint();
        Constraint ccc = new Constraint();


        solver.add(ris.contains(new LPair(3, y)));
        solver.showStoreAll();
        System.out.println(ris.expand());
        assertEquals(false, solver.check());


    }

    public void testRossiNonFunzionaNonEsistenziale01()
            throws Failure {
        IntLVar x = new IntLVar("x");
        LSet D = LSet.empty().ins(2).ins(1).setName("D");  // D = {1,2}
        D.output();
        LSet R = LSet.empty().ins(new LPair(1,5)).setName("R");  //R={[1,5]}
        R.output();
        IntLVar y = new IntLVar("y");
        LList P = new LPair(x,y).setName("P");  //P=[x,y]
        P.output();

        Ris ris = new Ris(x,D,R.contains(P),P);   //ris = { x : {1,2} | [x,y] in {[1,5]} @ [x,y] }
        ris.output();

        LList P1 = new LPair(1,5).setName("P1");  //P1=[1,5]
        P1.output();
        Solver solver = new Solver();
        assertEquals(true, solver.check(ris.contains(P1)));           // [1,5]  in ris
        assertEquals("(_x,5)", P.toString());
    }

    public void testRossiNonFunzionaNonEsistenziale02()
            throws Failure {
        IntLVar x = new IntLVar("x");
        LSet D = LSet.empty().ins(2).ins(3).setName("D");  // D = {1,2}
        D.output();
        LSet R = LSet.empty().ins(new LPair(1,5)).setName("R");  //R={[1,5]}
        R.output();
        IntLVar y = new IntLVar("y");
        LList P = new LPair(x,y).setName("P");  //P=[x,y]
        P.output();

        Ris ris = new Ris(x,D,R.contains(P),P);   //ris = { x : {1,2} | [x,y] in {[1,5]} @ [x,y] }
        ris.output();

        LList P1 = new LPair(1,5).setName("P1");  //P1=[1,5]
        P1.output();
        Solver solver = new Solver();
        assertEquals(false, solver.check(ris.contains(P1)));           // [1,5]  in ris
    }

    public void testRossiNonFunzionaNonEsistenziale03()
            throws Failure {
        IntLVar x = new IntLVar("x");
        LSet D = LSet.empty().ins(1).ins(2).setName("D");  // D = {1,2}
        D.output();
        LSet R = LSet.empty().ins(new LPair(1,5)).setName("R");  //R={[1,5]}
        R.output();
        IntLVar y = new IntLVar("y");
        LList P = new LPair(x,y).setName("P");  //P=[x,y]
        P.output();

        Ris ris = new Ris(x,D,R.contains(P),P);   //ris = { x : {1,2} | [x,y] in {[1,5]} @ [x,y] }
        ris.output();

        LList P1 = new LPair(1,5).setName("P1");  //P1=[1,5]
        P1.output();
        Solver solver = new Solver();
        assertEquals(true, solver.check(ris.contains(P1)));           // [1,5]  in ris
        assertEquals("(_x,5)", P.toString());
    }


    public void testRossiNonFunzionaEsistenziale01()
            throws Failure {
        IntLVar x = new IntLVar("x");
        LSet D = LSet.empty().ins(2).ins(1).setName("D");  // D = {1,2}
        D.output();
        LSet R = LSet.empty().ins(new LPair(1,5)).setName("R");  //R={[1,5]}
        R.output();
        IntLVar y = new IntLVar("_y");
        LList P = new LPair(x,y).setName("P");  //P=[x,y]
        P.output();

        Ris ris = new Ris(x,D,R.contains(P),P);   //ris = { x : {1,2} | [x,y] in {[1,5]} @ [x,y] }
        ris.output();

        LList P1 = new LPair(1,5).setName("P1");  //P1=[1,5]
        P1.output();
        Solver solver = new Solver();
        assertEquals(true, solver.check(ris.contains(P1)));           // [1,5]  in ris
        assertEquals("(_x,5)", P.toString());
    }

    public void testRossiNonFunzionaEsistenziale02()
            throws Failure {
        IntLVar x = new IntLVar("x");
        LSet D = LSet.empty().ins(2).ins(3).setName("D");  // D = {1,2}
        D.output();
        LSet R = LSet.empty().ins(new LPair(1,5)).setName("R");  //R={[1,5]}
        R.output();
        IntLVar y = new IntLVar("_y");
        LList P = new LPair(x,y).setName("P");  //P=[x,y]
        P.output();

        Ris ris = new Ris(x,D,R.contains(P),P);   //ris = { x : {1,2} | [x,y] in {[1,5]} @ [x,y] }
        ris.output();

        LList P1 = new LPair(1,5).setName("P1");  //P1=[1,5]
        P1.output();
        Solver solver = new Solver();
        assertEquals(false, solver.check(ris.contains(P1)));           // [1,5]  in ris
    }

    public void testRossiNonFunzionaEsistenziale03()
            throws Failure {
        IntLVar x = new IntLVar("x");
        LSet D = LSet.empty().ins(1).ins(2).setName("D");  // D = {1,2}
        D.output();
        LSet R = LSet.empty().ins(new LPair(1,5)).setName("R");  //R={[1,5]}
        R.output();
        IntLVar y = new IntLVar("_y");
        LList P = new LPair(x,y).setName("P");  //P=[x,y]
        P.output();

        Ris ris = new Ris(x,D,R.contains(P),P);   //ris = { x : {1,2} | [x,y] in {[1,5]} @ [x,y] }
        ris.output();

        LList P1 = new LPair(1,5).setName("P1");  //P1=[1,5]
        P1.output();
        Solver solver = new Solver();
        assertEquals(true, solver.check(ris.contains(P1)));           // [1,5]  in ris
        assertEquals("(_x,5)", P.toString());
    }



    public void testNotExistentialVariables1(){
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        Integer elements[] = {10,4,7,88,99,12,3,2,999,1000,11,8,9}; //min = 2
        IntLSet S = new IntLSet(new HashSet<>(Arrays.asList(elements)));

        Ris ris = new Ris(x, S, x.ge(y).and(y.in(S)));

        assertEquals(true, S.eq(ris).check());
        assertEquals("2", y.toString());
    }

    public void testNotExistentialVariables2() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");

        Integer elements[] = {10,4,7,88,99,12,3,2,999,1000,11,8,9}; //min = 2
        IntLSet S = new IntLSet(new HashSet<>(Arrays.asList(elements)));

        Ris ris = new Ris(x, S, y.in(S), LList.empty().ins(y).ins(x));
        try {
            ris.expand();
            fail();
        }catch(IllegalStateException illegalStateException){
            //nothing to do
        }
        IntLVar yI = new IntLVar("y");
        Ris ris2 = new Ris(x, S, yI.in(S), LList.empty().ins(yI).ins(x),yI);
        //this is wrong but still shows that not existential variables are not treated as if they were existential
        assertEquals("{[2,2],[2,99],[2,3],[2,4],[2,7],[2,999],[2,1000],[2,8],[2,9],[2,10],[2,11],[2,12],[2,88],[99,2],[99,99],[99,3],[99,4],[99,7],[99,999],[99,1000],[99,8],[99,9],[99,10],[99,11],[99,12],[99,88],[3,2],[3,99],[3,3],[3,4],[3,7],[3,999],[3,1000],[3,8],[3,9],[3,10],[3,11],[3,12],[3,88],[4,2],[4,99],[4,3],[4,4],[4,7],[4,999],[4,1000],[4,8],[4,9],[4,10],[4,11],[4,12],[4,88],[7,2],[7,99],[7,3],[7,4],[7,7],[7,999],[7,1000],[7,8],[7,9],[7,10],[7,11],[7,12],[7,88],[999,2],[999,99],[999,3],[999,4],[999,7],[999,999],[999,1000],[999,8],[999,9],[999,10],[999,11],[999,12],[999,88],[1000,2],[1000,99],[1000,3],[1000,4],[1000,7],[1000,999],[1000,1000],[1000,8],[1000,9],[1000,10],[1000,11],[1000,12],[1000,88],[8,2],[8,99],[8,3],[8,4],[8,7],[8,999],[8,1000],[8,8],[8,9],[8,10],[8,11],[8,12],[8,88],[9,2],[9,99],[9,3],[9,4],[9,7],[9,999],[9,1000],[9,8],[9,9],[9,10],[9,11],[9,12],[9,88],[10,2],[10,99],[10,3],[10,4],[10,7],[10,999],[10,1000],[10,8],[10,9],[10,10],[10,11],[10,12],[10,88],[11,2],[11,99],[11,3],[11,4],[11,7],[11,999],[11,1000],[11,8],[11,9],[11,10],[11,11],[11,12],[11,88],[12,2],[12,99],[12,3],[12,4],[12,7],[12,999],[12,1000],[12,8],[12,9],[12,10],[12,11],[12,12],[12,88],[88,2],[88,99],[88,3],[88,4],[88,7],[88,999],[88,1000],[88,8],[88,9],[88,10],[88,11],[88,12],[88,88]}", ris2.expand().toString());

    }

    //@Test
    public void testvoidvoidvoid() {
        IntLSet domain = new IntLSet();
        Ris ris1 = new Ris(new IntLVar(), domain, Constraint.truec());
        assertEquals(true, LSet.empty().union(LSet.empty(), ris1).check());
        //But it doesn't deduce anything about the domain.
        System.out.println(domain);
    }

    //@Test
    public void testUnion01() {
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        IntLVar x = new IntLVar();
        Ris ris = new Ris(x, domain, x.mod(2).eq(0));
        assertEquals(true, ris.union(LSet.empty(), IntLSet.empty().ins(2)).check());

    }

    //@Test
    public void testUnion02() {
        IntLSet domain = IntLSet.empty().ins(1).ins(2).ins(3);
        IntLVar x = new IntLVar();
        Ris ris = new Ris(x, domain, x.mod(2).eq(0));
        assertEquals(false, ris.union(LSet.empty(), IntLSet.empty().ins(3)).check());

    }

    //@Test
    public void testUnion03() {
        IntLSet domain = new IntLSet();
        IntLVar x = new IntLVar();
        Ris ris = new Ris(x, domain, x.mod(2).eq(1));
        assertEquals(true, IntLSet.empty().union( IntLSet.empty().ins(3), ris).check());

        ConstraintStringComparator comparator = new ConstraintStringComparator("{3/_N10272}", domain.toString());
        assertTrue(comparator.areEqualInSameOrder());

    }

    //@Test
    public void testUnion03bis() {
        IntLSet domain = new IntLSet();
        IntLVar x = new IntLVar();
        Ris ris = new Ris(x, domain, x.mod(2).eq(0));
        assertEquals(false, LSet.empty().union(IntLSet.empty().ins(3), ris).check());
    }


    //@Test
    public void testUnion04() {
        IntLSet domain1 = IntLSet.empty().ins(1).ins(0).ins(2).ins(3).ins(1000);
        IntLVar x1 = new IntLVar();
        Ris ris1 = new Ris(x1, domain1, x1.mod(2).eq(0));

        IntLSet domain2 = IntLSet.empty().ins(1001).ins(0).ins(2).ins(101);
        IntLVar x2 = new IntLVar();
        Ris ris2 = new Ris(x2, domain2, x2.mod(2).eq(0));

        IntLSet domain3 = IntLSet.empty().ins(1000).ins(0).ins(2).ins(101).ins(900);
        IntLVar x3 = new IntLVar();
        Ris ris3 = new Ris(x3, domain3, x3.mod(2).eq(0).and(x3.neq(900)));


        assertEquals(true, ris1.union(ris2,ris3).check());
    }

    //@Test
    public void testUnion04bis() {
        IntLSet domain1 = IntLSet.empty().ins(1).ins(0).ins(1000);
        IntLVar x1 = new IntLVar();
        Ris ris1 = new Ris(x1, domain1, x1.mod(2).eq(0));

        IntLSet domain2 = IntLSet.empty().ins(0).ins(101);
        IntLVar x2 = new IntLVar();
        Ris ris2 = new Ris(x2, domain2, x2.mod(2).eq(0));

        IntLSet domain3 = IntLSet.empty().ins(1000).ins(0).ins(101).ins(902);
        IntLVar x3 = new IntLVar();
        Ris ris3 = new Ris(x3, domain3, x3.mod(2).eq(0));


        assertEquals(false, ris1.union(ris2,ris3).check());

    }


    //@Test
    public void testUnion05() {
        IntLSet domain = new IntLSet().ins(4);
        IntLVar x = new IntLVar();
        Ris ris = new Ris(x, domain, x.mod(2).eq(0));

        IntLSet domain3 = new IntLSet().ins(1000).ins(0).ins(2).ins(101).ins(902);
        IntLVar x3 = new IntLVar();
        Ris ris3 = new Ris(x3, domain3, x3.mod(2).eq(0).and(x3.neq(900)));
        Solver solver = new Solver();
        solver.add(ris3.union(new IntLSet().ins(6), ris));
        assertEquals(true, solver.check());
        ConstraintStringComparator comparator = new ConstraintStringComparator("{0, 2, 4, 6, 902, 1000/_N10533}", domain.toString());
        assertTrue(comparator.areEqualInSameOrder());
    }

    //@Test
    public void testUnion05false() {
        IntLSet domain = new IntLSet().ins(4);
        IntLVar x = new IntLVar();
        Ris ris = new Ris(x, domain, x.mod(2).eq(0));

        IntLSet domain3 = new IntLSet().ins(1000).ins(0).ins(2).ins(101).ins(901);
        IntLVar x3 = new IntLVar();
        Ris ris3 = new Ris(x3, domain3, x3.mod(2).eq(1).or(x3.eq(901)));
        Solver solver = new Solver();
        solver.add(ris3.union(new IntLSet().ins(6), ris));

        assertEquals(false, solver.check());
    }


    //@Test
    public void testUnion06() {

        Ris ris1 = new Ris(new LVar(),  new LSet(), Constraint.truec());
        Ris ris2 = new Ris(new LVar(),  new LSet(), Constraint.truec());

        Ris ris3 = new Ris(new LVar(),  new LSet(), Constraint.truec());

        assertEquals(true, ris1.union(ris2, ris3).check());


    }

    //@Test
    public void testUnion07() {
        IntLVar a = new IntLVar("a");
        IntLSet domain1 = IntLSet.empty().ins(1).ins(0).ins(a).ins(3).ins(1000);
        IntLVar x1 = new IntLVar();
        Ris ris1 = new Ris(x1, domain1, x1.mod(2).eq(0));

        IntLSet domain2 = IntLSet.empty().ins(1001).ins(0).ins(2).ins(101);
        IntLVar x2 = new IntLVar();
        Ris ris2 = new Ris(x2, domain2, x2.mod(2).eq(0));

        IntLSet domain3 = IntLSet.empty().ins(1000).ins(0).ins(2).ins(101).ins(900);
        IntLVar x3 = new IntLVar();
        Ris ris3 = new Ris(x3, domain3, x3.mod(2).eq(0).and(x3.neq(900)));


        assertEquals(true, ris2.union(ris3,ris1).check());
        assertEquals((Integer)2, a.getValue());
    }

    public void testUnion09(){
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        IntLSet D = new IntLSet("D");

        Ris ris1 = new Ris(x, D, x.lt(2).and(x.gt(-2).and(x.mod(2).eq(0))));
        Ris ris2 = new Ris(y, D, x.lt(2).and(x.gt(-2).and(x.mod(2).eq(1))));
        IntLSet zero = IntLSet.empty().ins(0);

        Solver solver = new Solver();
        solver.add(zero.union(ris2,ris1));
        assertEquals(true, solver.check());
    }

    //@Test
    public void testWithPair01() {
        //ris ([ X,Y ] in D,Y is X * X )
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        LPair pair = new LPair(x,y);
        LSet domain = LSet.empty()
                .ins(new LPair(3,9))
                .ins(new LPair(4,5))
                .ins(new LPair(-10,100))
                .ins(new LPair(6,3))
                .ins(new LPair(1,1))
                .ins(new LPair(0,0));
        Ris ris = new Ris(pair, domain, y.eq(x.mul(x)));
        LList onesol =new LPair(new IntLVar(),new IntLVar());

        Solver solver = new Solver();
        solver.add(ris.contains(onesol));

        assertEquals(true, solver.check());
        assertEquals("(0,0)", onesol.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("(1,1)", onesol.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("(-10,100)", onesol.toString());
        assertEquals(true, solver.nextSolution());
        assertEquals("(3,9)", onesol.toString());
        assertEquals(false, solver.nextSolution());
    }

    //@Test
    public void testWithPair02() {
        // (5 ,y ) in { x : D | x > 0 • ( x,x ∗ x ) } , where D is a variable

        IntLVar y = new IntLVar();
        LList p = new LPair(5, y);

        //LList p = new LPair(5, y);

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D");

        LPair px = new LPair(x, x.mul(x));

        Ris ris = new Ris(x,domain, x.gt(0), px);


        Solver solver = new Solver();
        solver.add(ris.contains(p));

        assertEquals(true, solver.check());

        int yval = y.getValue();

        assertEquals(yval, 5*5);
    }

    //@Test
    public void testWithPair03() {
        // (5 ,y ) in { x : D | x > 0 • ( x,x ∗ x ) } , where D is a variable

        IntLVar y = new IntLVar();
        LPair p = new LPair(-5, y);

        //LList p = new LPair(5, y);

        IntLVar x = new IntLVar("x");
        IntLSet domain = new IntLSet("D");

        LPair px = new LPair(x, x.mul(x));

        Ris ris = new Ris(x,domain, x.gt(0), px);


        Solver solver = new Solver();
        solver.add(ris.contains(p));

        assertEquals(false, solver.check());

    }

    //@Test
    public void testExample01() {
        //{ x : [ − 10 , 10] | x mod 2 = 0 }
        IntLVar x = new IntLVar("x");
        LSet domain = new LSet(new Interval(-10, 10).toSet());
        Constraint f = x.mod(2).eq(0);
        Ris ris = new Ris(x, domain, f);

        Solver solver = new Solver();
        LSet sol = new LSet();
        solver.add(sol.eq(ris));

        assertEquals(true, solver.check());
        System.out.println(sol);
        assertEquals("{-10,-8,-6,-4,-2,0,2,4,6,8,10}", sol.toString());

    }

    //@Test
    public void testExamples02() {
        //{ x : D | x ≥ 5 and x ≤ 10 @ x * x }
        IntLVar x = new IntLVar("x");
        Constraint f = x.le(10).and(x.ge(5));
        LSet d = LSet.empty().ins(2).ins(5).ins(-1).ins(6);//new LSet("D");
        IntLVar p = x.mul(x);
        Ris ris = new Ris(x, d, f, p);

        Solver solver = new Solver();

        LSet sol = new LSet();

        solver.add(sol.eq(ris));

        assertEquals(true,solver.check());
        assertEquals("{36,25}", sol.toString());

    }

    //@Test
    public void testExamples03() throws Failure {
        //{ x : D | x ≥ 5 and x ≤ 10 @ [ x,x ∗ x ] }
        IntLVar x = new IntLVar("x");
        Constraint f = x.le(10).and(x.ge(5));
        LSet d = LSet.empty().ins(2).ins(5).ins(-1).ins(6);
        IntLVar z = x.mul(x);
        LList p = LList.empty().ins(z).ins(x);
        Ris ris = new Ris(x, d, f, p);

        Constraint cc = new Constraint();
        System.out.println(cc);

        Solver solver = new Solver();

        LSet sol = new LSet();

        solver.add(sol.eq(ris));

        assertEquals(true,solver.check());
        assertEquals("{[6,36],[5,25]}", sol.toString());
    }

    //@Test
    public void testExamples04() {
        LSet x = new LSet("x");
        Constraint f = x.size(2);
        LSet d = LSet.empty()  //new LSet("D");
                .ins(LSet.empty().ins(1).ins(2).ins(3))
                .ins(LSet.empty().ins(-1).ins(1))
                .ins(LSet.empty())
                .ins(LSet.empty().ins(55).ins("ciao"));
        Ris ris = new Ris(x, d, f);

        Solver solver = new Solver();

        LSet sol = new LSet();

        solver.add(sol.eq(ris));

        assertEquals(true,solver.check());
        System.out.println(sol);
        assertEquals("{{ciao,55},{1,-1}}", sol.toString());
    }

    //@Test
    public void testExamples05(){
        //{ x : D | x.neq ([2 , 1])  @ x }
        LList x = new LList("x");
        Constraint f = x.neq(LList.empty().ins(1).ins(2));
        LSet d = LSet.empty()//new LSet("D");
                .ins(LList.empty().ins(5).ins(3).ins("ciao"))
                .ins(LList.empty())
                .ins(LList.empty().ins(1))
                .ins(LList.empty().ins(0).ins(-4))
                .ins(LList.empty().ins(1).ins(2))
                .ins(LList.empty().ins(2).ins(1));
        Ris ris = new Ris(x, d, f);//nella tesi c'è anche p come 4° parametro, che non ci vuole


        Solver solver = new Solver();

        LSet sol = LSet.empty()
                .ins(LList.empty().ins(5).ins(3).ins("ciao"))
                .ins(LList.empty())
                .ins(LList.empty().ins(1))
                .ins(LList.empty().ins(0).ins(-4))
                .ins(LList.empty().ins(2).ins(1));
        solver.add(sol.eq(ris));

        assertEquals(true,solver.check());
        assertEquals("{[1,2],[-4,0],[1],[],[ciao,3,5]}", sol.toString());
    }

    //@Test
    public void testExamples06() {
        // {x : [1, 30] | x mod 6 = 0 @ x}
        //{24, 18, 12, 6, 30}

        IntLVar x = new IntLVar("x");
        Constraint f = x.mod(6).eq(0);
        LSet d = new LSet(new Interval(1, 30).toSet());
        Ris ris = new Ris(x, d, f);
        LSet s = LSet.empty().ins(30).ins(6).ins(12).ins(18).ins(24);
        assertEquals(true,ris.equals(s)); //true
    }

    //@Test
    public void testExamples07() throws Failure {
        IntLVar x = new IntLVar("x");
        Constraint f =
                x.le(-15).and(x.ge(-25)).or(x.le(25).and(x.ge(15)));
        LSet d = new LSet(new Interval(-50, 50).toSet());
        IntLVar p = x.mul(x);
        Ris ris = new Ris(x, d, f, p);
        //{x : D | (x<=-15 && x>=-25) || (x<=25 && x>=15) @ x*x }
        LSet S = ris.expand();
        //?={625,576,529,484,441,400,361,324,289,256,225}
        System.out.println(S);
        assertEquals(11,S.getSize()); // 11 = 25 - 15 + 1
        assertEquals("{625,576,529,484,441,400,361,324,289,256,225}", S.toString());
    }

    //@Test
    public void testExamples08() {
        IntLVar x = new IntLVar("x");
        LSet set = new LSet(new Interval(0, 10).toSet());
        Constraint f = x.mod(2).eq(0);
        IntLVar p = x.mul(2);
        Ris ris = new Ris(x, set, f, p);
        // {x : [0, 10] | x mod 2 = 0 @ x*2}}
        IntLVar y = new IntLVar("y");
        LSet set2 =	LSet.empty().ins(20).ins(16).ins(12).ins(8).ins(4).ins(0);
        // {0, 4, 8, y, 16, 20 }
        Solver solver = new Solver();
        solver.add(ris.eq(set2));
        // {x : [0, 10] | x mod 2 = 0 @ x*2}} = {0, 4, 8, 12, 16, 20}
        assertEquals(true,solver.check());
    }

    //@Test
    public void testExamples09() {
        IntLVar x = new IntLVar("x");
        IntLVar d = new IntLVar("d");
        LSet set = LSet.empty().ins(d).ins(4).ins(8);
        Constraint f = x.ge(0);
        IntLVar p = x.mul(2);
        Ris ris = new Ris(x, set, f, p);

        // {x : {8,4,d} | x mod 2 = 0 @ x*2}}
        IntLVar y = new IntLVar("y");
        LSet set1 = new LSet(new Interval(-5, 5).toSet());
        Constraint f1 = x.mod(2).eq(0);
        Ris ris1 = new Ris(y, set1, f1);
        // {y : [-5, 5] | x mod 2 = 0 @ x }
        Solver solver = new Solver();
        solver.add(ris.neq(ris1));;
        //{x : {8,4,d} | x mod 2 = 0 @ x*2}} != {y : [-5, 5] | x mod 2 = 0 @ x }
        assertEquals(true, solver.check());
    }

    //@Test
    public void testExamples10() {
        IntLVar y = new IntLVar("y");
        IntLVar x = new IntLVar("x");
        LSet S = LSet.empty().ins(4).ins(7).ins(9).ins(8);
        Constraint f = x.ge(y);
        Ris ris = new Ris(x, S, f);
        Solver solver = new Solver();
        solver.add(y.in(S));
        solver.add(ris.eq(S));
        // y in S AND S = {x : S | x >= y @ x}
        assertEquals(true, solver.check());
    }

    //@Test
    public void testExamples11() {
        LVar r1 = new LVar("r1");
        LVar r2 = new LVar("r2");
        LVar r3 = new LVar("r3");
        LVar r4 = new LVar("r4");
        LVar r5 = new LVar("r5");
        LVar r6 = new LVar("r6");
        LVar[] regionsArray = {r1,r2,r3,r4,r5,r6};
        LSet regions = LSet.empty().insAll(regionsArray);
        LSet[] mapArray =
                {LSet.empty().ins(r1).ins(r2).ins(r3),LSet.empty().ins(r2).ins(r3).ins(r4),
                        LSet.empty().ins(r3).ins(r4).ins(r5),
                        LSet.empty().ins(r3).ins(r5).ins(r6)};
        LSet map = LSet.empty().insAll(mapArray);
        String[] colorsArray = {"Red","Blue", "Green"};
        LSet colors = LSet.empty().insAll(colorsArray);
        LSet x = new LSet("x");
        Constraint f = x.size(3);
        Ris ris = new Ris(x, map, f);
        ris.output();
        Solver solver = new Solver();
        solver.add(regions.eq(colors));
        solver.add(ris.eq(map));
        solver.showStoreAll();
        assertEquals(true, solver.check());
    }

    //@Test
    public void testExamples12() {
        IntLVar x = new IntLVar("x");
        Constraint f = x.ge(-10);
        LSet d = new LSet(new Interval(-10, 10).toSet());
        IntLVar z = x.mul(x);
        LList p = LList.empty().ins(z).ins(x);
        Ris ris = new Ris(x, d, f, p);

        IntLVar y = new IntLVar("y");
        LList B = LList.empty().ins(25).ins(y);
        Solver solver = new Solver();
        solver.add(ris.contains(B));
        solver.showStoreAll();
        assertEquals(true, solver.check());
        assertEquals((Integer)(-5), y.getValue());
        assertEquals(true, solver.nextSolution());
        assertEquals((Integer)5, y.getValue());
        assertEquals(false,solver.nextSolution());


    }
    //@Test
    public void testRecursiveRis01() throws Failure{
        LSet fact= new LSet();
        fact.setName("fact");
        IntLVar x = new IntLVar("x");
        IntLVar x1 = new IntLVar("x1");
        IntLVar y = new IntLVar("y");
        IntLVar f = new IntLVar("f");

        IntLSet dom = new IntLSet();//1,4);
        dom.setName("dom");
        LList pair = new LPair(x1,f);
        LList pairxy = new LPair(x,y);
        pair.setName("p");
        Constraint constraint = x.gt(0).and(fact.contains(pair)).and(y.eq(f.mul(x))).and(x1.eq(x.sub(1)));
        Ris factr = new Ris(x, dom, constraint, pairxy,x1,y,f);
        factr.setName("ris");
        Solver solver = new Solver();

        solver.add(fact.eq(factr.ins(new LPair(0, 1))));

        IntLVar z = new IntLVar("z");
        solver.add(fact.contains(new LPair(5,z)));
        assertTrue(solver.check());
        assertEquals(120, (int) z.getValue());

        IntLVar z2 = new IntLVar("z2");
        solver.add(fact.contains(new LPair(z2, 479001600)));
        assertTrue(solver.check());
        assertEquals(12, (int) z2.getValue());
        solver.add(fact.contains(new LPair(new IntLVar(), 1000))); //NO SOLUTION
        assertFalse(solver.check());
    }

    void translate(Ris thesaurus, LVar italian, LVar english) throws Failure{
        thesaurus.contains(new LPair(italian, english)).solve();
    }

    void translate(Ris thesaurus, String italian, String english) throws Failure{
        LVar italianvar = new LVar("italian", italian);
        LVar englishvar = new LVar("english", english);
        translate(thesaurus, italianvar, englishvar);

    }

    private void makeDictionary(Ris ris) throws Failure{
        translate(ris, "giardino", "garden");
        translate(ris, "fiore", "flower");
        translate(ris, "scuola", "school");
        translate(ris, "ciao", "hi");
        translate(ris, "enorme", "enormous");
        translate(ris, "enorme", "huge");

    }
    //@Test
    public void testThesaurus() throws Failure{
        LSet synonyms = new LSet("synonyms");
        LSet italian_words = new LSet("italian_words");
        LSet english_words = new LSet("english_words");
        LVar italian = new LVar("italian");
        LVar english = new LVar("english");
        Constraint constraint = italian.in(italian_words).and(english.in(english_words));
        Ris ris = new Ris(new LPair(italian, english), synonyms, constraint);
        makeDictionary(ris);

        LVar ciao = new LVar("ciao", "ciao");
        LVar en_ciao = new LVar("en_ciao");
        translate(ris,ciao,en_ciao);
        assertEquals("hi", en_ciao.getValue());

        LVar scuola_en = new LVar();
        translate(ris, new LVar("", "scuola"), scuola_en);
        assertEquals("school", scuola_en.toString());

        LVar newv = new LVar("", "new");
        LVar nuovo = new LVar("", "nuovo");
        translate(ris, "nuovo", "new");

        LVar it = new LVar();
        translate(ris, it, newv);
        assertEquals("nuovo", it.getValue());
        ConstraintStringComparator c1 = new ConstraintStringComparator("{giardino,fiore,scuola,ciao,enorme,nuovo/_N20915}", italian_words.toString());
        assertTrue(c1.areEqualInSameOrder());
        ConstraintStringComparator c2 = new ConstraintStringComparator("{garden,flower,school,hi,enormous,huge,new/_N20919}", english_words.toString());
        assertTrue(c2.areEqualInSameOrder());

    }

    private String allDivisors(Ris divisors, int n) {
        String res = "";
        for(int i = 2; i < n; ++i)
            res = divisors.contains(new LPair(n,i)).check()? res + " " + i: res;

        return res;
    }

    //@Test
    public void testDivisors() {
        IntLVar integer = new IntLVar("int");
        IntLSet dom1 = new IntLSet("dom");
        IntLVar divisor = new IntLVar("I");
        Ris divisors = new Ris(new LPair(integer, divisor), dom1, integer.mod(divisor).eq(0).and(divisor.gt(1)).and(divisor.lt(integer)).and(integer.label()).and(divisor.label()));

        assertEquals(true, divisors.contains(new LPair(12, 3)).check());
        assertEquals(false, divisors.contains(new LPair(12, 5)).check());
        assertEquals(true, divisors.contains(new LPair(9, 3)).check());
        assertFalse(divisors.contains(new LPair(5,9)).check());


        assertEquals(" 3", allDivisors(divisors, 9));
        assertEquals(" 2 4 5 10", allDivisors(divisors, 20));
        assertEquals(" 3 7", allDivisors(divisors, 21));
    }

    Integer apply(Ris function, Integer i) throws Failure {

        IntLVar x = new IntLVar(i);
        IntLVar y = new IntLVar();
        function.contains(new LPair(x, y)).solve();

        return y.getValue();
    }

    Integer[] apply(Ris function, Integer[] arr) throws Failure {

        Integer[] images = new Integer[arr.length];
        for(int i = 0; i < arr.length; ++i) {
            images[i] = apply(function, arr[i]);
        }

        return images;

    }
    //@Test
    public void testApply01() throws Failure {

        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        IntLSet dom = new IntLSet("dom");
        //{(x, x + x * x)}
        Ris ris = new Ris(new LPair(x, y), dom, y.eq(x.sum(x.mul(x))));
        //System.out.println(ris.F(new LPair(new LVar("a"), new LVar("b"))));
        Integer[] arguments = {1,2,3,4,7,-1,0};
        Integer[] results = apply(ris, arguments);

        assertEquals("[2, 6, 12, 20, 56, 0, 0]", Arrays.asList(results).toString());

        assertEquals(new Integer(2), apply(ris, -2));
        assertEquals(new Integer(30), apply(ris, 5));
        assertEquals(new Integer(999000), apply(ris, -1000));


    }

    public boolean isFunctionDefinedFor(Ris function, Integer arg) {
        return function.contains(new LPair(arg, new IntLVar())).check();
    }

    Integer[] filter(Ris filter, Integer[] arr) {
        List<Integer> filtered = new LinkedList<Integer>();
        for(Integer e: arr) {
            if(isFunctionDefinedFor(filter, e))
                filtered.add(e);
        }
        return filtered.toArray(new Integer[filtered.size()]);
    }



    //@Test
    public void testFilter01()  {

        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        IntLSet dom = new IntLSet("dom");
        //{(x*x, x)}
        Ris ris = new Ris(new LPair(y,x), dom, y.eq(x.mul(x)));
        Integer[] arguments = {1, -3, 0, 5, 4, 9, 100, 10000};
        Integer[] filtered = filter(ris, arguments);
        assertEquals("[1, 0, 4, 9, 100, 10000]", Arrays.asList(filtered).toString());


        assertEquals(true, isFunctionDefinedFor(ris, 16));
        assertEquals(false, isFunctionDefinedFor(ris, 99));

    }

    Integer[] filterAndApply(Ris function, Integer[] arr) throws Failure{

        return filterAndApply(function, function, arr);

    }

    Integer[] filterAndApply(Ris function, Ris filter, Integer[] arr) throws Failure{

        return apply(function, filter(filter, arr));

    }

    //@Test
    public void testFilterAndApply() throws Failure {
        IntLVar x = new IntLVar();
        IntLVar y = new IntLVar();
        IntLSet domfilter = new IntLSet();
        IntLSet domfunction = new IntLSet();
        Ris filter = new Ris(new LPair(x, x), domfilter, x.mod(2).eq(0));
        Ris function = new Ris(new LPair(x, y), domfunction, y.eq(x.div(2)));
        //System.out.println("new control term " + function.getNewControlTerm());
        Integer[] arr = {1, -2, 0, 4, 1000, 7, 9, 888, -9};
        Integer[] results = filterAndApply(function, filter, arr);

        assertEquals("[-1, 0, 2, 500, 444]", Arrays.asList(results).toString());

    }

    //@Test
    public void testSubset01() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,5);
        Ris r1 = new Ris(x, s, x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(0));
        r2.subset(r1).solve();
    }
    //@Test
    public void testSubset01bis() throws Failure{
        System.out.println(new LRel().dom(new LSet()));
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,5);
        Ris r1 = new Ris(x, s, x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(0));
        r1.union(r2,r1).solve();
    }

    //@Test
    public void testSubset02() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(0,5);
        Ris r1 = new Ris(x, s, x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(1));
        assertEquals(false, r2.subset(r1).check());
    }

    //@Test
    public void testSubset03() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet();
        Ris r1 = new Ris(x, s, x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(0));
        r2.subset(r1).solve();
    }

    //@Test
    public void testSubset04() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet();
        Ris r1 = new Ris(x, s, x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(1));
        assertEquals(true, r2.subset(r1).check());
    }

    //@Test
    public void testSubset05() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(-4,4);
        Ris r1 = new Ris(x, s, x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(1));
        Solver solver = new Solver();
        solver.add(s.neq(IntLSet.empty()));
        solver.add(r2.subset(r1));
        solver.add(r1.neq(IntLSet.empty()));
        solver.add(r2.neq(IntLSet.empty()));
        assertEquals(false, solver.check());
    }

    //@Test
    public void testSubset06() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet();
        Ris r1 = new Ris(x, s, x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(1));
        Solver solver = new Solver();
        solver.add(s.neq(IntLSet.empty()));
        solver.add(r2.subset(r1));
        solver.add(r1.neq(IntLSet.empty()));
        solver.add(r2.neq(IntLSet.empty()));
        //assertEquals(false, solver.check());
    }

    //@Test
    public void testSubset07() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(-10,10);
        Ris r1 = new Ris(x, new IntLSet(), x.mod(2).eq(0));
        Ris r2 = new Ris(x, s, x.mod(4).eq(1));
        Solver solver = new Solver();

        solver.add(r2.subset(r1));

        assertEquals(false, solver.check());
    }

    //@Test
    public void testSubset08() throws Failure{
        IntLVar x = new IntLVar("x");
        IntLSet s = new IntLSet(-10,10);
        Ris r1 = new Ris(x, new IntLSet(), x.gt(-20).and(x.lt(20).and(x.mod(2).eq(0))));
        Ris r2 = new Ris(x, s, x.mod(4).eq(0));
        Solver solver = new Solver();

        solver.add(r2.subset(r1));

        assertEquals(true, solver.check());
    }

    //@Test
    public void testNeqRemoval01() {
        LVar x = new LVar();
        LSet dom = new LSet();
        Ris ris = new Ris(x, dom, x.eq(x));

        Solver solver = new Solver();


        solver.add(ris.eq(LSet.empty()));
        solver.add(dom.neq(LSet.empty()));
        assertEquals(false, solver.check());


    }

    //@Test
    public void testNeqRemoval02() {
        IntLVar x = new IntLVar();
        IntLSet dom = new IntLSet();
        Ris ris = new Ris(x, dom, x.eq(x));
        IntLSet A = new IntLSet();
        Solver solver = new Solver();
        solver.add(ris.eq(LSet.empty()));
        solver.add(dom.eq(A));
        solver.add(A.neq(LSet.empty()));

        assertEquals(false, solver.check());

    }

    //@Test
    public void testNeqRemoval03() {
        IntLVar x = new IntLVar();
        IntLSet dom = new IntLSet();
        Ris ris = new Ris(x, dom, Constraint.truec());
        Solver solver = new Solver();
        solver.add(dom.neq(IntLSet.empty()));
        solver.add(ris.union(ris, ris));
        solver.add(ris.neq(IntLSet.empty()));

        assertEquals(true, solver.check());
    }

    //@Test
    public void testNeqRemoval04() {
        IntLVar x = new IntLVar();
        IntLSet dom = new IntLSet();
        Ris ris = new Ris(x, dom, Constraint.truec());
        Solver solver = new Solver();
        solver.add(dom.neq(IntLSet.empty()));
        solver.add(ris.union(ris, ris));
        solver.add(ris.eq(IntLSet.empty()));

        assertEquals(false, solver.check());
    }

    //@Test
    public void testNeqRemoval04bis() {
        IntLVar x = new IntLVar();
        IntLSet dom = new IntLSet();
        Ris ris = new Ris(x, dom, x.lt(1000));
        Solver solver = new Solver();
        solver.add(dom.neq(IntLSet.empty()));
        solver.add(ris.union(ris, ris));
        solver.add(ris.eq(IntLSet.empty()));

        assertEquals(true, solver.check());
    }

    //@Test
    public void testNeqRemoval05() {
        IntLVar x = new IntLVar();
        IntLSet dom = new IntLSet();
        Ris ris = new Ris(x, dom, Constraint.truec());
        Solver solver = new Solver();
        solver.add(dom.eq(IntLSet.empty()));
        solver.add(ris.union(ris, ris));
        solver.add(ris.neq(IntLSet.empty()));

        assertEquals(false, solver.check());
    }

    //@Test
    public void testNeqRemovalOR01() {
        IntLVar x = new IntLVar();
        IntLSet dom = new IntLSet();
        Ris ris = new Ris(x, dom, x.lt(1000));
        Solver solver = new Solver();
        solver.add(dom.neq(IntLSet.empty()).or(dom.neq(IntLSet.empty())));
        solver.add(ris.union(ris, ris).or(ris.union(ris, ris)));
        solver.add(ris.eq(IntLSet.empty()).or(ris.eq(IntLSet.empty())));

        assertEquals(true, solver.check());
    }

    //@Test
    public void testNeqRemovalOR02() {
        IntLVar x = new IntLVar();
        IntLSet dom = new IntLSet();
        Ris ris = new Ris(x, dom, Constraint.truec());
        Solver solver = new Solver();
        solver.add(dom.eq(IntLSet.empty()).or(dom.eq(IntLSet.empty())));
        solver.add(ris.union(ris, ris).or(ris.union(ris, ris)));
        solver.add(ris.neq(IntLSet.empty()).or(ris.neq(IntLSet.empty())));

        assertEquals(false, solver.check());
    }

    public void testExpand() throws Failure, IOException {

        Solver solver = new Solver();

        // s1 = ris(X in {2,3,4},true,X).
        LSet d1 = LSet.empty().ins(2).ins(3).ins(4);
        IntLVar x1 = new IntLVar("x1");
        Ris r1 = new Ris(x1, d1, Constraint.truec(), x1);

        LSet s1 = new LSet("s1");
        solver.solve(s1.eq(r1));
        assertEquals("{4,3,2}", s1.toString());
        assertEquals("{4,3,2}", r1.expand().toString());

        // s2 = ris(X in {2,Z,4},true,X).
        solver.clearStore();
        IntLVar z = new IntLVar("z");
        LSet d2 = LSet.empty().ins(2).ins(z).ins(4);
        IntLVar x2 = new IntLVar("x2");
        Ris r2 = new Ris(x2, d2, Constraint.truec(), x2);
        LSet s2 = new LSet("s2");

        solver.solve(s2.eq(r2));
        assertEquals("{4,2/{ _x2 : {_z} | true @ _x2 }}", s2.toString());
        assertEquals("{4,2/{ _x2 : {_z} | true @ _x2 }}", r2.expand().toString());

    }

    public static LVar minValue(LSet S) throws Failure{
        Solver solver = new Solver();
        IntLVar x = new IntLVar(), y = new IntLVar();
        Ris ris = new Ris(x,S,y.le(x));
        solver.add(y.in(S).and(S.subset(ris)));
        solver.solve();
        return y;
    }

    public void test03A()throws Failure {
        Integer[] sampleSetElems = {8, 2};
        LSet R = new LSet().insAll(sampleSetElems).setName("R");
        LVar min = minValue(R).setName("min");
        min.output();
    }
    public void test03B()throws Failure {
        Integer[] sampleSetElems = {8, 2};
        LSet R = new LSet().insAll(sampleSetElems).ins(new IntLVar()).setName("R");
        LVar min = minValue(R).setName("min");
        min.output();
    }

    public void testMinMaxTheorem(){
        IntLSet dom = new IntLSet();
        int n = 1000000;
        IntLVar min = new IntLVar("min",-n,n);
        IntLVar max = new IntLVar("max",-n,n);
        IntLVar x = new IntLVar("x");
        IntLVar y = new IntLVar("y");
        Ris minRis = new Ris(x,dom, x.ge(min));
        Ris maxRis = new Ris(x,dom,x.le(max));
        Solver solver = new Solver();
        solver.add(      min.in(dom)
                        .and(max.in(dom))
                        .and(dom.subset(minRis))
                        .and(dom.subset(maxRis))
                        .and(min.gt(max)));
        assertFalse(solver.check());
    }

    public void testReachableNodes(){
        int k = 15;
        IntLSet N = new IntLSet(1,k);
        LVar n = new LVar(1);
        LSet E = LSet.empty();
        for(int i = 1; i < k; ++i)
            E = E.ins(new LPair(i, i+1));

        LVar x = new LVar("x");
        LVar y = new LVar("y");
        LSet R = new LSet();
        Ris R_R = new Ris(x,N,x.eq(n)
                .or(y.in(R).and(new LPair(y,x).in(E))),x,y);
        R.eq(R_R).check();
        assertEquals(R_R.expand(),N);
    }

    private Constraint grep(LVar inv, LSet file, LVar word, LSet result){
        LSet line1 = new LSet("line1");
        LSet line2 = new LSet("line2");
        return (inv.eq("").and(result.eq(new Ris(line1, file, word.in(line1)))))
                .or(inv.eq("v").and(result.eq(new Ris(line2, file, word.nin(line2)))));
    }

    public void testPropertyC5(){
        LSet F = new LSet("F");
        LVar W = new LVar("W");
        LSet R1 = new LSet("R1"), R2 = new LSet("R2");
        assertFalse(grep(new LVar("opt1", ""),F,W,R1).and(grep(new LVar("opt2","v"), F,W,R2)).and(R1.nunion(R2,F)).check());
    }

}
