package jsetl.test;

import jsetl.*;

import org.junit.Test;
import junit.framework.TestCase;

public class SetLVarTest extends TestCase {

	@Test
	public void
	testConstructors() {
		SetLVar X = new SetLVar(SetInterval.universe(), 0);
		X.setName("X");
		SetLVar Y = new SetLVar(X);
		SetLVar Z = new SetLVar("Z", X);
		assertEquals(X, Y);
		assertEquals(Y, Z);
		assertEquals((int) Z.card().getDomain().getGlb(), 0);
		assertEquals(Y.clone().card(), X.card());
	}
	
	@Test
	public void
	testInNinEqConstraints() {
		Solver solver = new Solver();
		SetLVar S = new SetLVar("S");
		SetLVar S1 = new SetLVar("S1");
		MultiInterval lb = new MultiInterval();
		MultiInterval ub = new MultiInterval(1, 4);
		SetInterval s = new SetInterval(lb, ub);
		solver.add(S.dom(s));
		ub.remove(3);
		SetInterval t = new SetInterval(new MultiInterval(2), ub);
		solver.add(S.dom(t));
		assertTrue(solver.check());
		assertTrue(S.getDomain().equals(t));
		
		IntLVar X = new IntLVar("X");
		solver.add(X.in(S));
		IntLVar X1 = new IntLVar(5);
		solver.add(X1.in(S1));
		solver.add(S.neq(S1).and(S.card().eq(S1.card())));
		assertTrue(solver.check());
		assertEquals((int) S1.getDomain().getGlb().getLub(), 5);
		assertTrue(S1.card().getDomain().equals(new MultiInterval(1, 3)));
		
		solver.add(X.eq(2));
		solver.add(X.nin(S1));
		solver.add(X1.nin(S));	
		IntLVar X2 = new IntLVar("X2");
		solver.add(X2.in(new MultiInterval(1, 3)));
		solver.add(X2.in(ub));
		IntLVar X3 = new IntLVar("X3");
		solver.add(X3.nin(ub));
		solver.add(X3.nin(new MultiInterval(-1)));
		solver.add(S1.eq(new MultiInterval(3, 5)));
		assertTrue(solver.check());
		assertEquals(S1.getValue(), new MultiInterval(3, 5));
		assertEquals(X2.getDomain(), new MultiInterval(1, 2));
	    assertFalse(X3.getDomain().contains(1));
	    
	    IntLVar X4 = new IntLVar("X4", 0, 1);
	    SetLVar S2 = new SetLVar("S2", new MultiInterval(0), 
	    		                   new MultiInterval(0, 2));
	    solver.add(X4.nin(S2));
	    solver.add(S2.card().eq(2));
	    assertTrue(solver.check());
	    assertEquals((int) X4.getValue(), 1);
	    assertEquals(S2.toString(), "{0, 2}");
	}
	
	@Test
    public void
    testNeqConstraint() {
    	Solver solver = new Solver();
        MultiInterval lb = new MultiInterval();
        MultiInterval ub = new MultiInterval(1);
        SetLVar X = new SetLVar("X", lb, ub);
        SetLVar Y = new SetLVar("Y", lb, ub);
        SetLVar Z = new SetLVar("Z", lb, ub);
        solver.add(X.neq(Y));
        solver.add(Y.neq(Z));
        solver.add(Z.neq(X));
        assertTrue(solver.check());
        solver.add(X.label());
        solver.add(Y.label());
        solver.add(Z.label());
        assertFalse(solver.check());
	}
	
	@Test
	public void
	testComplDisjConstraints() {
		MultiInterval lb = new MultiInterval(0);
		MultiInterval ub = new MultiInterval(-2, 2);
		SetLVar S  = new SetLVar("S", lb, ub);
		SetLVar S1 = new SetLVar("S1");
		SetLVar S2 = new SetLVar("S2");
		Solver solver = new Solver();
		solver.add(S.eq(S1));
		solver.add(S2.eq(S1.compl()));
		assertTrue(solver.check());
		assertFalse(S2.getDomain().getLub().contains(0));
		
		IntLVar X  = new IntLVar("X", 2);
		IntLVar X1 = new IntLVar("X1", 1);
		solver.add(X.in(S));
		solver.add(X1.nin(S1));
		assertTrue(solver.check());
		assertEquals(S, S1);
		assertTrue(S1.card().getDomain().equals(new MultiInterval(2, 4)));
		
		MultiInterval mi = new MultiInterval(0, 2);
		mi.remove(1);
		solver.add(S.eq(mi));
		solver.add(S2.neq(S1));
		solver.add(S2.eq(S.compl()));
		assertTrue(solver.check());
		assertTrue(S2.getValue().contains(1));
		assertFalse(S.getValue().contains(1));
		assertEquals((int) S1.card().getValue(), 2);
		assertEquals((int) S2.card().getValue(), Interval.SUP - 2);
		
		mi = MultiInterval.universe();
		mi.remove(0);
		mi.remove(2);
		solver.add(S.eq(mi.complement()));
		SetLVar S3 = new SetLVar("S3");
		SetLVar S4 = new SetLVar("S4");
		SetLVar S5 = new SetLVar("S5");
		mi = new MultiInterval(-1, 1);
		solver.add(S3.eq(mi.complement(SetInterval.SUP)));
		solver.add(S4.disj(S3));
		solver.add((S4.card().sum(S3.card()))
					.le(Interval.SUP));
		solver.add(S4.disj(S));
		solver.add(S4.disj(S2));
		solver.add(S5.disj(mi));
		solver.add(S5.disj(new MultiInterval(Interval.INF, 0)));
		solver.add(S5.disj(new MultiInterval(100, Interval.SUP)));
		assertTrue(solver.check());
		assertFalse(S3.getValue().contains(new Interval(-1, 1)));
		assertEquals((int) S3.card().getValue(), 1073741820);
		assertTrue(S4.getValue().isEmpty());
		assertEquals(S5.card().getDomain(), new MultiInterval(0, 98));
	}
	
	@Test
	public void
	testIntersectConstraint() {
		MultiInterval lb = new MultiInterval(5);
		MultiInterval ub = new MultiInterval(1, 10);
		SetLVar S  = new SetLVar("S");
		SetLVar S1 = new SetLVar("S1", lb, ub);
		SetLVar S2 = new SetLVar("S2");
		IntLVar N = new IntLVar("N");
		Solver solver = new Solver();
		solver.add(S.eq(S1.intersect(S2)));
		solver.add(new IntLVar(7).in(S1));
		solver.add(new IntLVar(7).in(S2));
		solver.add(new IntLVar(0).nin(S2));
		solver.add(S.card().eq(N));
		solver.add(N.ge(S1.card()));
		solver.add(N.gt(3));
		lb = new MultiInterval(4, 6);
		ub = lb.union(new MultiInterval(1, 2)).union(new MultiInterval(7));
		SetLVar set1 = new SetLVar(lb, ub);
		MultiInterval lb1 = new MultiInterval(6, 8);
		MultiInterval ub1 = lb1.union(new MultiInterval(2, 5));
		ub1.remove(4);
		SetLVar set2 = new SetLVar(lb1, ub1);
		solver.add(S.eq(set1.intersect(set2)));
		solver.add(S2.eq(S2.intersect(new MultiInterval(-100, 100))));
		solver.add(S2.card().eq(200));
		assertTrue(solver.check());
		lb1 = new MultiInterval(2).union(new MultiInterval(5, 7));
		ub1 = new MultiInterval(-100, 100);
		ub1.remove(0);
		assertEquals(S, S1);
		assertEquals(S1.getValue(), lb1);
		assertEquals(S2.getValue(), ub1);
		assertEquals((int) N.getValue(), 4);
	}
	
	@Test
	public void
	testUnionConstraint() {
		MultiInterval lb = new MultiInterval(5);
		MultiInterval ub = new MultiInterval(1, 10);
		SetLVar S  = new SetLVar("S");
		SetLVar S1 = new SetLVar("S1", lb, ub);
		lb = new MultiInterval(-5, 0);
		ub = new MultiInterval(-10, 8);
		SetLVar S2 = new SetLVar("S2", lb, ub);
		Solver solver = new Solver();
		solver.add(S.eq(S1.union(S2)));
		assertTrue(solver.check());
		assertTrue(S.card().getDomain().equals(new MultiInterval(7, 21)));
		assertEquals((int) S1.getDomain().getGlb().getLub(), 5);
		assertEquals(S2.card().getDomain(), new MultiInterval(6, 19));

		IntLVar N = new IntLVar("N");
		solver.add(new IntLVar(6).nin(S1));
		solver.add(new IntLVar(6).nin(S2));
		solver.add(new IntLVar(4).in(S));
		solver.add(S2.card().eq(N));
		solver.add(N.eq(18));
		solver.add(S1.card().eq(1));
		solver.add(S.eq(new MultiInterval(-10, 5)
		         .union(new MultiInterval(7, 8))));
		assertTrue(solver.check());
		assertEquals(S, S2);
		assertEquals(S1.getValue().getLub(), S1.getValue().getGlb());
	}
	
	@Test
	public void
	testDiffConstraint1() {
		SetLVar R  = new SetLVar("R");
		MultiInterval lb = new MultiInterval();
		MultiInterval ub = new MultiInterval(1, 20);
		SetLVar R1 = new SetLVar("R1", new SetInterval(lb, ub), 10);
		ub = new MultiInterval(20, 30);
		SetLVar R2 = new SetLVar("R2", lb, ub);
		Solver solver = new Solver();
		solver.add(R.eq(R1.diff(R2)));
		assertTrue(solver.check());
		assertEquals((int) R1.card().getDomain().size(), 1); 
		assertEquals(R2.card().getDomain(), new MultiInterval(0, 11));
		assertEquals(R.card().getDomain(), new MultiInterval(9, 10));
	}
		
	@Test
	public void
	testDiffConstraint2() {
		SetLVar T  = new SetLVar("T");
		MultiInterval lb = new MultiInterval(1, 5);
		MultiInterval ub = new MultiInterval(1, 20);
		SetLVar T1 = new SetLVar("T1", new SetInterval(lb, ub), 6);
		SetLVar T2 = new SetLVar("T2", new SetInterval(lb, ub), 6);
		Solver solver = new Solver();
		solver.add(T.eq(T1.diff(T2)));
		assertTrue(solver.check());
		assertEquals((int) T1.card().getDomain().size(), 1); 
		assertEquals(T1.card().getDomain(), T2.card().getDomain());
		assertEquals(T.card().getDomain(), new MultiInterval(0, 1));
	}
	
	@Test
	public void
	testIntersSpecialSizeConstraint() {
		MultiInterval lb = new MultiInterval();
		MultiInterval ub = new MultiInterval(0 , 1);
		Solver solver = new Solver();
		MultiInterval card = new MultiInterval(0)
		              .union(new MultiInterval(2));
		SetLVar I1 = new SetLVar("I1", new SetInterval(lb, ub), card);
		SetLVar I2 = new SetLVar("I2", new SetInterval(lb, ub), card);
		SetLVar I3 = new SetLVar("I3");
		solver.add(I3.eq(I1.intersect(I2)));
		assertTrue(solver.check());
		assertEquals(I1.card().getDomain(), card);
		assertEquals(I2.card().getDomain(), card);
		assertEquals(I3.card().getDomain(), card);
	}
	
	@Test
	public void
	testUnionSpecialSizeConstraint() {		
		Solver solver = new Solver();
		MultiInterval lb = new MultiInterval();
		MultiInterval ub = new MultiInterval(0 , 3);
		MultiInterval card = new MultiInterval(0)
		              .union(new MultiInterval(4));
		SetLVar U1 = new SetLVar("U1", new SetInterval(lb, ub), card);
		SetLVar U2 = new SetLVar("U2", new SetInterval(lb, ub), card);
		SetLVar U3 = new SetLVar("U3");
		solver.add(U3.eq(U1.union(U2)));
		assertTrue(solver.check());
		assertEquals(U1.card().getDomain(), card);
		assertEquals(U2.card().getDomain(), card);
		assertEquals(U3.card().getDomain(), card);
	}
	
	@Test
	public void
	testDiffSpecialSizeConstraint() {	
		Solver solver = new Solver();
		MultiInterval lb = new MultiInterval();
		MultiInterval ub = new MultiInterval(0 , 3);
		MultiInterval card = new MultiInterval(0)
        			  .union(new MultiInterval(4));
		SetLVar D1 = new SetLVar("D1", new SetInterval(lb, ub), card);
		SetLVar D2 = new SetLVar("D2", new SetInterval(lb, ub), card);
		lb = new MultiInterval(0).union(new MultiInterval(4));
		SetLVar D3 = new SetLVar("D3");
		solver.add(D3.eq(D1.diff(D2)));
		assertTrue(solver.check());
		assertEquals(D1.card().getDomain(), lb);
		assertEquals(D2.card().getDomain(), lb);
		assertEquals(D3.card().getDomain(), lb);
	}
	
	@Test
	public void
	testSubsetAndOtherConstraints() {
		MultiInterval lb = new MultiInterval(0);
		MultiInterval ub = new MultiInterval(-2, 2);
		SetLVar S  = new SetLVar("S", lb, ub);
		SetLVar S1 = new SetLVar("S1");
		SetLVar S2 = new SetLVar("S2");
		Solver solver = new Solver();
		solver.add(S.subset(S1));
		solver.add(S2.subset(S));
		assertTrue(solver.check());
		assertFalse(S.card().getDomain().contains(0));
		assertEquals(S1.getDomain().getLub(), SetInterval.SUP);
		assertTrue(S2.card().getDomain().contains(0));
		
		lb = new MultiInterval(-1, 1);
		solver.add(S.subset(lb));
		solver.add(new SetLVar(lb).subset(S));
		solver.add(S.card().eq(new IntLVar(-4, 4)));
		IntLVar N = new IntLVar("N");
		solver.add(S1.card().eq(N));
		solver.add(N.dom(-1, 3));
		solver.add(S2.card().eq(new IntLVar(-5, 3)));
		solver.add(S.card().eq(3));
		solver.add(S2.eq(S.intersect(S1)));
		solver.add(S2.eq(S.union(S1)));
		SetLVar S3 = new SetLVar("S3", new MultiInterval(0, 5));
		SetLVar S4 = new SetLVar("S4");
		solver.add(S4.eq(S3.diff(S)));
		assertTrue(solver.check());
		assertEquals(S2, S);
		assertEquals(S1.getValue(), lb);
		assertTrue(S3.getValue().contains(new Interval(2, 5)));
		assertFalse(S3.getValue().subset(S4.getValue()));
		
		lb = new MultiInterval();
		ub = new MultiInterval(1, 3);
		SetLVar S5 = new SetLVar("S5", lb, ub);
		LabelingOptions lop = new LabelingOptions();
		lop.valueForSetLVar = SetHeuristic.FIRST_IN;
		solver.add(S5.label(lop));
		IntLVar M = new IntLVar("M", 1, 2);
		solver.add(S5.card().eq(M));
		assertTrue(solver.check());
        int c = 0;
        do {
        	int card = S5.card().getValue();
            assertTrue(0 < card && card < 3);
            ++c;
        } while (solver.nextSolution());
        assertEquals(c, 6);
	}
	
	@Test
	public void
	testSetColoring() {
		SetLVar Cols = new SetLVar("Cols");
		Solver solver = new Solver();
		
		MultiInterval empty = new MultiInterval();
		solver.add(Cols.intersect(new MultiInterval(1)
		                   .union(new MultiInterval(4))).neq(empty));
		solver.add(Cols.intersect(new MultiInterval(1, 2)).neq(empty));
		solver.add(Cols.intersect(new MultiInterval(3, 4)).neq(empty));
		solver.add(Cols.intersect(new MultiInterval(1)
		   				   .union(new MultiInterval(3))).neq(empty));
		solver.add(Cols.intersect(new MultiInterval(2)
		                   .union(new MultiInterval(4))).neq(empty));
		
		solver.add(Cols.strictSubset(new MultiInterval(1, 4)));
		solver.add(Cols.card().eq(2));
		solver.add(Cols.label());
	}
	
	@Test
	public void
	testStoreSimplification() {
		Solver solver = new Solver();
		SetLVar x = new SetLVar("x");
		SetLVar y = new SetLVar("y");
		SetLVar z = new SetLVar("z");
		solver.add(z.eq(x.union(y).diff(x).intersect(y)));
		int before = solver.size();
		assertTrue(solver.check());
		int after = solver.size();
		assertTrue(before > after);
	}
}
